package com.ibm.gbs.va.grit;

import android.app.Activity;
import android.arch.lifecycle.Lifecycle;
import android.arch.lifecycle.ProcessLifecycleOwner;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.v7.app.AppCompatActivity;

import com.ibm.gbs.va.grit.dagger.AppComponent;
import com.ibm.gbs.va.grit.dagger.DaggerAppComponent;
import com.ibm.gbs.va.grit.dagger.UserComponent;
import com.ibm.gbs.va.grit.model.type.Badge;
import com.ibm.gbs.va.grit.rest.MessageHandler;
import com.ibm.gbs.va.grit.rest.NetworkErrorDelegate;
import com.ibm.gbs.va.grit.rest.NetworkErrorListener;
import com.ibm.gbs.va.grit.util.BadgeDelegate;
import com.ibm.gbs.va.grit.util.TypefaceUtil;

import net.danlew.android.joda.JodaTimeAndroid;

import java.util.ArrayDeque;
import java.util.List;
import java.util.ListIterator;
import java.util.Queue;

import javax.inject.Inject;

import dagger.android.AndroidInjector;
import dagger.android.support.DaggerApplication;

public class GritApplication extends DaggerApplication implements NetworkErrorDelegate, BadgeDelegate {

    private AppComponent curApplicationProvider;

    @Override
    protected AndroidInjector<? extends DaggerApplication> applicationInjector() {
        curApplicationProvider = DaggerAppComponent.builder().application(this).build();
        return curApplicationProvider;
    }

    @Inject
    public UserComponent.Builder userScopeBlueprint;
    private UserComponent currentUserScope;

    public UserComponent getCurrentUserScope() {
        return currentUserScope;
    }

    public void resetCurrentUserScope() {
        // since we won't ever need to go back to a previous scope/have multiples at once
        // we just dereference the old one and make a new one
        currentUserScope = userScopeBlueprint.build();
    }

    public void purgeUserData(String username) {
        curApplicationProvider.getGritDB().purgeUser(username);
        curApplicationProvider.getKeystore().deleteKeystoreAliasEntry(R.string.USERPREF_ENCRYPTED_PASSWORD_KEY_ALIAS);
    }

    private AppCompatActivity currentActivity = null;

    @Override
    public void onCreate() {
        super.onCreate();
        JodaTimeAndroid.init(this);
        registerActivityLifecycleCallbacks(new ActivityLifecycleCallbacks() {
            @Override
            public void onActivityCreated(Activity activity, Bundle bundle) {

            }

            @Override
            public void onActivityStarted(Activity activity) {

            }

            @Override
            public void onActivityResumed(Activity activity) {
                if (activity instanceof AppCompatActivity) {
                    currentActivity = (AppCompatActivity) activity;
                    while (pausedErrorQueue.peek() != null) {
                        handleError(pausedErrorQueue.poll());
                    }
                }
            }

            @Override
            public void onActivityPaused(Activity activity) {
                currentActivity = null;
            }

            @Override
            public void onActivityStopped(Activity activity) {

            }

            @Override
            public void onActivitySaveInstanceState(Activity activity, Bundle bundle) {

            }

            @Override
            public void onActivityDestroyed(Activity activity) {

            }
        });

        TypefaceUtil.overrideFont(getApplicationContext(), "sans", "fonts/ibm_plex_snas_regular.ttf");

        //set our initial Userscope
        resetCurrentUserScope();
    }

    private final Queue<String> pausedErrorQueue = new ArrayDeque<>();

    @Override
    public boolean handleError(String message) {
        if (currentActivity == null) {
            //our current activity is paused or otherwise indisposed, we store this error and process it when the app resumes
            pausedErrorQueue.add(message);
            return true;
        }

        if (currentActivity instanceof NetworkErrorListener) {
            List<NetworkErrorDelegate> delegateList = ((NetworkErrorListener) currentActivity).getNetworkErrorListeners();
            ListIterator<NetworkErrorDelegate> i = delegateList.listIterator();
            while (i.hasNext()) {
                NetworkErrorDelegate delegate = i.next();
                // clean up dead delegates
                if (delegate.getLifecycle().getCurrentState() == Lifecycle.State.DESTROYED)
                    i.remove();
                else if (delegate.getLifecycle().getCurrentState().isAtLeast(Lifecycle.State.STARTED)) {
                    if (delegate.handleError(message)) {
                        return true;
                    }
                }
            }
        }

        if (currentActivity instanceof NetworkErrorDelegate) {
            NetworkErrorDelegate delegate = (NetworkErrorDelegate) currentActivity;
            if (delegate.getLifecycle().getCurrentState().isAtLeast(Lifecycle.State.STARTED)) {
                if (delegate.handleError(message)) {
                    return true;
                }
            }
        }

        if (currentActivity != null && currentActivity.hasWindowFocus()) { // this should block displaying multiple popups
            MessageHandler.handleError(currentActivity, message);
        }

        return true;
    }

    @Override
    public boolean handleError(int strResource) {
        handleError(getString(strResource));
        return true;
    }

    @Override
    public boolean handleBadges(@NonNull List<Badge> badgesToHandle) {
        if (currentActivity instanceof BadgeDelegate) {
            BadgeDelegate delegate = (BadgeDelegate) currentActivity;
            if (delegate.getLifecycle().getCurrentState().isAtLeast(Lifecycle.State.STARTED)) {
                return delegate.handleBadges(badgesToHandle);
            }
        }
        //if the current activity can't  handle it, it's better to ignore it to maintain flow
        return false;
    }

    @NonNull
    @Override
    public Lifecycle getLifecycle() {
        return ProcessLifecycleOwner.get().getLifecycle();
    }

    public AppComponent getCurApplicationProvider() {
        return curApplicationProvider;
    }
}
