package com.ibm.gbs.va.grit.activity;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.ViewModelProvider;
import android.arch.lifecycle.ViewModelProviders;
import android.content.Context;
import android.content.Intent;
import android.graphics.Rect;
import android.os.Bundle;
import android.os.Handler;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.view.MotionEvent;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.fragment.dialog.GritDialogFragment;
import com.ibm.gbs.va.grit.fragment.question.QuestionFragment;
import com.ibm.gbs.va.grit.model.dialog.TextSection;
import com.ibm.gbs.va.grit.model.type.Badge;
import com.ibm.gbs.va.grit.rest.NetworkErrorDelegate;
import com.ibm.gbs.va.grit.util.RemoteLog;
import com.ibm.gbs.va.grit.viewmodel.SurveyViewModel;
import com.stfalcon.chatkit.messages.MessageInput;

import javax.inject.Inject;

public class SurveyActivity extends BaseViewPagerActivity implements NetworkErrorDelegate {
    @Inject
    ViewModelProvider.Factory viewModelFactory;

    private SurveyViewModel model;

    private String currentSurveyName;

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        TAG = "SurveyActivity";
        super.onCreate(savedInstanceState);
        model = ViewModelProviders.of(this, viewModelFactory).get(SurveyViewModel.class);
        viewModel.showInfoButton.set(true);

        model.updateCurrentUserQuestions(result -> {
            if (!result) handleError(R.string.initial_q_set_fetch_failure);
        });

        model.getCurrentSurveyName().observe(this, currentSurveyName -> this.currentSurveyName = currentSurveyName);

        // block exiting survey if baseline
        viewModel.disableExiting.set(!model.isInitialBaselineComplete());
    }

    @Override
    protected void onPostCreate(@Nullable Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);
        // show introductory dialog if user is on initial baseline survey
        if (!model.isInitialBaselineComplete() && model.isSurveyInfoComplete()) {
            GritDialogFragment.showSurveyIntroDialog(getSupportFragmentManager(), getResources());
            model.setSurveyInfoComplete(false);
        }
        model.getQuestionsAndAnswers().observe(this, qAndAs -> {
        });
    }

    @Override
    protected void onStart() {
        super.onStart();
        int inProgressPage = model.getInProgressPage();
        if (inProgressPage > 0) {
            new Handler().postDelayed(() -> viewPager.setCurrentItem(inProgressPage),
                    200);
        }
    }

    @Override
    protected void onPause() {
        super.onPause();
        RemoteLog.d(TAG, "Setting in progress page to: " + viewPager.getCurrentItem());
        model.setInProgressPage(viewPager.getCurrentItem());
    }

    // Dismiss keyboard when clicking out of an EditText
    @Override
    public boolean dispatchTouchEvent(MotionEvent event) {
        if (event.getAction() == MotionEvent.ACTION_DOWN) {
            View v = getCurrentFocus();
            if (v instanceof MessageInput || v instanceof EditText) {
                Rect outRect = new Rect();
                v.getGlobalVisibleRect(outRect);
                if (!outRect.contains((int) event.getRawX(), (int) event.getRawY())) {
                    InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                    imm.hideSoftInputFromWindow(v.getWindowToken(), 0);
                }
            }
        }
        return super.dispatchTouchEvent(event);
    }

    @Override
    void onFinishClicked() {
        model.saveAllAnswers();
        clearAdapter();
        // wait until submit survey completes
        model.submitQuestionSet(booleanBadgePair -> {
            if (booleanBadgePair.first == null) return;

            String finishedSurveyName = model.getPreviousSurveyName();
            boolean hasMoreQuestions = booleanBadgePair.first;

            if (viewModel.disableExiting.get()) viewModel.disableExiting.set(false);

            if (hasMoreQuestions)
                evalSubmissionResults(model.getCurrentSurveyName().getValue(), true, null, booleanBadgePair.second);
            else {
                model.getNextPlausibleSurvey(queuedSurveyName ->
                        evalSubmissionResults(finishedSurveyName, false, queuedSurveyName, booleanBadgePair.second));
            }
        });
    }

    private void evalSubmissionResults(String finishedSurveyName, boolean qSetNotComplete, String queuedSurveyName, @Nullable Badge badge) {

        if (queuedSurveyName != null) { // survey is done WITH next available
            evalNextSurveyResults(finishedSurveyName, queuedSurveyName, badge);

        } else if (qSetNotComplete && badge != null) { // show badge & ask to take next section
            GritDialogFragment.showSurveyEndWithNextAndBadgeDialog(
                    getSupportFragmentManager(),
                    getResources(),
                    badge,
                    nextBtn -> takeNextQuestionSet(),
                    saveAndQuitBtn -> saveAndExitSurvey(),
                    TextSection.ofDynamicText(
                            getString(R.string.set_complete_w_badge_and_more_sections, badge.getDefaultTitle()),
                            R.style.Body_Inverse,
                            true));

        } else if (qSetNotComplete) { // ask to take next section
            GritDialogFragment.showSurveyEndWithNextDialog(
                    getSupportFragmentManager(),
                    getString(R.string.dialog_congratulations),
                    nextBtn -> takeNextQuestionSet(),
                    saveAndQuitBtn -> saveAndExitSurvey(),
                    TextSection.ofDynamicText(
                            getString(R.string.set_complete_w_more_sections, currentSurveyName),
                            R.style.Body,
                            true));

        } else if (badge != null) { // Question Set end with Badge
            GritDialogFragment.showSurveyEndWithBadgeDialog(
                    getSupportFragmentManager(),
                    getResources(),
                    badge,
                    closeBtn -> saveAndExitSurvey(),
                    TextSection.ofDynamicText(
                            getString(R.string.survey_complete, finishedSurveyName),
                            R.style.Body_Inverse,
                            true));

        } else { // show completion dialog
            GritDialogFragment.showSurveyEndDialog(
                    getSupportFragmentManager(),
                    finishedSurveyName,
                    closeBtn -> saveAndExitSurvey(),
                    TextSection.ofDynamicText(
                            getString(R.string.survey_complete, finishedSurveyName),
                            R.style.Body,
                            true));
        }
    }

    private void evalNextSurveyResults(String finishedSurveyName, String currentSurveyName, Badge badge) {
        if (badge != null) { // show badge & ask to take next survey
            GritDialogFragment.showSurveyEndWithNextAndBadgeDialog(
                    getSupportFragmentManager(),
                    getResources(),
                    badge,
                    nextBtn -> takeNextSurvey(),
                    saveAndQuitBtn -> saveAndExitSurvey(),
                    TextSection.ofDynamicText(
                            getString(R.string.survey_complete_w_badge_and_next_survey, badge.getDefaultTitle(), currentSurveyName),
                            R.style.Body_Inverse,
                            true));

        } else { // ask to take next survey
            GritDialogFragment.showSurveyEndWithNextDialog(
                    getSupportFragmentManager(),
                    finishedSurveyName,
                    nextBtn -> takeNextSurvey(),
                    saveAndQuitBtn -> saveAndExitSurvey(),
                    TextSection.ofDynamicText(
                            getString(R.string.survey_complete_w_next_survey, finishedSurveyName, currentSurveyName),
                            R.style.Body,
                            true));
        }
    }

    private void takeNextSurvey() {
        model.queueNextPlausibleSurvey(dismissed -> {
            if (dismissed) model.updateCurrentUserQuestions(then -> resetAdapter());
        });
    }

    private void takeNextQuestionSet() {
        model.updateCurrentUserQuestions(then -> resetAdapter());
    }

    @Override
    void onCloseClicked() {
        model.saveAllAnswers();
        GritDialogFragment.showSaveAndExitSurveyDialog(getSupportFragmentManager(), v -> saveAndExitSurvey());
    }

    @Override
    void onInfoClicked() {
        GritDialogFragment.showSingleBtnInfoDialog(getSupportFragmentManager(), R.string.dialog_survey_info_title,
                TextSection.ofText(R.string.dialog_survey_info_body_1, R.style.Caption2, false),
                TextSection.ofBullet(R.string.dialog_survey_info_body_2, R.style.Caption, false),
                TextSection.ofBullet(R.string.dialog_survey_info_body_3, R.style.Caption, false),
                TextSection.ofText(R.string.dialog_survey_info_body_4, R.style.Caption2, false),
                TextSection.ofBullet(R.string.dialog_survey_info_body_5, R.style.Caption, false));
    }

    @Override
    LiveData<Integer> getTotalPages() {
        return model.getSize();
    }

    @Override
    Fragment getFragment(int index) {
        return QuestionFragment.newInstance(index);
    }

    private void saveAndExitSurvey() {
        Intent intent = new Intent(this, DashboardActivity.class);
        startActivity(intent);
        finish();
    }

    @Override
    public boolean handleError(String message) {
        Intent intent = new Intent(this, DashboardActivity.class);
        if (!message.equals(getString(R.string.initial_q_set_fetch_failure)))
            intent.putExtra(DashboardActivity.SURVEY_SUBMIT_ERROR_OCCURRED_ARG, message);
        startActivity(intent);
        finish();
        return true;
    }

    @Override
    public boolean handleError(int messageRes) {
        return handleError(getString(messageRes));
    }
}
