package com.ibm.gbs.va.grit.biometrics;

import android.annotation.TargetApi;
import android.content.Context;
import android.hardware.biometrics.BiometricPrompt;
import android.os.Build;
import android.os.CancellationSignal;
import android.support.v4.hardware.fingerprint.FingerprintManagerCompat;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.util.CommonCallback;
import com.ibm.gbs.va.grit.util.RemoteLog;

import javax.crypto.Cipher;

public class Biometrics {
    private final Context context;
    private CommonCallback biometricsSuccessCallback;
    private CommonCallback biometricsErrorCallback;

    private static final String TAG = "Biometrics";

    public Biometrics(Context context) {
        this.context = context;
    }

    /* BIOMETRICS for API 28+ */
    @TargetApi(Build.VERSION_CODES.P)
    public void displayBiometricsPrompt(CommonCallback biometricsSuccesscallback, CommonCallback biometricsErrorCallback, Cipher cipher) {
        this.biometricsSuccessCallback = biometricsSuccesscallback;
        this.biometricsErrorCallback = biometricsErrorCallback;
        BiometricPrompt biometricPrompt = new BiometricPrompt.Builder(context)
                .setTitle(context.getString(R.string.biometrics_title))
                .setSubtitle(context.getString(R.string.biometrics_sub_title))
                .setDescription(context.getString(R.string.biometrics_description))
                .setNegativeButton(context.getString(R.string.cancel), context.getMainExecutor(), (dialogInterface, i) ->
                        callBiometricsErrorCallback("CANCEL", 0, "User Cancelled Biometrics Dialog"))
                .build();
        CancellationSignal cancellationSignal = getCancellationSignal();
        BiometricPrompt.AuthenticationCallback authenticationCallback = getAuthenticationCallback();
        BiometricPrompt.CryptoObject cryptoObject = new BiometricPrompt.CryptoObject(cipher);
        biometricPrompt.authenticate(cryptoObject, cancellationSignal, context.getMainExecutor(), authenticationCallback);
    }

    private void callBiometricsErrorCallback(String errorMessage, int errorCode, String errorDescription) {
        try {
            biometricsErrorCallback.setMessage(errorMessage);
            biometricsErrorCallback.setErrorCode(errorCode);
            biometricsErrorCallback.setErrorDescription(errorDescription);
            biometricsErrorCallback.call();
        } catch (Exception e) {
            RemoteLog.e(TAG, "ERROR calling biometrics callback: " + e.getMessage(), e);
        }
    }

    @TargetApi(Build.VERSION_CODES.P)
    private CancellationSignal getCancellationSignal() {
        // With this cancel signal, we can cancel biometric prompt operation
        return new CancellationSignal();
    }

    @TargetApi(Build.VERSION_CODES.P)
    private BiometricPrompt.AuthenticationCallback getAuthenticationCallback() {
        // Callback for biometric authentication result
        return new BiometricPrompt.AuthenticationCallback() {
            @Override
            public void onAuthenticationSucceeded(BiometricPrompt.AuthenticationResult result) {
                super.onAuthenticationSucceeded(result);
                try {
                    biometricsSuccessCallback.setMessage("SUCCESS");
                    biometricsSuccessCallback.call();
                } catch (Exception e) {
                    RemoteLog.e(TAG, "ERROR calling success callback: " + e.getMessage(), e);
                }
            }

            @Override
            public void onAuthenticationError(int errorCode, CharSequence errString) {
                super.onAuthenticationError(errorCode, errString);
                callBiometricsErrorCallback("AUTHENTICATION_ERROR", errorCode, errString.toString());
            }

            @Override
            public void onAuthenticationFailed() {
                super.onAuthenticationFailed();
                callBiometricsErrorCallback("AUTHENTICATION_FAILURE", 0, "Biometrics Authentication Failed");
            }
        };
    }

    /*
     * Applies only to API 28 (Pie) (and maybe greater)
     */
    public static boolean isBiometricPromptEnabled(Context context) {
        //CURRENTLY THERE IS NO WAY TO DETERMINE IF USER IS ENROLLED IN THE DEVICE'S BIOMETRICS
        // HOWEVER, CURRENTLY ONLY FINGERPRINT IS IS AVAILABLE FOR BIOMETRICS, SO CHECK FOR ENROLLED FINGERPRINT
        FingerprintManagerCompat fingerprintManager = FingerprintManagerCompat.from(context);
        return fingerprintManager.hasEnrolledFingerprints() && (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P);
    }
}
