package com.ibm.gbs.va.grit.biometrics;

import android.app.KeyguardManager;
import android.content.Context;
import android.os.Build;
import android.security.keystore.KeyGenParameterSpec;
import android.security.keystore.KeyPermanentlyInvalidatedException;
import android.security.keystore.KeyProperties;
import android.support.v4.app.FragmentManager;

import com.ibm.gbs.va.grit.util.CommonCallback;
import com.ibm.gbs.va.grit.util.RemoteLog;

import java.io.IOException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;

public class BiometricsManager {
    private static final String TAG = "BiometricsManager";

    private final Context context;
    private Biometrics biometrics;
    private FingerprintHelper fingerprintHelper;

    private static final String KEY_NAME_NOT_INVALIDATED = "key_not_invalidated";
    private static final String DEFAULT_KEY_NAME = "default_key";

    private KeyStore mKeyStore;
    private KeyGenerator mKeyGenerator;
    private Cipher defaultCipher;
    private Cipher cipherNotInvalidated;

    public BiometricsManager(Context context) {
        this.context = context;
        if (Biometrics.isBiometricPromptEnabled(context)) {
            // Use Biometrics for API >= 28
            biometrics = new Biometrics(context);
        } else {
            //Use Fingerprint for APIs 23-27
            fingerprintHelper = new FingerprintHelper();
        }
        setUpEncryption();
    }

    /*
     * Check for Biometric and Fingerprint Capabilities
     *
     * Anything lower than API 28 will not work with Biometrics, needs Fingerprint
     *
     */
    public static boolean isBiometricsEnabled(Context context) {
        KeyguardManager keyguardManager = context.getSystemService(KeyguardManager.class);
        try {
            if (keyguardManager.isKeyguardSecure() && (Biometrics.isBiometricPromptEnabled(context) || FingerprintHelper.isFingerprintEnabled(context))) {
                return true;
            } else {
                return false;
            }
        } catch (Exception e) {
            RemoteLog.e(TAG, "ERROR: FingerprintManager libs not found, returninf false - " + e.getMessage(), e);
            return false;
        }
    }

    public void showBiometricsAlert(FragmentManager fragmentManager, CommonCallback biometricsSuccessCallback, CommonCallback biometricsErrorCallback) {
        Cipher fpCipher = defaultCipher;
        String fpCipherKey = DEFAULT_KEY_NAME;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            RemoteLog.v(TAG, "USE > 23");
            fpCipher = cipherNotInvalidated;
            fpCipherKey = KEY_NAME_NOT_INVALIDATED;
        }
        FingerprintAuthenticationDialogFragment.Stage fpStage;
        if (initCipher(fpCipher, fpCipherKey)) {
            // Show the fingerprint dialog. The user has the option to use the fingerprint with
            // crypto, or you can fall back to using a server-side verified password.
            fpStage = FingerprintAuthenticationDialogFragment.Stage.FINGERPRINT;
        } else {
            // This happens if the lock screen has been disabled or or a fingerprint got
            // enrolled. Thus show the dialog to authenticate with their password first
            // and ask the user if they want to authenticate with fingerprints in the
            // future
            fpStage = FingerprintAuthenticationDialogFragment.Stage.NEW_FINGERPRINT_ENROLLED;
        }
        if (Biometrics.isBiometricPromptEnabled(context)) {
            // Use Biometrics for API >= 28
            biometrics.displayBiometricsPrompt(biometricsSuccessCallback, biometricsErrorCallback, fpCipher);
        } else {
            //Use Fingerprint for APIs 23-27
            fingerprintHelper.startFingerprintAlert(fragmentManager, fpCipher, biometricsSuccessCallback, biometricsErrorCallback, fpStage);
        }
    }

    /*
     * SET UP FINGERPRINT AUTHENTICATION
     */
    private void setUpEncryption() {
        try {
            mKeyStore = KeyStore.getInstance("AndroidKeyStore");
        } catch (KeyStoreException e) {
            throw new RuntimeException("Failed to get an instance of KeyStore", e);
        }
        try {
            mKeyGenerator = KeyGenerator
                    .getInstance(KeyProperties.KEY_ALGORITHM_AES, "AndroidKeyStore");
        } catch (NoSuchAlgorithmException | NoSuchProviderException e) {
            throw new RuntimeException("Failed to get an instance of KeyGenerator", e);
        }
        try {
            defaultCipher = Cipher.getInstance(KeyProperties.KEY_ALGORITHM_AES + "/"
                    + KeyProperties.BLOCK_MODE_CBC + "/"
                    + KeyProperties.ENCRYPTION_PADDING_PKCS7);
            cipherNotInvalidated = Cipher.getInstance(KeyProperties.KEY_ALGORITHM_AES + "/"
                    + KeyProperties.BLOCK_MODE_CBC + "/"
                    + KeyProperties.ENCRYPTION_PADDING_PKCS7);
        } catch (NoSuchAlgorithmException | NoSuchPaddingException e) {
            throw new RuntimeException("Failed to get an instance of Cipher", e);
        }

        // Now the protection level of USE_FINGERPRINT permission is normal instead of dangerous.
        // See http://developer.android.com/reference/android/Manifest.permission.html#USE_FINGERPRINT
        // The line below prevents the false positive inspection from Android Studio
        // noinspection ResourceType
        createKey(DEFAULT_KEY_NAME, true);
        createKey(KEY_NAME_NOT_INVALIDATED, false);
    }

    /**
     * Initialize the {@link Cipher} instance with the created key in the {@link #createKey(String,
     * boolean)} method.
     *
     * @param keyName the key name to init the cipher
     * @return {@code true} if initialization is successful, {@code false} if the lock screen has
     * been disabled or reset after the key was generated, or if a fingerprint got enrolled after
     * the key was generated.
     */
    private boolean initCipher(Cipher cipher, String keyName) {
        try {
            mKeyStore.load(null);
            SecretKey key = (SecretKey) mKeyStore.getKey(keyName, null);
            cipher.init(Cipher.ENCRYPT_MODE, key);
            return true;
        } catch (KeyPermanentlyInvalidatedException e) {
            RemoteLog.e(TAG, "ERROR initializing cipher: " + e.getMessage(), e);
            return false;
        } catch (KeyStoreException | CertificateException | UnrecoverableKeyException | IOException
                | NoSuchAlgorithmException | InvalidKeyException e) {
            throw new RuntimeException("Failed to init Cipher", e);
        }
    }

    /**
     * Creates a symmetric key in the Android Key Store which can only be used after the user has
     * authenticated with fingerprint.
     *
     * @param keyName                          the name of the key to be created
     * @param invalidatedByBiometricEnrollment if {@code false} is passed, the created key will not
     *                                         be invalidated even if a new fingerprint is enrolled.
     *                                         The default value is {@code true}, so passing {@code
     *                                         true} doesn't change the behavior (the key will be
     *                                         invalidated if a new fingerprint is enrolled.). Note
     *                                         that this parameter is only valid if the app works on
     *                                         Android N developer preview.
     */
    private void createKey(String keyName, boolean invalidatedByBiometricEnrollment) {
        // The enrolling flow for fingerprint. This is where you ask the user to set up fingerprint
        // for your flow. Use of keys is necessary if you need to know if the set of
        // enrolled fingerprints has changed.
        try {
            mKeyStore.load(null);
            // Set the alias of the entry in Android KeyStore where the key will appear
            // and the constrains (purposes) in the constructor of the Builder
            KeyGenParameterSpec.Builder builder = new KeyGenParameterSpec.Builder(keyName,
                    KeyProperties.PURPOSE_ENCRYPT |
                            KeyProperties.PURPOSE_DECRYPT)
                    .setBlockModes(KeyProperties.BLOCK_MODE_CBC)
                    // Require the user to authenticate with a fingerprint to authorize every use
                    // of the key
                    .setUserAuthenticationRequired(true)
                    .setEncryptionPaddings(KeyProperties.ENCRYPTION_PADDING_PKCS7);

            // This is a workaround to avoid crashes on devices whose API level is < 24
            // because KeyGenParameterSpec.Builder#setInvalidatedByBiometricEnrollment is only
            // visible on API level +24.
            // Ideally there should be a compat library for KeyGenParameterSpec.Builder but
            // which isn't available yet.
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                builder.setInvalidatedByBiometricEnrollment(invalidatedByBiometricEnrollment);
            }
            mKeyGenerator.init(builder.build());
            mKeyGenerator.generateKey();
        } catch (NoSuchAlgorithmException | InvalidAlgorithmParameterException
                | CertificateException | IOException e) {
            throw new RuntimeException(e);
        }
    }
}
