package com.ibm.gbs.va.grit.biometrics;


import android.hardware.fingerprint.FingerprintManager;
import android.os.CancellationSignal;
import android.widget.ImageView;
import android.widget.TextView;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.util.CommonCallback;
import com.ibm.gbs.va.grit.util.RemoteLog;


/**
 * Small helper class to manage text/icon around fingerprint authentication UI.
 */
class FingerprintUiHelper extends FingerprintManager.AuthenticationCallback {
    private static final long ERROR_TIMEOUT_MILLIS = 1600;
    private static final long SUCCESS_DELAY_MILLIS = 1300;
    private static final String TAG = "FingerprintUiHelper";

    private final FingerprintManager mFingerprintManager;
    private final ImageView mIcon;
    private final TextView mErrorTextView;
    private CancellationSignal mCancellationSignal;

    private boolean mSelfCancelled;
    private final CommonCallback fingerprintSuccessCallback;
    private final CommonCallback fingerprintErrorCallback;

    /**
     * Constructor for {@link FingerprintUiHelper}.
     */
    FingerprintUiHelper(FingerprintManager fingerprintManager,
                        ImageView icon, TextView errorTextView, CommonCallback fingerprintSuccessCallback, CommonCallback fingerprintErrorCallback) {
        mFingerprintManager = fingerprintManager;
        mIcon = icon;
        mErrorTextView = errorTextView;
        this.fingerprintSuccessCallback = fingerprintSuccessCallback;
        this.fingerprintErrorCallback = fingerprintErrorCallback;
    }

    public boolean isFingerprintAuthAvailable() {
        // The line below prevents the false positive inspection from Android Studio
        // noinspection ResourceType
        return mFingerprintManager.isHardwareDetected()
                && mFingerprintManager.hasEnrolledFingerprints();
    }

    public void startListening(FingerprintManager.CryptoObject cryptoObject) {
        if (!isFingerprintAuthAvailable()) {
            return;
        }
        mCancellationSignal = new CancellationSignal();
        mSelfCancelled = false;
        // The line below prevents the false positive inspection from Android Studio
        // noinspection ResourceType
        mFingerprintManager
                .authenticate(cryptoObject, mCancellationSignal, 0 /* flags */, this, null);
        mIcon.setImageResource(R.drawable.ic_fingerprint_svg);
    }

    public void stopListening() {
        if (mCancellationSignal != null) {
            mSelfCancelled = true;
            mCancellationSignal.cancel();
            mCancellationSignal = null;
            callFingerprintErrorCallback("CANCEL");
        }
    }

    private void callFingerprintErrorCallback(String errorMessage) {
        try {
            fingerprintErrorCallback.setMessage(errorMessage);
            fingerprintErrorCallback.call();
        } catch (Exception e) {
            RemoteLog.e(TAG, "ERROR calling fingerprint error: " + e.getMessage(), e);
        }
    }

    @Override
    public void onAuthenticationError(int errMsgId, CharSequence errString) {
        if (!mSelfCancelled) {
            showError(errString);
            mIcon.postDelayed(() -> callFingerprintErrorCallback("AUTHENTICATION_ERROR"), ERROR_TIMEOUT_MILLIS);
        }
    }

    @Override
    public void onAuthenticationHelp(int helpMsgId, CharSequence helpString) {
        showError(helpString);
    }

    @Override
    public void onAuthenticationFailed() {
        showError(mIcon.getResources().getString(R.string.fingerprint_not_recognized));
    }

    @Override
    public void onAuthenticationSucceeded(FingerprintManager.AuthenticationResult result) {
        mErrorTextView.removeCallbacks(mResetErrorTextRunnable);
        mIcon.setImageResource(R.drawable.ic_success_filled);
        mErrorTextView.setTextColor(
                mErrorTextView.getResources().getColor(R.color.colorSuccess, null));
        mErrorTextView.setText(
                mErrorTextView.getResources().getString(R.string.fingerprint_success));
        mIcon.postDelayed(() -> {
        }, SUCCESS_DELAY_MILLIS);
        try {
            fingerprintSuccessCallback.call();
        } catch (Exception e) {
            RemoteLog.e(TAG, "ERROR calling fingerprintSuccessCallback: " + e.getMessage(), e);
        }
    }

    private void showError(CharSequence error) {
        mIcon.setImageResource(R.drawable.ic_invalid_filled);
        mErrorTextView.setText(error);
        mErrorTextView.setTextColor(
                mErrorTextView.getResources().getColor(R.color.colorError, null));
        mErrorTextView.removeCallbacks(mResetErrorTextRunnable);
        mErrorTextView.postDelayed(mResetErrorTextRunnable, ERROR_TIMEOUT_MILLIS);
        try {
            fingerprintErrorCallback.call();
        } catch (Exception e) {
            RemoteLog.e(TAG, "ERROR calling fingerprintErrorCallback: " + e.getMessage(), e);
        }
    }

    private final Runnable mResetErrorTextRunnable = new Runnable() {
        @Override
        public void run() {
            mErrorTextView.setTextColor(
                    mErrorTextView.getResources().getColor(R.color.colorDarkText, null));
            mErrorTextView.setText(
                    mErrorTextView.getResources().getString(R.string.fingerprint_hint));
            mIcon.setImageResource(R.drawable.ic_fingerprint_svg);
        }
    };
}
