package com.ibm.gbs.va.grit.fragment.login;

import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.arch.lifecycle.LiveData;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.annotation.StringRes;
import android.support.v4.app.FragmentManager;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.android.databinding.library.baseAdapters.BR;
import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.activity.AuthenticationActivity;
import com.ibm.gbs.va.grit.biometrics.Biometrics;
import com.ibm.gbs.va.grit.biometrics.BiometricsManager;
import com.ibm.gbs.va.grit.databinding.FragmentLoginBinding;
import com.ibm.gbs.va.grit.fragment.AnalyticsFragment;
import com.ibm.gbs.va.grit.fragment.dialog.AlertFragment;
import com.ibm.gbs.va.grit.model.ValidationRule;
import com.ibm.gbs.va.grit.repository.UserRepository;
import com.ibm.gbs.va.grit.rest.MessageHandler;
import com.ibm.gbs.va.grit.rest.NetworkErrorDelegate;
import com.ibm.gbs.va.grit.util.CommonCallback;
import com.ibm.gbs.va.grit.util.Keystore;
import com.ibm.gbs.va.grit.util.LocalValidationHelper;
import com.ibm.gbs.va.grit.util.StringUtil;

import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

public class LoginFragment extends AnalyticsFragment
        implements NetworkErrorDelegate, IAuthenticationFragment {

    LocalValidationHelper emailValidationHelper;
    LocalValidationHelper passwordValidationHelper;
    FragmentLoginBinding binding;
    private boolean isRegisteredForBiometricLogin;
    private BiometricsManager biometricsManager;
    LiveData<Boolean> loggingIn;
    List<LocalValidationHelper> validationHelpers;

    @StringRes
    private int passwordKeyAlias, cipherIvKeyAlias;

    @Inject
    UserRepository userRepo;
    @Inject
    Keystore keystore;

    public final static String ENTRY_MSG = "ENTRY_MSG";

    public static LoginFragment newInstance(String snackbarMsg) {
        LoginFragment f = new LoginFragment();

        Bundle args = new Bundle();
        args.putString(ENTRY_MSG, snackbarMsg);
        f.setArguments(args);

        return f;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        TAG = "LoginFragment";
        addNetworkDelegate();
        binding = FragmentLoginBinding.inflate(inflater, container, false);
        binding.setLifecycleOwner(this);

        prePopulateEmail();

        validationHelpers = new ArrayList<>();
        emailValidationHelper = new LocalValidationHelper(binding.emailInputLayout, ValidationRule.NO_EMPTY);
        emailValidationHelper.validateAfterChange();
        validationHelpers.add(emailValidationHelper);

        passwordValidationHelper = new LocalValidationHelper(binding.passwordInputLayout, ValidationRule.NO_EMPTY);
        passwordValidationHelper.validateAfterChange();
        validationHelpers.add(passwordValidationHelper);

        binding.loginSubmitBtn.setOnClickListener(view ->
                LocalValidationHelper.validateInputs(validationHelpers, l ->
                        onSubmit(binding.emailInputText.getText().toString(), binding.passwordInputText.getText().toString())));

        binding.loginLinkBtn.setOnClickListener(v -> getAuthenticationActivity().goToSignUp());
        binding.learnMoreBtn.setOnClickListener(v -> getAuthenticationActivity().goToWalkthrough());

        binding.forgotPasswordLink.setOnClickListener(v -> getAuthenticationActivity().goToForgotPassword());
        binding.titleBar.setVariable(BR.titleText, getString(R.string.login));

        Bundle args = getArguments();
        if (args != null) {
            String message = args.getString(ENTRY_MSG);
            if (message != null && !message.isEmpty()) {
                MessageHandler.handleSuccess(getActivity(), message);
            }
        }

        setUpBiometrics();

        return binding.getRoot();
    }

    void prePopulateEmail() {
        String savedUserEmail = userRepo.getUserEmail();
        if (!savedUserEmail.isEmpty()) {
            binding.emailInputText.setText(savedUserEmail);
        }
    }

    //region Biometrics
    private void setUpBiometrics() {
        passwordKeyAlias = R.string.USERPREF_ENCRYPTED_PASSWORD_KEY_ALIAS;
        cipherIvKeyAlias = R.string.USERPREF_ENCRYPTED_PASSWORD_CIPHER_IV_KEY_ALIAS;

        //Check if Biometrics or Fingerprint is enabled, if not, hide checkbox
        boolean biometricsAreEnabled = BiometricsManager.isBiometricsEnabled(getContext());
        isRegisteredForBiometricLogin = keystore.keystoreAliasExists(passwordKeyAlias);
        if (biometricsAreEnabled) {
            biometricsManager = new BiometricsManager(getContext());
            binding.fingerprintCheckbox.setVisibility(View.VISIBLE);
            if (isRegisteredForBiometricLogin) {
                showBiometricsLoginPrompt();
            }
        } else {
            binding.fingerprintCheckbox.setVisibility(View.GONE);
        }
    }

    void biometricEnrollment() {
        //ENROLL FOR BIOMETRICS LOGIN
        final FragmentManager fragmentManager = getFragmentManager();
        biometricsManager.showBiometricsAlert(fragmentManager, new CommonCallback() {
                    public Void call() {

                        //SAVE EMAIL
                        userRepo.setCurrentBiometricsEmail(binding.emailInputText.getText().toString());

                        //ENCRYPT AND SAVE PASSWORD
                        keystore.encryptAndSaveToUserPrefs(binding.passwordInputText.getText().toString(), passwordKeyAlias, cipherIvKeyAlias);

                        //NEED TO WAIT FOR BIOMETRICS TO FINISH BEFORE DOING THIS
                        getAuthenticationActivity().enterApp();
                        return null;
                    }
                }, new CommonCallback() {
                    public Void call() {
                        if (getMessage().equals("CANCEL")) {
                            getAuthenticationActivity().enterApp();
                        }
                        if (getMessage().equals("AUTHENTICATION_ERROR")) {
                            userRepo.signUserOut();
                            if (!getClass().getSimpleName().equals("LoginFragment")) {
                                getAuthenticationActivity().goToLogin(null);
                            }
                        }
                        binding.fingerprintCheckbox.setChecked(false);
                        return null;
                    }
                }
        );
    }

    void showBiometricsLoginPrompt() {
        boolean userHasBiometrics = !TextUtils.isEmpty(userRepo.getCurrentBiometricsEmail());
        if (userHasBiometrics)
            binding.fingerprintCheckbox.setChecked(true);

        //If Biometrics is registered and for some reason checkbox is unchecked, but then re-checked, prompt for bio login.
        binding.fingerprintCheckbox.setOnCheckedChangeListener((compoundButton, b) -> {
            if (compoundButton.isChecked()) {
                userRepo.setCurrentUserEmail(binding.emailInputText.getText().toString()); // user might have changed what email was typed in
                showBiometricsLoginPrompt();
            }
        });

        if (!userHasBiometrics) return; // don't prompt if they aren't registered

        //SHOW BIOMETRICS LOGIN ALERT & HANDLE ACTIONS AFTER CALLBACKS
        final FragmentManager fragmentManager = getFragmentManager();
        biometricsManager.showBiometricsAlert(fragmentManager, new CommonCallback() {
                    public Void call() {
                        //GET USER NAME FROM PREFS AND PASSWORD FROM KEYSTORE (OR IF USER GIVES PASSWORD ON FingerPrint) AND AUTOLOGIN
                        //RETRIEVE EMAIL
                        String bioEmailFromPrefs = userRepo.getCurrentBiometricsEmail();

                        // RETRIEVE AND DECRYPT PASSWORD WITH KEYSTORE
                        String decryptedPasswordFromPrefs = keystore.decryptTextFromUserPrefs(passwordKeyAlias, cipherIvKeyAlias);

                        //Login
                        onSubmit(bioEmailFromPrefs, decryptedPasswordFromPrefs);
                        return null;
                    }
                }, new CommonCallback() {
                    public Void call() {

                        binding.fingerprintCheckbox.setChecked(false);

                        // For API >= 28 Only - If tries Bio to many times and is locked, show alert if tries while locked.
                        if (Biometrics.isBiometricPromptEnabled(getContext()) && getMessage().equals("AUTHENTICATION_ERROR")) {
                            // Use Biometrics for API >= 28
                            if (getActivity().hasWindowFocus()) {
                                FragmentManager fragmentManager = getFragmentManager();
                                AlertFragment alertFragment = AlertFragment.newInstance(getString(R.string.biometrics_error_too_many_failed_attempts));
                                alertFragment.show(fragmentManager, getString(R.string.biometrics_error_too_many_failed_attempts));
                            }
                        }
                        return null;
                    }
                }
        );
    }
    //endregion

    void onSubmit(String email, String password) {
        showProgress(true);
        email = StringUtil.convertEmailCase(email);
        loggingIn = userRepo.login(email, password);
        loggingIn.observe(this, userLoggedIn -> {
            if (userLoggedIn != null && userLoggedIn) {
                if (binding.fingerprintCheckbox.isChecked() && !isRegisteredForBiometricLogin) {
                    biometricEnrollment();
                } else {
                    getAuthenticationActivity().enterApp();
                }
            }
            loggingIn.removeObservers(this);
            showProgress(false);
        });
    }

    void showProgress(final boolean show) {
        int shortAnimTime = getResources().getInteger(android.R.integer.config_shortAnimTime);
        binding.loginProgress.setVisibility(show ? View.VISIBLE : View.GONE);
        binding.loginProgress.animate().setDuration(shortAnimTime).alpha(
                show ? 1 : 0).setListener(new AnimatorListenerAdapter() {
            @Override
            public void onAnimationEnd(Animator animation) {
                binding.loginProgress.setVisibility(show ? View.VISIBLE : View.GONE);
            }
        });
    }

    @Override
    public AuthenticationActivity getAuthenticationActivity() {
        return (AuthenticationActivity) getActivity();
    }

    void addNetworkDelegate() {
        getAuthenticationActivity().addDelegateToList(this);
    }

    @Override
    public boolean handleError(String message) {
        MessageHandler.handleError(getActivity(), message, loggingIn);
        emailValidationHelper.setErrorIcon(true);
        passwordValidationHelper.setErrorIcon(true);
        return true;
    }

    @Override
    public boolean handleError(int strResource) {
        return handleError(getString(strResource));
    }

}
