package com.ibm.gbs.va.grit.fragment.settings;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.ViewModelProvider;
import android.arch.lifecycle.ViewModelProviders;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.activity.DashboardActivity;
import com.ibm.gbs.va.grit.databinding.FragmentEditProfileBinding;
import com.ibm.gbs.va.grit.fragment.AnalyticsFragment;
import com.ibm.gbs.va.grit.fragment.dialog.ChangePasswordDialogFragment;
import com.ibm.gbs.va.grit.fragment.dialog.GritDialogFragment;
import com.ibm.gbs.va.grit.model.ValidationRule;
import com.ibm.gbs.va.grit.model.dialog.TextSection;
import com.ibm.gbs.va.grit.repository.UserRepository;
import com.ibm.gbs.va.grit.rest.MessageHandler;
import com.ibm.gbs.va.grit.rest.NetworkErrorDelegate;
import com.ibm.gbs.va.grit.rest.NetworkErrorListener;
import com.ibm.gbs.va.grit.util.Consumer;
import com.ibm.gbs.va.grit.util.LocalValidationHelper;
import com.ibm.gbs.va.grit.util.RemoteLog;
import com.ibm.gbs.va.grit.viewmodel.EditProfileViewModel;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import javax.inject.Inject;

public class EditProfileFragment extends AnalyticsFragment implements NetworkErrorDelegate {
    private static final String argShowChangePassword = "arg_show_change_password";

    private final List<LocalValidationHelper> validations = new LinkedList<>();
    private final List<LocalValidationHelper> birthdateValidations = new LinkedList<>();

    private LiveData<UserRepository.UPDATE_STATE> updatingUser;
    private FragmentEditProfileBinding binding;
    private EditProfileViewModel model;
    private boolean emailBeingEdited = false;
    private boolean haveEditsBeenMade = false;

    @Inject
    ViewModelProvider.Factory viewModelFactory;

    public static EditProfileFragment createInstanceForChangePassword() {
        EditProfileFragment frag = new EditProfileFragment();
        Bundle args = new Bundle();
        args.putBoolean(argShowChangePassword, true);
        frag.setArguments(args);
        return frag;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        TAG = "EditProfileFragment";
        assert getActivity() != null;
        if (getActivity() instanceof NetworkErrorListener) {
            ((NetworkErrorListener) getActivity()).addDelegateToList(this);
        }

        binding = FragmentEditProfileBinding.inflate(inflater, container, false);
        model = ViewModelProviders.of(this, viewModelFactory).get(EditProfileViewModel.class);
        model.user.observe(this, user -> {
        });
        binding.setLifecycleOwner(this);
        binding.setModel(model);
        binding.locationText.setAdapter(new ArrayAdapter<>(getActivity(), R.layout.list_item_dropdown, model.citiesList));
        birthdateValidations.addAll(Arrays.asList(
                new LocalValidationHelper(binding.monthLayout, ValidationRule.MONTH),
                new LocalValidationHelper(binding.dayLayout, ValidationRule.DAY),
                new LocalValidationHelper(binding.yearLayout, ValidationRule.YEAR))
        );
        validations.addAll(Arrays.asList(
                new LocalValidationHelper(binding.nameLayout, ValidationRule.NO_EMPTY),
                new LocalValidationHelper(binding.emailLayout, ValidationRule.EMAIL)
        ));
        validations.addAll(birthdateValidations);
        for (LocalValidationHelper h : validations) {
            h.validateAfterChange();
        }

        binding.changePasswordBtn.setOnClickListener(v -> openChangePasswordDialog());
        model.isEmailBeingEdited.observe(this, emailBeingEdited -> this.emailBeingEdited = emailBeingEdited != null && emailBeingEdited);
        model.haveEditsBeenMade.observe(this, haveEditsBeenMade -> this.haveEditsBeenMade = haveEditsBeenMade != null && haveEditsBeenMade);
        binding.saveBtn.setOnClickListener(v -> LocalValidationHelper.validateInputs(validations, w -> {
            View failedView = evaluateDateFields(w);
            if (failedView != null) {
                failedView.requestFocus();
            } else if (!haveEditsBeenMade) {
                MessageHandler.handleWarning(getActivity(), getString(R.string.please_edit_a_field));
            } else if (emailBeingEdited) {
                GritDialogFragment.showTwoBtnWarningDialog(getFragmentManager(),
                        R.string.change_email,
                        okBtn -> submitUpdate(),
                        TextSection.ofText(R.string.change_email_dialog_body));
            } else {
                // call api
                submitUpdate();
            }
        }));

        return binding.getRoot();
    }

    private View evaluateDateFields(View failedView) {
        String errStr = getString(R.string.empty_input_err);
        Boolean allEmpty = true;
        Boolean allFilled = true;
        for (LocalValidationHelper item : birthdateValidations) {
            allEmpty = allEmpty && TextUtils.isEmpty(item.getText());
            allFilled = allFilled && !TextUtils.isEmpty(item.getText());
        }
        if (!(allEmpty || allFilled)) {
            for (LocalValidationHelper item : birthdateValidations) {
                if (TextUtils.isEmpty(item.getText())) {
                    item.setError(errStr);
                    failedView = failedView != null ? failedView : item.getInputLayout();
                }
            }
        }
        return failedView;
    }

    private void submitUpdate() {
        updatingUser = model.updateUser();
        updatingUser.observe(this, userUpdated -> {
            if (userUpdated == null) return;

            switch (userUpdated) {
                case UPDATED_WITH_BADGE: // we don't show an extra dialog, only badge dialog (handled elsewhere)
                    returnToDashboard(null);
                    break;
                case UPDATED_NO_BADGE: // we show the dialog here
                    returnToDashboard(then ->
                            GritDialogFragment.showSuccessDialog(getFragmentManager(), R.string.success_dialog_profile_update_body));
                    break;
                case UPDATED_FAILED: // failure
                    userUpdateFailed(null);
                    break;
            }
            updatingUser.removeObservers(this);
        });
    }

    private void returnToDashboard(@Nullable Consumer<Void> callback) {
        if (getActivity() != null) {
            callback = callback != null ? callback : c -> {
            };
            DashboardActivity dashActivity = ((DashboardActivity) getActivity());
            dashActivity.onBackPressed(); // pop to settings fragment
            dashActivity.swapFragments(R.id.dashboard);
            callback.accept(null);
        } else RemoteLog.w(TAG, "fragment not attached to activity");
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        Bundle args = getArguments();
        if (args != null && args.getBoolean(argShowChangePassword)) {
            binding.nestedScrollView.post(() -> binding.nestedScrollView.fullScroll(View.FOCUS_DOWN));
            openChangePasswordDialog();
        }
    }

    private void openChangePasswordDialog() {
        ChangePasswordDialogFragment.showInstance(getFragmentManager());
    }

    private void userUpdateFailed(String message) {
        MessageHandler.handleError(getActivity(), message);
        for (LocalValidationHelper item : validations) {
            item.clearIcons();
        }
    }

    @Override
    public boolean handleError(String message) {
        if (updatingUser != null && updatingUser.hasObservers()) {
            updatingUser.removeObservers(this);
            userUpdateFailed(message);
            return true;
        }
        return false;
    }

    @Override
    public boolean handleError(int messageRes) {
        return handleError(getString(messageRes));
    }
}