package com.ibm.gbs.va.grit.model.notifications;

import android.arch.persistence.room.ColumnInfo;
import android.arch.persistence.room.Entity;
import android.arch.persistence.room.ForeignKey;
import android.arch.persistence.room.Ignore;
import android.arch.persistence.room.Index;
import android.arch.persistence.room.PrimaryKey;
import android.content.Context;
import android.support.annotation.DrawableRes;
import android.support.annotation.IdRes;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;

import com.google.gson.Gson;
import com.google.gson.annotations.SerializedName;
import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.model.type.MultiLangText;
import com.ibm.gbs.va.grit.model.type.User;

import org.joda.time.DateTime;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@Entity(
        foreignKeys = {
                @ForeignKey(entity = User.class,
                        parentColumns = "email",
                        childColumns = "user_email",
                        onDelete = ForeignKey.RESTRICT,
                        onUpdate = ForeignKey.CASCADE)},
        indices = @Index("user_email")
)
public class Notification implements Comparable<Notification> {

    /**
     * The ID of the notification. This will be unique within a context a user (but ids can be
     * shared between users).
     */
    @SerializedName("notificationId")
    @ColumnInfo(name = "id")
    private String notificationId;

    /**
     * The date (and time) that the notification will become "active". This will be in ISO-8601 date
     * and time format and always in UTC time.
     */
    @SerializedName("effectiveDate")
    @ColumnInfo(name = "effective_date")
    private Date effectiveDate;

    @SerializedName("title")
    @ColumnInfo(name = "title")
    private MultiLangText title;

    @SerializedName("description")
    @ColumnInfo(name = "description")
    private MultiLangText description;

    /**
     * Optional field for notification specific extra data.
     */
    @SerializedName("metadata")
    @ColumnInfo(name = "metadata")
    private String metadata;

    /**
     * If this is true, when the effectiveDate is passed, the mobile app will send a local
     * notification to the user.
     */
    @SerializedName("systemNotification")
    @ColumnInfo(name = "system_notification")
    private Boolean systemNotification;

    @SerializedName("notificationType")
    @ColumnInfo(name = "notification_type")
    private Type type;

    public Notification() {
    }

    @Ignore
    public Notification(String id, MultiLangText title, MultiLangText description, Type type) {
        this.notificationId = id;
        this.effectiveDate = new Date(0);
        this.title = title;
        this.description = description;
        this.systemNotification = false;
        this.type = type;
    }

    //region Static Initializers
    public static Notification localJobInfoNotification(Context ctx) {
        MultiLangText notificationTitle = new MultiLangText();
        notificationTitle.setAll(ctx.getString(R.string.complete_job_info_title));
        MultiLangText notificationBody = new MultiLangText();
        notificationBody.setAll(ctx.getString(R.string.complete_job_info_body));
        return new Notification("localJobInfo", notificationTitle, notificationBody, Type.UPDATE_JOB_INFO);
    }

    public static Notification localProfileInfoNotification(Context ctx) {
        MultiLangText notificationTitle = new MultiLangText();
        notificationTitle.setAll(ctx.getString(R.string.complete_profile_title));
        MultiLangText notificationBody = new MultiLangText();
        notificationBody.setAll(ctx.getString(R.string.complete_profile_body));
        return new Notification("localProfileInfo", notificationTitle, notificationBody, Type.UPDATE_PROFILE);
    }

    public static Notification localSquadInfoNotification(Context ctx) {
        MultiLangText notificationTitle = new MultiLangText();
        notificationTitle.setAll(ctx.getString(R.string.complete_squad_title));
        MultiLangText notificationBody = new MultiLangText();
        notificationBody.setAll(ctx.getString(R.string.complete_squad_body));
        return new Notification("localSquadInfo", notificationTitle, notificationBody, Type.UPDATE_SQUAD);
    }

    public static Notification localBaselineInfoNotification(Context ctx) {
        MultiLangText notificationTitle = new MultiLangText();
        notificationTitle.setAll(ctx.getString(R.string.complete_initial_baseline_title));
        MultiLangText notificationBody = new MultiLangText();
        notificationBody.setAll(ctx.getString(R.string.complete_initial_baseline_body));
        return new Notification("localBaselineSurvey", notificationTitle, notificationBody, Type.BASELINE_INITIAL);
    }

    public static Notification localContinueSurveyNotification(Context ctx) {
        MultiLangText notificationTitle = new MultiLangText();
        notificationTitle.setAll(ctx.getString(R.string.survey_in_progress_title));
        MultiLangText notificationBody = new MultiLangText();
        notificationBody.setAll(ctx.getString(R.string.survey_in_progress_body));
        return new Notification("localContinueSurvey", notificationTitle, notificationBody, Type.CONTINUING_SURVEY);
    }
    //endregion

    //region Getters/Setters
    public String getNotificationId() {
        return notificationId;
    }

    public void setNotificationId(String notificationId) {
        this.notificationId = notificationId;
    }

    public Date getEffectiveDate() {
        return effectiveDate;
    }

    public void setEffectiveDate(Date effectiveDate) {
        this.effectiveDate = effectiveDate;
    }

    public MultiLangText getTitle() {
        return title;
    }

    public void setTitle(MultiLangText title) {
        this.title = title;
    }

    public MultiLangText getDescription() {
        return description;
    }

    public void setDescription(MultiLangText description) {
        this.description = description;
    }

    public String getMetadata() {
        return metadata;
    }

    public void setMetadata(String metadata) {
        this.metadata = metadata;
    }

    public Boolean getSystemNotification() {
        return systemNotification;
    }

    public void setSystemNotification(Boolean systemNotification) {
        this.systemNotification = systemNotification;
    }

    public Type getType() {
        return type;
    }

    public void setType(Type type) {
        this.type = type;
    }
    //endregion

    //region Transients
    /**
     * Internal id used to store in the Room DB
     */
    @SuppressWarnings("NullableProblems")
    @NonNull
    @ColumnInfo(name = "internal_id")
    @PrimaryKey(autoGenerate = true)
    private transient Long internalId;

    /**
     * Associated user for the notification
     */
    @ColumnInfo(name = "user_email")
    private transient String userEmail;

    @NonNull
    public Long getInternalId() {
        return internalId;
    }

    public void setInternalId(@NonNull Long internalId) {
        this.internalId = internalId;
    }

    public String getUserEmail() {
        return userEmail;
    }

    public void setUserEmail(String userEmail) {
        this.userEmail = userEmail;
    }
    //endregion

    @NonNull
    @Override
    public String toString() {
        return new Gson().toJson(this);
    }

    @Override
    public boolean equals(@Nullable Object obj) {
        if (obj instanceof Notification)
            return notificationId.equals(((Notification) obj).notificationId);
        return super.equals(obj);
    }

    @Override
    public int compareTo(@NonNull Notification other) {
        if (equals(other)) // drop duplicates
            return 0;
        if (this.type.priorityWeight == other.type.priorityWeight) // on match, sort by ascending date
            return new DateTime(this.effectiveDate).compareTo(new DateTime(other.effectiveDate));
        return Integer.compare(this.type.priorityWeight, other.type.priorityWeight);
    }

    public enum Type {
        @SerializedName("IMMEDIATE_HELP")
        IMMEDIATE_HELP("IMMEDIATE_HELP", false, 0),
        @SerializedName("DIGITAL_ASSISTANT")
        DIGITAL_ASSISTANT("DIGITAL_ASSISTANT", false, 10),
        @SerializedName("FEEDBACK")
        FEEDBACK("FEEDBACK", false, 20),
        @SerializedName("CONTINUING_SURVEY")
        CONTINUING_SURVEY("CONTINUING_SURVEY", true, 30),
        @SerializedName("BASELINE_INITIAL")
        BASELINE_INITIAL("BASELINE_INITIAL", true, 35),
        @SerializedName("MOOD_TEXT")
        MOOD_TEXT("MOOD_TEXT", true, 40),
        @SerializedName("MOOD_FACES")
        MOOD_FACES("MOOD_FACES", true, 45),
        @SerializedName("LIFE_CHANGES")
        LIFE_CHANGES("LIFE_CHANGES", true, 50),
        @SerializedName("BASELINE")
        BASELINE("BASELINE", true, 60),
        @SerializedName("UPDATE_PROFILE")
        UPDATE_PROFILE("UPDATE_PROFILE", true, 70),
        @SerializedName("UPDATE_SQUAD")
        UPDATE_SQUAD("UPDATE_SQUAD", true, 75),
        @SerializedName("UPDATE_JOB_INFO")
        UPDATE_JOB_INFO("UPDATE_JOB_INFO", true, 80),
        @SerializedName("DEFAULT_NOTIFICATION_TYPE")
        DEFAULT_NOTIFICATION_TYPE("DEFAULT_NOTIFICATION_TYPE", false, 1000);


        private final String value;
        private final boolean hasBadge;
        private int priorityWeight;

        private final static Map<String, Type> CONSTANTS = new HashMap<>();

        static {
            for (Type c : values()) {
                CONSTANTS.put(c.value, c);
            }
        }

        Type(String value, boolean hasBadge, int priorityWeight) {
            this.value = value;
            this.hasBadge = hasBadge;
            this.priorityWeight = priorityWeight;
        }

        public String value() {
            return this.value;
        }

        public static Type fromValue(String value) {
            Type constant = CONSTANTS.get(value);
            if (constant == null) {
                throw new IllegalArgumentException(value);
            } else {
                return constant;
            }
        }

        public boolean hasBadge() {
            return hasBadge;
        }

        public boolean isSurvey() {
            switch (this) {
                case BASELINE:
                case MOOD_FACES:
                case MOOD_TEXT:
                case LIFE_CHANGES:
                    return true;

                case BASELINE_INITIAL:
                case CONTINUING_SURVEY:
                case UPDATE_PROFILE:
                case UPDATE_SQUAD:
                case UPDATE_JOB_INFO:
                case DIGITAL_ASSISTANT:
                case IMMEDIATE_HELP:
                case FEEDBACK:
                case DEFAULT_NOTIFICATION_TYPE:
                    return false;
            }
            return false;
        }

        public boolean isLocal() {
            switch (this) {
                case BASELINE_INITIAL:
                case CONTINUING_SURVEY:
                case UPDATE_PROFILE:
                case UPDATE_SQUAD:
                case UPDATE_JOB_INFO:
                case DEFAULT_NOTIFICATION_TYPE:
                    return true;

                case BASELINE:
                case MOOD_FACES:
                case MOOD_TEXT:
                case LIFE_CHANGES:
                case DIGITAL_ASSISTANT:
                case IMMEDIATE_HELP:
                case FEEDBACK:
                    return false;
            }
            return true;
        }

        @DrawableRes
        public int getImageId() {
            switch (this) {
                case BASELINE_INITIAL:
                case BASELINE:
                    return R.drawable.notification_baseline_assessment;
                case MOOD_FACES:
                case MOOD_TEXT:
                    return R.drawable.notification_mood_assessment;
                case LIFE_CHANGES:
                    return R.drawable.notification_life_changes_reassessment;
                case CONTINUING_SURVEY:
                    return R.drawable.notification_assessment_in_progress;
                case UPDATE_PROFILE:
                    return R.drawable.notification_base_profile;
                case UPDATE_SQUAD:
                    return R.drawable.notification_squad;
                case DIGITAL_ASSISTANT:
                    return R.drawable.notification_digital_assistant;
                case IMMEDIATE_HELP:
                    return R.drawable.notification_immediate_help;
                case UPDATE_JOB_INFO:
                    return R.drawable.notification_employment;
                case FEEDBACK:
                    return R.drawable.notification_feedback;
                case DEFAULT_NOTIFICATION_TYPE:
                    return R.drawable.notification_default;
            }
            return R.drawable.notification_default;
        }

        // returns -1 if not found
        @IdRes
        public int getMenuId() {
            switch (this) {
                case BASELINE_INITIAL:
                case BASELINE:
                case MOOD_FACES:
                case MOOD_TEXT:
                case LIFE_CHANGES:
                case CONTINUING_SURVEY:
                    return R.id.digitalSelf;
                case UPDATE_PROFILE:
                    return R.id.settings_editProfile;
                case UPDATE_SQUAD:
                    return R.id.squad;
                case DIGITAL_ASSISTANT:
                    return R.id.chat;
                case IMMEDIATE_HELP:
                    return R.id.immediateHelp;
                case UPDATE_JOB_INFO:
                    return R.id.jobs;
                case FEEDBACK:
                case DEFAULT_NOTIFICATION_TYPE:
                    return -1;
            }
            return -1;
        }

        public MessageAction getAction() {
            switch (this) {
                case BASELINE:
                    return MessageAction.BeginBaseline;
                case MOOD_FACES:
                    return MessageAction.BeginMoodFace;
                case MOOD_TEXT:
                    return MessageAction.BeginMoodText;
                case LIFE_CHANGES:
                    return MessageAction.BeginLifeChangeRe;
                case BASELINE_INITIAL:
                    return MessageAction.BeginBaselineInitial;
                case CONTINUING_SURVEY:
                    return MessageAction.ContinueSurvey;
                case UPDATE_PROFILE:
                    return MessageAction.UpdateProfile;
                case UPDATE_SQUAD:
                    return MessageAction.UpdateSquad;
                case DIGITAL_ASSISTANT:
                    return MessageAction.DigitalAssist;
                case IMMEDIATE_HELP:
                    return MessageAction.ImmediateHelp;
                case UPDATE_JOB_INFO:
                    return MessageAction.UpdateJobInfo;
                case FEEDBACK:
                    return MessageAction.Feedback;
                case DEFAULT_NOTIFICATION_TYPE:
                    return MessageAction.DEFAULT_NOTIFICATION_TYPE;
            }
            return MessageAction.DEFAULT_NOTIFICATION_TYPE;
        }

        public int getPriorityWeight() {
            return priorityWeight;
        }
    }


    public enum MessageAction {
        BeginBaseline,
        BeginBaselineInitial,
        BeginMoodText,
        BeginMoodFace,
        BeginLifeChangeRe,
        ContinueSurvey,
        UpdateSquad,
        UpdateJobInfo,
        UpdateProfile,
        DigitalAssist,
        Feedback,
        ImmediateHelp,
        DEFAULT_NOTIFICATION_TYPE
    }
}
