package com.ibm.gbs.va.grit.model.survey;

import android.arch.persistence.room.ColumnInfo;
import android.arch.persistence.room.Entity;
import android.arch.persistence.room.ForeignKey;
import android.arch.persistence.room.Ignore;
import android.arch.persistence.room.Index;
import android.support.annotation.NonNull;
import android.text.TextUtils;

import com.google.gson.Gson;
import com.google.gson.annotations.SerializedName;
import com.ibm.gbs.va.grit.model.type.User;

import java.util.LinkedList;

@Entity(
        primaryKeys = {"user_email", "question_id"},
        foreignKeys = {
                @ForeignKey(entity = User.class,
                        parentColumns = "email",
                        childColumns = "user_email",
                        onDelete = ForeignKey.CASCADE,
                        onUpdate = ForeignKey.CASCADE),
                @ForeignKey(entity = Question.class,
                        parentColumns = "id",
                        childColumns = "question_id",
                        onDelete = ForeignKey.RESTRICT)},
        indices = {
                @Index("user_email"),
                @Index("question_id")}
)
public class Answer {

    // ////////////////////////////////
    // FIELDS
    @SerializedName("questionId")
    @ColumnInfo(name = "question_id")
    @NonNull
    private String questionId;

    @SerializedName("questionVariantId")
    @ColumnInfo(name = "question_variant_id")
    private int questionVariantId;

    @SerializedName("refused")
    @ColumnInfo(name = "refused")
    private boolean refused;

    @SerializedName("noneOfTheAbove")
    @ColumnInfo(name = "answer_none_of_the_above")
    private boolean noneOfTheAbove;

    @SerializedName("numericalAnswer")
    @ColumnInfo(name = "numerical_answer")
    private Integer numericalAnswer;

    @SerializedName("text")
    @ColumnInfo(name = "text")
    private String text;

    @SerializedName("other")
    @ColumnInfo(name = "other")
    private String other;

    @SerializedName("answerIds")
    @ColumnInfo(name = "answer_ids")
    private LinkedList<String> answerIds;

    /**
     * Room Constructor - refused is calculated instead of retrieved.
     */
    public Answer(@NonNull String questionId, int questionVariantId, @SuppressWarnings("unused") boolean refused, boolean noneOfTheAbove, Integer numericalAnswer, String text, String other, LinkedList<String> answerIds, @NonNull String userEmail) {
        this.questionId = questionId;
        this.questionVariantId = questionVariantId;
        this.noneOfTheAbove = noneOfTheAbove;
        this.numericalAnswer = numericalAnswer;
        this.text = text;
        this.other = other;
        this.answerIds = answerIds;
        this.userEmail = userEmail;
        this.refused = isRefused();
    }

    @Ignore
    public Answer(Question q) {
        this.questionId = q.getQuestionId();
        this.questionVariantId = q.getQuestionText().get(0).getVariantId();
        this.noneOfTheAbove = false;
        this.numericalAnswer = null;
        this.text = null;
        this.other = q.getQuestionType() == Question.QuestionType.MULTI_SELECT_OTHER ?
                "" :
                null;
        this.answerIds = new LinkedList<>();
        this.refused = isRefused();
    }

    @Ignore
    public Answer(Question q, @NonNull String userEmail) {
        this(q);
        this.userEmail = userEmail;
        this.refused = isRefused();
    }

    // ////////////////////////////////
    // GETTERS & SETTERS
    @NonNull
    public String getQuestionId() {
        return questionId;
    }

    public void setQuestionId(@NonNull String questionId) {
        this.questionId = questionId;
    }

    public int getQuestionVariantId() {
        return questionVariantId;
    }

    public void setQuestionVariantId(int questionVariantId) {
        this.questionVariantId = questionVariantId;
    }

    public boolean isRefused() {
        refused = TextUtils.isEmpty(text) &&
                answerIds == null || answerIds.isEmpty() &&
                numericalAnswer == null &&
                !noneOfTheAbove;
        return refused;
    }

    public void setRefused(@SuppressWarnings("unused") boolean refused) {
        isRefused();
    }

    public boolean isNoneOfTheAbove() {
        return noneOfTheAbove;
    }

    public void setNoneOfTheAbove(boolean noneOfTheAbove) {
        this.noneOfTheAbove = noneOfTheAbove;
        if (noneOfTheAbove) {
            answerIds = new LinkedList<>();
        }
    }

    public Integer getNumericalAnswer() {
        return numericalAnswer;
    }

    public void setNumericalAnswer(Integer numericalAnswer) {
        this.numericalAnswer = numericalAnswer;
    }

    public String getText() {
        return text;
    }

    public void setText(String text) {
        this.text = text;
    }

    public String getOther() {
        return other;
    }

    public void setOther(String other) {
        this.other = other;
    }

    public LinkedList<String> getAnswerIds() {
        return answerIds;
    }

    public void setAnswerIds(LinkedList<String> answerIds) {
        this.answerIds = answerIds;
    }

    public boolean setSingularAnswer(String answerId) {
        if (answerIds == null) {
            answerIds = new LinkedList<>();
        }
        if (answerIds.size() == 0) answerIds.add(answerId);
        else if (!answerIds.get(0).equals(answerId)) {
            answerIds.pop();
            answerIds.push(answerId);
        } else {
            return false;
        }
        return true;
    }

    public void setMultiAnswer(String answerId) {
        if (answerIds == null) {
            answerIds = new LinkedList<>();
        }
        if (!answerIds.contains(answerId)) answerIds.add(answerId);
        else answerIds.remove(answerId);
    }

    // ////////////////////////////////
    // TRANSIENT FIELDS
    @ColumnInfo(name = "user_email")
    @NonNull
    transient
    private String userEmail;

    @NonNull
    public String getUserEmail() {
        return userEmail;
    }

    public void setUserEmail(@NonNull String userEmail) {
        this.userEmail = userEmail;
    }

    @NonNull
    @Override
    public String toString() {
        return new Gson().toJson(this);
    }
}
