package com.ibm.gbs.va.grit.model.type;

import android.arch.persistence.room.ColumnInfo;
import android.arch.persistence.room.Entity;
import android.arch.persistence.room.ForeignKey;
import android.arch.persistence.room.Ignore;
import android.arch.persistence.room.Index;
import android.arch.persistence.room.PrimaryKey;
import android.databinding.BaseObservable;
import android.databinding.Bindable;
import android.support.annotation.NonNull;
import android.text.TextUtils;

import com.google.gson.Gson;
import com.google.gson.annotations.SerializedName;
import com.ibm.gbs.va.grit.BR;
import com.ibm.gbs.va.grit.model.notifications.Notification;
import com.ibm.gbs.va.grit.model.survey.QuestionSet;

import androidx.annotation.Nullable;

@Entity(
        foreignKeys = {
                @ForeignKey(entity = QuestionSet.class,
                        parentColumns = "id",
                        childColumns = "question_set_id",
                        onDelete = ForeignKey.SET_NULL,
                        onUpdate = ForeignKey.CASCADE),
                @ForeignKey(entity = Notification.class,
                        parentColumns = "internal_id",
                        childColumns = "survey_spawning_notification_id",
                        onDelete = ForeignKey.SET_NULL,
                        onUpdate = ForeignKey.CASCADE)},
        indices = {
                @Index("question_set_id"),
                @Index("survey_spawning_notification_id"),
                @Index(value = "email", unique = true)}
)
public class User extends BaseObservable {
    private static final String DATE_REGEX = "^\\d{4}-\\d{2}-\\d{2}$";

    // ////////////////////////////////
    // FIELDS
    @SerializedName("name")
    @ColumnInfo(name = "name")
    private String name;

    @SerializedName("email")
    @ColumnInfo(name = "email")
    private String email;

    @SerializedName("location")
    @ColumnInfo(name = "location")
    private String location;

    @SerializedName("birthdate")
    @ColumnInfo(name = "birthdate")
    private String birthdate;

    public User() {
    }

    @Ignore
    public User(@NonNull String email, String token) {
        this.email = email;
        this.token = token;
    }

    @Ignore
    public User(String name, @NonNull String email, String birthdate) {
        this.name = name;
        this.email = email;
        this.birthdate = birthdate;
    }

    //region Getters/Setters
    @Bindable
    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
        notifyPropertyChanged(BR.name);
    }

    @Bindable
    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
        notifyPropertyChanged(BR.email);
    }

    @Bindable
    public String getLocation() {
        return location;
    }

    public void setLocation(String location) {
        this.location = location;
        notifyPropertyChanged(BR.location);
    }

    public String getBirthdate() {
        // construct birthdate from transients
        String construct = year + "-" + month + "-" + day;
        if (construct.matches(DATE_REGEX))
            birthdate = construct;
        if (TextUtils.isEmpty(year + month + day))
            birthdate = null;

        return birthdate;
    }

    public void setBirthdate(String birthdate) {
        this.birthdate = birthdate;
        if (!TextUtils.isEmpty(birthdate) && birthdate.matches(DATE_REGEX)) {
            String[] birthdateArr = birthdate.split("-");
            year = birthdateArr[0];
            month = birthdateArr[1];
            day = birthdateArr[2];
            notifyPropertyChanged(BR.year);
            notifyPropertyChanged(BR.month);
            notifyPropertyChanged(BR.day);
        }
    }
    //endregion

    //region Transients
    transient private String year;
    transient private String month;
    transient private String day;

    @ColumnInfo(name = "question_set_id")
    transient private String questionSetId;

    @ColumnInfo(name = "survey_spawning_notification_id")
    transient private Long surveySpawningNotificationId;

    /**
     * Key used in the local Room DB
     */
    @PrimaryKey(autoGenerate = true)
    @ColumnInfo(name = "primary_key")
    transient private int primaryKey;

    @ColumnInfo(name = "token")
    transient private String token;

    @Bindable
    public String getMonth() {
        return month;
    }

    public void setMonth(String month) {
        this.month = month;
        notifyPropertyChanged(BR.month);
        getBirthdate(); // refresh birthdate field
    }

    @Bindable
    public String getDay() {
        return day;
    }

    public void setDay(String day) {
        this.day = day;
        notifyPropertyChanged(BR.day);
        getBirthdate(); // refresh birthdate field
    }

    @Bindable
    public String getYear() {
        return year;
    }

    public void setYear(String year) {
        this.year = year;
        notifyPropertyChanged(BR.year);
        getBirthdate(); // refresh birthdate field
    }

    public String getQuestionSetId() {
        return questionSetId;
    }

    public void setQuestionSetId(String questionSetId) {
        this.questionSetId = questionSetId;
    }

    public Long getSurveySpawningNotificationId() {
        return surveySpawningNotificationId;
    }

    public void setSurveySpawningNotificationId(Long surveySpawningNotificationId) {
        this.surveySpawningNotificationId = surveySpawningNotificationId;
    }

    public int getPrimaryKey() {
        return primaryKey;
    }

    public void setPrimaryKey(int primaryKey) {
        this.primaryKey = primaryKey;
    }

    public String getToken() {
        return token;
    }

    public void setToken(String token) {
        this.token = token;
    }
    //endregion

    public User duplicate() {
        User newUser = new User();

        newUser.setPrimaryKey(primaryKey);
        newUser.setName(name);
        newUser.setEmail(email);
        newUser.setLocation(location);
        newUser.setBirthdate(getBirthdate());
        newUser.setQuestionSetId(questionSetId);
        newUser.setSurveySpawningNotificationId(surveySpawningNotificationId);
        newUser.setToken(token);

        return newUser;
    }

    public boolean isComplete() {
        return (!TextUtils.isEmpty(name) &&
                !TextUtils.isEmpty(email) &&
                !TextUtils.isEmpty(location) &&
                !TextUtils.isEmpty(getBirthdate()));
    }

    @NonNull
    @Override
    public String toString() {
        return new Gson().toJson(this);
    }

    @Override
    public boolean equals(@Nullable Object obj) {
        if (obj instanceof User) {
            User other = (User) obj;
            return TextUtils.equals(name, other.name) &&
                    TextUtils.equals(email, other.email) &&
                    TextUtils.equals(location, other.location) &&
                    TextUtils.equals(getBirthdate(), other.getBirthdate());
        }
        return super.equals(obj);
    }
}
