package com.ibm.gbs.va.grit.repository;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.MutableLiveData;
import android.text.TextUtils;

import com.ibm.gbs.va.grit.GritApplication;
import com.ibm.gbs.va.grit.model.chat.ChatMessage;
import com.ibm.gbs.va.grit.model.chat.Paragraph;
import com.ibm.gbs.va.grit.model.request.BaseRequest;
import com.ibm.gbs.va.grit.model.request.ConverseRequest;
import com.ibm.gbs.va.grit.rest.GritCallback;
import com.ibm.gbs.va.grit.rest.RetrofitApiServices;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

public class ChatRepository {
    private final RetrofitApiServices restServices;
    private final GritApplication gritApplication;
    private String conversationToken;

    private final MutableLiveData<List<ChatMessage>> messages = new MutableLiveData<>();

    public ChatRepository(RetrofitApiServices restServices, GritApplication gritApplication) {
        this.restServices = restServices;
        this.gritApplication = gritApplication;
        messages.setValue(new ArrayList<>());

        startConversation();
    }

    private void startConversation() {
        this.restServices.startConversation(new BaseRequest()).enqueue(new GritCallback<ChatMessage>(gritApplication) {
            @Override
            public void onSuccess(ChatMessage payload) {
                conversationToken = payload.getConversationToken();
                messages.setValue(Collections.singletonList(payload));
            }
        });
    }

    public void converse(ChatMessage message) {
        assert message.getText() != null;
        assert messages.getValue() != null;

        String textToSend = message.getFollowup() != null ?
                message.getFollowup().getSelection().getValue() :
                message.getText();

        // add the user's message
        List<ChatMessage> list = new ArrayList<>(messages.getValue());
        list.add(message);
        messages.setValue(list);

        // request the bot's response
        if (TextUtils.isEmpty(conversationToken)) { // conversation never started?
            startConversation();
        } else {
            ConverseRequest req = new ConverseRequest(conversationToken, textToSend);
            this.restServices.converse(req).enqueue(new GritCallback<ChatMessage>(gritApplication) {
                @Override
                public void onSuccess(ChatMessage payload) {
                    conversationToken = payload.getConversationToken();

                    for (ChatMessage singleMessage : breakUpComplexMessage(payload)) {
                        // post each message separately

                        List<ChatMessage> list = new ArrayList<>(messages.getValue());
                        list.add(singleMessage);
                        messages.setValue(list);

                    }
                }
            });
        }
    }

    public LiveData<List<ChatMessage>> getMessages() {
        return messages;
    }

    private List<ChatMessage> breakUpComplexMessage(ChatMessage parentMsg) {
        if (parentMsg.getMap() != null && !parentMsg.getChatResponse().isEmpty()) {
            // map message also has text
            ChatMessage precedingMsg = ChatMessage.duplicateBasicPropertiesOf(parentMsg);
            return Arrays.asList(precedingMsg, parentMsg);

        } else if (parentMsg.getFollowup() != null && parentMsg.getChatResponse().size() > 1) {
            // followup has a multi-part chatResponse
            List<Paragraph> pList = new ArrayList<>(parentMsg.getChatResponse());
            ChatMessage precedingMsg = ChatMessage.duplicateBasicPropertiesOf(parentMsg);
            precedingMsg.setChatResponse(pList.subList(0, pList.size() - 1));
            parentMsg.setChatResponse(pList.subList(pList.size() - 1, pList.size()));
            return Arrays.asList(precedingMsg, parentMsg);

        } else return Collections.singletonList(parentMsg);
    }
}
