package com.ibm.gbs.va.grit.repository;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.MutableLiveData;

import com.ibm.gbs.va.grit.GritApplication;
import com.ibm.gbs.va.grit.model.request.BaseRequest;
import com.ibm.gbs.va.grit.model.request.GetJournalEventsRequest;
import com.ibm.gbs.va.grit.model.response.GetAwardedBadgesResponse;
import com.ibm.gbs.va.grit.model.response.GetJournalEventsResponse;
import com.ibm.gbs.va.grit.model.response.GetMoodScoresResponse;
import com.ibm.gbs.va.grit.model.type.Badge;
import com.ibm.gbs.va.grit.model.type.BadgeListBadge;
import com.ibm.gbs.va.grit.model.type.DigitalJournalEvent;
import com.ibm.gbs.va.grit.model.type.MoodScore;
import com.ibm.gbs.va.grit.rest.GritCallback;
import com.ibm.gbs.va.grit.rest.RetrofitApiServices;

import org.joda.time.LocalDate;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class DigitalSelfRepository {
    private static final String TAG = "DigitalSelfRepository";

    private final GritApplication gritApplication;
    private final RetrofitApiServices restServices;
    private final BadgeRepository badgeRepo;

    private final MutableLiveData<List<MoodScore>> currentMoodScores = new MutableLiveData<>();
    private final MutableLiveData<List<DigitalJournalEvent>> currentEvents = new MutableLiveData<>();
    private final MutableLiveData<HashMap<Badge.BadgeCategory, List<BadgeListBadge>>> currentCategorizedBadgeList = new MutableLiveData<>();

    private final HashMap<String, LocalDate> validEventRanges = new HashMap<>();

    public DigitalSelfRepository(GritApplication gritApplication, RetrofitApiServices restServices, BadgeRepository badgeRepository) {
        this.gritApplication = gritApplication;
        this.restServices = restServices;
        this.badgeRepo = badgeRepository;
        populateEventRanges();
    }

    public LiveData<List<MoodScore>> getMoodGraphEntries() {
        restServices.getMoodScores(new BaseRequest())
                .enqueue(new GritCallback<GetMoodScoresResponse>(gritApplication) {
                    @Override
                    public void onSuccess(GetMoodScoresResponse payload) {
                        if (payload.getScores() == null) payload.setScores(new ArrayList<>());
                        currentMoodScores.setValue(payload.getScores());
                    }
                });
        return currentMoodScores;
    }

    public LiveData<List<DigitalJournalEvent>> getActivityTrackerEvents(LocalDate startDate) {
        LocalDate endDate = null;
        if (startDate != null)
            endDate = startDate.dayOfMonth().withMaximumValue();
        restServices.getJournalEvents(new GetJournalEventsRequest(startDate, endDate))
                .enqueue(new GritCallback<GetJournalEventsResponse>(gritApplication) {
                    @Override
                    public void onSuccess(GetJournalEventsResponse payload) {
                        if (payload.getEvents() == null) {
                            payload.setEvents(new ArrayList<>());
                        }
                        //make sure all eventTypes are shown
                        List<DigitalJournalEvent.EventType> returnedTypes = new ArrayList<>();
                        for (DigitalJournalEvent curEvent : payload.getEvents())
                            returnedTypes.add(curEvent.getEventType());
                        List<DigitalJournalEvent.EventType> missingTypes = DigitalJournalEvent.EventType.normalizeEvents(returnedTypes);
                        if (missingTypes.size() > 0) {
                            for (DigitalJournalEvent.EventType eventTypeToAdd : missingTypes) {
                                DigitalJournalEvent eventToAdd = new DigitalJournalEvent(eventTypeToAdd.getTypeName(), new ArrayList<>());
                                payload.getEvents().add(eventToAdd);
                            }
                        }
                        currentEvents.setValue(payload.getEvents());
                    }
                });
        return currentEvents;
    }

    public LiveData<HashMap<Badge.BadgeCategory, List<BadgeListBadge>>> getAwardedBadges() {
        restServices.getAwardedBadges(new BaseRequest())
                .enqueue(new GritCallback<GetAwardedBadgesResponse>(gritApplication) {
                    @Override
                    public void onSuccess(GetAwardedBadgesResponse payload) {
                        HashMap<Badge.BadgeCategory, List<BadgeListBadge>> badgeMap = new HashMap<>();
                        for (Badge.BadgeCategory curBadgeCategory : Badge.BadgeCategory.values())
                            if (curBadgeCategory != Badge.BadgeCategory.DEFAULT_BADGE_CATEGORY)
                                badgeMap.put(curBadgeCategory, new ArrayList<>());
                        //map returned list of strings to fullBadgeList;
                        for (Badge curBadge : badgeRepo.getAllBadges()) {
                            if (curBadge.getBadgeCategory() != Badge.BadgeCategory.DEFAULT_BADGE_CATEGORY) {
                                if (payload.getBadges().contains(curBadge.getName()))
                                    badgeMap.get(curBadge.getBadgeCategory()).add(new BadgeListBadge(curBadge, true));
                                else
                                    badgeMap.get(curBadge.getBadgeCategory()).add(new BadgeListBadge(curBadge, false));
                            }
                        }

                        currentCategorizedBadgeList.setValue(badgeMap);
                    }
                });
        return currentCategorizedBadgeList;
    }

    private void populateEventRanges() {
        LocalDate currentDate = new LocalDate();
        int validRangeWidth = 3;
        for (int k = 0; k < validRangeWidth; k++) {
            validEventRanges.put(
                    currentDate.minusMonths(k).dayOfMonth().withMinimumValue().monthOfYear().getAsText()
                    , currentDate.minusMonths(k).dayOfMonth().withMinimumValue());
        }
    }

    public HashMap<String, LocalDate> getValidEventRanges() {
        return validEventRanges;
    }
}
