package com.ibm.gbs.va.grit.repository;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.MutableLiveData;
import android.arch.lifecycle.Transformations;
import android.support.annotation.NonNull;
import android.text.TextUtils;

import com.ibm.gbs.va.grit.GritApplication;
import com.ibm.gbs.va.grit.model.notifications.Notification;
import com.ibm.gbs.va.grit.model.request.AddSavedJobRequest;
import com.ibm.gbs.va.grit.model.request.BaseRequest;
import com.ibm.gbs.va.grit.model.request.SearchSkillsRequest;
import com.ibm.gbs.va.grit.model.request.UploadResumeInfoRequest;
import com.ibm.gbs.va.grit.model.request.UploadResumeRequest;
import com.ibm.gbs.va.grit.model.request.UploadResumeRequest.UploadType;
import com.ibm.gbs.va.grit.model.response.AddSavedJobResponse;
import com.ibm.gbs.va.grit.model.response.BaseResponse;
import com.ibm.gbs.va.grit.model.response.SkillsResponse;
import com.ibm.gbs.va.grit.model.response.UploadCompleteResponse;
import com.ibm.gbs.va.grit.model.response.UploadResumeInfoResponse;
import com.ibm.gbs.va.grit.model.response.UploadResumeResponse;
import com.ibm.gbs.va.grit.model.type.EmploymentInfo;
import com.ibm.gbs.va.grit.model.type.Job;
import com.ibm.gbs.va.grit.model.type.Role;
import com.ibm.gbs.va.grit.model.type.SavedJob;
import com.ibm.gbs.va.grit.rest.GritCallback;
import com.ibm.gbs.va.grit.rest.RetrofitApiServices;
import com.ibm.gbs.va.grit.util.Consumer;
import com.ibm.gbs.va.grit.util.RemoteLog;
import com.ibm.gbs.va.grit.util.UserPrefs;

import java.util.ArrayList;
import java.util.List;

import retrofit2.Call;
import retrofit2.Response;


public class JobRepository {
    private static final String TAG = "Employment";

    private final MutableLiveData<EmploymentInfo> employmentInfo = new MutableLiveData<>();
    private final MutableLiveData<List<String>> targetIndustries = new MutableLiveData<>();
    private final MutableLiveData<List<Job>> jobs = new MutableLiveData<>();
    private final MutableLiveData<List<SavedJob>> savedJobs = new MutableLiveData<>();
    private final MutableLiveData<SkillsResponse> skills = new MutableLiveData<>();
    private final SearchSkillsRequest skillsRequest = new SearchSkillsRequest("", 0);
    private int page;

    private final RetrofitApiServices restServices;
    private final UserPrefs userPrefs;
    private final NotificationRepository messageRepo;
    private final GritApplication gritApplication;

    public JobRepository(RetrofitApiServices restServices, UserPrefs userPrefs, NotificationRepository messageRepo, GritApplication gritApplication) {
        isLoadingData.setValue(0);
        employmentInfo.observeForever(employmentInfo -> {
            skillsRequest.setTargetIndustry(employmentInfo.getTargetIndustry());
            if (employmentInfo != null && employmentInfo.getWemProfileExists() != null && employmentInfo.getWemProfileExists()) {
                setJobs();
                getSavedJobs();
                setSkills();
            } else {
                RemoteLog.d(TAG, "No wem profile exists, not updating jobs or skills");
            }
        });

        this.restServices = restServices;
        this.userPrefs = userPrefs;
        this.messageRepo = messageRepo;
        this.gritApplication = gritApplication;
        this.jobs.setValue(new ArrayList<>());
        this.savedJobs.setValue(new ArrayList<>());
        this.skills.setValue(new SkillsResponse(new ArrayList<>()));
        setEmploymentInfo(null);
        setPage(1);
    }

    private void setPage(int page) {
        this.page = page;
    }

    public LiveData<List<String>> getTargetIndustries() {
        if (targetIndustries.getValue() == null) { // only fetch once per session
            restServices.getTargetIndustries(new BaseRequest()).enqueue(new GritCallback<List<String>>(gritApplication) {
                @Override
                public void onSuccess(List<String> payload) {
                    targetIndustries.setValue(payload);
                }
            });
        }
        return targetIndustries;
    }

    public LiveData<EmploymentInfo> getEmploymentInfo() {
        return employmentInfo;
    }

    public void setEmploymentInfo(UploadResumeResponse resumeDataForUpdate) {
        restServices.getEmploymentInfo(new BaseRequest())
                .enqueue(new GritCallback<EmploymentInfo>(gritApplication) {
                    @Override
                    public void onSuccess(EmploymentInfo payload) {
                        employmentInfo.setValue(payload);
                        // check to make sure it isn't already complete to avoid excessive writing
                        // would rather read a bunch and write once instead of write a bunch
                        if (!userPrefs.getPrefJobInfoComplete()) {
                            if (payload.isComplete())
                                messageRepo.dismissNotification(Notification.Type.UPDATE_JOB_INFO);
                        }
                        if (TextUtils.isEmpty(skillsRequest.getTargetIndustry())) {
                            skillsRequest.setTargetIndustry(payload.getTargetIndustry());
                        }
                        if (resumeDataForUpdate != null && employmentInfo.getValue() != null) {
                            updateResumeInfo(resumeDataForUpdate);
                        }
                    }

                    @Override
                    public void onFailure(@NonNull Call<BaseResponse<EmploymentInfo>> call, @NonNull Throwable t) {
                        super.onFailure(call, t);
                        employmentInfo.setValue(new EmploymentInfo());
                    }
                });
    }

    public LiveData<List<Job>> getJobs() {
        return jobs;
    }

    public void setJobs() {
        if (canMakeWemCall()) {
            pushDataLoading(JOB_DATA_LOADING);
            restServices.searchJobs(new BaseRequest()).enqueue(new GritCallback<List<Job>>(gritApplication) {
                @Override
                public void onSuccess(List<Job> payload) {
                    jobs.setValue(payload);
                }

                @Override
                public void onResponse(@NonNull Call<BaseResponse<List<Job>>> call, @NonNull Response<BaseResponse<List<Job>>> response) {
                    popDataLoading(JOB_DATA_LOADING);
                    super.onResponse(call, response);
                }
            });
        }
    }

    public LiveData<SkillsResponse> getSkills() {
        return skills;
    }

    public LiveData<SkillsResponse> setSkills() {
        if (canMakeWemCall() && !TextUtils.isEmpty(skillsRequest.getTargetIndustry())) {
            pushDataLoading(SKILL_DATA_LOADING);
            restServices.searchSkills(skillsRequest).enqueue(new GritCallback<List<Role>>(gritApplication) {
                @Override
                public void onSuccess(List<Role> payload) {
                    skills.setValue(new SkillsResponse(payload));
                }

                @Override
                public void onResponse(@NonNull Call<BaseResponse<List<Role>>> call, @NonNull Response<BaseResponse<List<Role>>> response) {
                    popDataLoading(SKILL_DATA_LOADING);
                    super.onResponse(call, response);
                }
            });
        }
        return skills;
    }

    private boolean canMakeWemCall() {
        EmploymentInfo jobInfo = employmentInfo.getValue();
        return jobInfo != null && jobInfo.getWemProfileExists();
    }

    public LiveData<UploadResumeInfoResponse> updateEmploymentInfo(EmploymentInfo newEmploymentInfo) {
        MutableLiveData<UploadResumeInfoResponse> result = new MutableLiveData<>();

        RemoteLog.d(TAG, "EmploymentInfo object being updated to: " + newEmploymentInfo);
        restServices.uploadEmploymentInfo(new UploadResumeInfoRequest(newEmploymentInfo))
                .enqueue(new GritCallback<UploadResumeInfoResponse>(gritApplication) {
                    @Override
                    public void onSuccess(UploadResumeInfoResponse payload) {
                        if (payload.hasValidationErrors()) {
                            RemoteLog.d(TAG, "Invalid inputs while updating employmentInfo");
                        } else {
                            RemoteLog.d(TAG, "EmploymentInfo has been updated");
                            newEmploymentInfo.setWemProfileExists(true);
                            employmentInfo.setValue(newEmploymentInfo);
                            // check to make sure it isn't already complete to avoid excessive writing
                            // would rather read a bunch and write once instead of write a bunch
                            if (!userPrefs.getPrefJobInfoComplete()) {
                                if (newEmploymentInfo.isComplete())
                                    messageRepo.dismissNotification(Notification.Type.UPDATE_JOB_INFO);
                            }
                        }
                        result.setValue(payload);
                    }
                });
        return result;
    }

    public LiveData<Boolean> uploadedResume(UploadType uploadType, String img, String filename) {
        MutableLiveData<Boolean> result = new MutableLiveData<>();

        UploadResumeRequest uploadResumeRequest = new UploadResumeRequest(uploadType, filename, uploadType == UploadType.PHOTO ? this.page : null, img);
        restServices.uploadResume(uploadResumeRequest)
                .enqueue(new GritCallback<UploadResumeResponse>(gritApplication) {
                    @Override
                    public void onSuccess(UploadResumeResponse payload) {
                        if (uploadType != UploadType.PHOTO) {
                            updateResumeInfo(payload);
                        }
                        setPage(page + 1);
                        result.setValue(true);
                    }
                });
        return result;
    }

    public LiveData<String> uploadComplete() {
        MutableLiveData<String> result = new MutableLiveData<>();

        restServices.uploadComplete(new BaseRequest())
                .enqueue(new GritCallback<UploadCompleteResponse>(gritApplication) {
                    @Override
                    public void onSuccess(UploadCompleteResponse payload) {
                        updateResumeInfo(payload);
                        setPage(1);
                        result.setValue(payload.getReadText());
                    }
                });
        return result;
    }

    private void updateResumeInfo(UploadResumeResponse payload) {
        EmploymentInfo ei = employmentInfo.getValue();
        if (ei == null) {
            setEmploymentInfo(payload);
        } else {
            ei.setResumeFilename(payload.getResumeFilename());
            ei.setUploadTimestamp(payload.getUploadTimestamp());
            employmentInfo.setValue(ei);
            if (!userPrefs.getPrefJobInfoComplete()) {
                if (employmentInfo.getValue().isComplete())
                    messageRepo.dismissNotification(Notification.Type.UPDATE_JOB_INFO);
            }
        }
    }

    public LiveData<List<SavedJob>> getSavedJobs() {
        if (canMakeWemCall()) {
            pushDataLoading(SAVED_JOBS_DATA_LOADING);
            restServices.getSavedJobs(new BaseRequest()).enqueue(new GritCallback<List<SavedJob>>(gritApplication) {
                @Override
                public void onSuccess(List<SavedJob> payload) {
                    savedJobs.setValue(payload);
                }

                @Override
                public void onResponse(@NonNull Call<BaseResponse<List<SavedJob>>> call, @NonNull Response<BaseResponse<List<SavedJob>>> response) {
                    popDataLoading(SAVED_JOBS_DATA_LOADING);
                    super.onResponse(call, response);
                }
            });
        }
        return savedJobs;
    }

    public void addSavedJob(Job job, Consumer<Void> callback) {
        addSavedJob(new SavedJob(job), callback);
    }

    public void addSavedJob(SavedJob savedJob, Consumer<Void> callback) {
        restServices.addSavedJob(new AddSavedJobRequest(savedJob))
                .enqueue(new GritCallback<AddSavedJobResponse>(gritApplication) {
                    @Override
                    public void onSuccess(AddSavedJobResponse payload) {
                        savedJob.setId(payload.getId());

                        // save to savedJobs livedata
                        List<SavedJob> savedJobsList = savedJobs.getValue();
                        savedJobsList.add(savedJob);
                        savedJobs.setValue(new ArrayList<>(savedJobsList));

                        // save to jobs livedata
                        List<Job> jobsList = jobs.getValue();
                        for (Job item : jobsList) {
                            if (item.getJobId().equals(savedJob.getJobId())) {
                                item.setSavedJobId(payload.getId());
                                break;
                            }
                        }
                        jobs.setValue(new ArrayList<>(jobsList));

                        callback.accept(null);
                    }
                });
    }

    public void deleteSavedJob(Job job, Consumer<Void> callback) {
        List<SavedJob> savedJobsList = savedJobs.getValue();
        for (SavedJob item : savedJobsList) {
            if (TextUtils.equals(item.getJobId(), job.getJobId())) {
                deleteSavedJob(item, callback);
                return;
            }
        }
    }

    public void deleteSavedJob(SavedJob savedJob, Consumer<Void> callback) {
        restServices.deleteSavedJob(new AddSavedJobResponse(savedJob.getId()))
                .enqueue(new GritCallback<Object>(gritApplication) {
                    @Override
                    public void onSuccess(Object payload) {

                        // remove from savedJobs livedata
                        List<SavedJob> savedJobsList = savedJobs.getValue();
                        savedJobsList.remove(savedJob);
                        savedJobs.setValue(new ArrayList<>(savedJobsList));

                        // remove from jobs livedata
                        List<Job> jobsList = jobs.getValue();
                        for (Job item : jobsList) {
                            if (item.getJobId().equals(savedJob.getJobId())) {
                                item.setSavedJobId(null);
                                break;
                            }
                        }
                        jobs.setValue(new ArrayList<>(jobsList));
                        callback.accept(null);
                    }
                });
    }

    public SearchSkillsRequest getSkillsRequest() {
        return skillsRequest;
    }

    //region Jobs / Skills Data Loading
    private static final byte JOB_DATA_LOADING = 1;
    private static final byte SKILL_DATA_LOADING = 2;
    private static final byte SAVED_JOBS_DATA_LOADING = 4;

    private final MutableLiveData<Integer> isLoadingData = new MutableLiveData<>();

    @SuppressWarnings("ConstantConditions")
    private void pushDataLoading(int number) {
        isLoadingData.setValue(isLoadingData.getValue() + number);
    }

    private void popDataLoading(int number) {
        pushDataLoading(-number);
    }

    public LiveData<Boolean> isJobsOrSkillsLoading() {
        return Transformations.map(isLoadingData, num -> num != 0);
    }
    //endregion
}
