package com.ibm.gbs.va.grit.repository;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.Transformations;
import android.support.annotation.Nullable;
import android.util.ArraySet;

import com.ibm.gbs.va.grit.GritApplication;
import com.ibm.gbs.va.grit.model.notifications.Notification;
import com.ibm.gbs.va.grit.model.request.BaseRequest;
import com.ibm.gbs.va.grit.model.request.ResolveNotificationRequest;
import com.ibm.gbs.va.grit.model.response.GetNotificationsResponse;
import com.ibm.gbs.va.grit.model.type.Status;
import com.ibm.gbs.va.grit.rest.GritCallback;
import com.ibm.gbs.va.grit.rest.RetrofitApiServices;
import com.ibm.gbs.va.grit.room.NotificationDao;
import com.ibm.gbs.va.grit.util.Consumer;
import com.ibm.gbs.va.grit.util.RemoteLog;
import com.ibm.gbs.va.grit.util.UserPrefs;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.ConcurrentSkipListSet;

public class NotificationRepository {
    private static final String TAG = "NotificationRepository";

    private final RetrofitApiServices restServices;
    private final UserPrefs userPrefs;
    private final GritApplication gritApplication;
    private final NotificationDao notificationDao;

    private final String userEmail;
    private final LiveData<Notification> surveyNotification;
    private final LiveData<String> currentSurveyName;
    private Notification previousSurveySpawningNotification = null;

    public NotificationRepository(RetrofitApiServices restServices, UserPrefs userPrefs, GritApplication gritApplication, NotificationDao notificationDao) {
        this.restServices = restServices;
        this.gritApplication = gritApplication;
        this.notificationDao = notificationDao;
        this.userPrefs = userPrefs;

        userEmail = userPrefs.getCurrentUserEmail();

        surveyNotification = notificationDao.getSurveySpawningNotificationForUser(userEmail);
        currentSurveyName = Transformations.map(surveyNotification, n ->
                n != null ? n.getTitle().getEn() : null);
    }

    public void setSurveyNotification(Notification notification, Consumer<Boolean> callback) {
        notificationDao.setSurveySpawningNotificationForUser(userEmail, notification.getInternalId());
        dismissNotification(notification, dismissed -> {
            if (!dismissed) notificationDao.setSurveySpawningNotificationForUser(userEmail, null);
            callback.accept(dismissed);
        });
    }

    public void popSurveyNotification(Consumer<Boolean> callback) {
        RemoteLog.d(TAG, "Dropping Current Survey Notification...");
        notificationDao.setSurveySpawningNotificationForUser(userEmail, null);
        fetchNotifications(callback);
    }

    public LiveData<String> getCurrentSurveyName() {
        return currentSurveyName;
    }

    public Notification getNextAvailableSurveyNotification() {
        ConcurrentSkipListSet<Notification> notifications = notificationDao.getDisplayableNotificationsForUserNow(userEmail);
        Notification previousNotification = getPreviousSurveySpawningNotification();
        if (notifications == null) return null;

        if (previousNotification != null && previousNotification.getType() == Notification.Type.LIFE_CHANGES) {
            // prioritize serving oldest Baseline surveys (which are always behind Life Changes)
            for (Notification item : notifications.tailSet(previousNotification, false)) {
                if (item.getType().isSurvey())
                    return item;
            }
        }

        for (Notification item : notifications) {
            if (item.getType().isSurvey()) return item;
        }

        return null;
    }

    public Notification getPreviousSurveySpawningNotification() {
        return previousSurveySpawningNotification;
    }

    void recordPreviousSurveyNotification() {
        previousSurveySpawningNotification = notificationDao.getSurveySpawningNotificationForUserNow(userEmail);
    }

    public LiveData<ConcurrentSkipListSet<Notification>> getNotificationsToDisplay() {
        return notificationDao.getDisplayableNotificationsForUser(userEmail);
    }

    public LiveData<Notification> getSurveySpawningNotificationForUser() {
        return surveyNotification;
    }

    public void fetchNotifications(Consumer<Boolean> callback) {
        callback = callback != null ? callback : c -> {
        };
        Consumer<Boolean> finalCallback = callback;
        restServices.getNotifications(new BaseRequest())
                .enqueue(new GritCallback<GetNotificationsResponse>(gritApplication) {
                    @Override
                    public void onSuccess(GetNotificationsResponse payload) {
                        ArraySet<Notification> set = new ArraySet<>();
                        set.addAll(payload.getNotifications());
                        if (!userPrefs.getPrefInitialBaselineComplete())
                            for (Notification item : set) {
                                if (item.getType().isSurvey()) {
                                    userPrefs.setPrefInitialBaselineComplete(true);
                                    break;
                                }
                            }
                        set.addAll(checkLocalNotifications());
                        notificationDao.saveNotificationsForUser(new ArrayList<>(set), userEmail);

                        Notification baseline = getNotificationFromType(Notification.Type.BASELINE_INITIAL);
                        if (notificationDao.getSurveySpawningNotificationForUserNow(userEmail) == null && baseline != null) {
                            notificationDao.setSurveySpawningNotificationForUser(userEmail, baseline.getInternalId());
                        }
                        finalCallback.accept(true);
                    }
                });
    }

    Notification getSurveyNotificationNow() {
        return notificationDao.getSurveySpawningNotificationForUserNow(userEmail);
    }

    private List<Notification> checkLocalNotifications() {
        LinkedList<Notification> list = new LinkedList<>();

        if (!userPrefs.getPrefJobInfoComplete())
            list.add(Notification.localJobInfoNotification(gritApplication));
        if (!userPrefs.getPrefProfileComplete())
            list.add(Notification.localProfileInfoNotification(gritApplication));
        if (!userPrefs.getPrefSquadComplete())
            list.add(Notification.localSquadInfoNotification(gritApplication));
        if (!userPrefs.getPrefInitialBaselineComplete())
            list.add(Notification.localBaselineInfoNotification(gritApplication));
        if (userPrefs.getPrefInitialBaselineComplete() && notificationDao.getSurveySpawningNotificationForUserNow(userEmail) != null)
            list.add(Notification.localContinueSurveyNotification(gritApplication));
        return list;
    }


    public void dismissNotification(Notification notificationToDismiss, Consumer<Boolean> callback) {
        callback = callback != null ? callback : c -> {
        };
        if (notificationToDismiss == null) {
            //Cannot dismiss notification that does not exist, so we do nothing
            return;
        }
        switch (notificationToDismiss.getType()) {
            case BASELINE:
            case MOOD_FACES:
            case MOOD_TEXT:
            case LIFE_CHANGES:
            case DIGITAL_ASSISTANT:
            case FEEDBACK:
            case IMMEDIATE_HELP:
            case DEFAULT_NOTIFICATION_TYPE:
                Consumer<Boolean> finalCallback = callback;
                restServices.resolveNotification(new ResolveNotificationRequest(notificationToDismiss.getNotificationId()))
                        .enqueue(new GritCallback<Object>(gritApplication) {
                            @Override
                            public void onSuccess(Object payload) {
                                RemoteLog.d(TAG, "Notification: " + notificationToDismiss.getType() + " has been successfully dismissed");
                                fetchNotifications(finalCallback);
                            }

                            @Override
                            public void onError(Status.StatusType statusType, String statusMessage, int displayMessage) {
                                super.onError(statusType, statusMessage, displayMessage);
                                finalCallback.accept(false);
                            }
                        });
                return;
            case UPDATE_SQUAD:
                userPrefs.setPrefSquadComplete(true);
                break;
            case UPDATE_PROFILE:
                userPrefs.setPrefProfileComplete(true);
                break;
            case UPDATE_JOB_INFO:
                userPrefs.setPrefJobinfoComplete(true);
                break;
        }
        fetchNotifications(callback);
    }

    public void dismissNotification(Notification.Type type) {
        dismissNotification(getNotificationFromType(type), null);
    }

    @Nullable
    private Notification getNotificationFromType(Notification.Type type) {
        ConcurrentSkipListSet<Notification> list = notificationDao.getNotificationsForUserNow(userEmail);
        if (list == null) return null;
        for (Notification item : list) {
            if (item.getType() == type) {
                return item;
            }
        }
        return null;
    }

    public boolean isOngoingSurvey() {
        return surveyNotification.getValue() != null;
    }
}
