package com.ibm.gbs.va.grit.repository;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.MutableLiveData;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.util.Pair;

import com.ibm.gbs.va.grit.GritApplication;
import com.ibm.gbs.va.grit.model.notifications.Notification;
import com.ibm.gbs.va.grit.model.request.GetQuestionSetRequest;
import com.ibm.gbs.va.grit.model.request.SubmitSurveyRequest;
import com.ibm.gbs.va.grit.model.response.BaseResponse;
import com.ibm.gbs.va.grit.model.response.SubmitSurveyResponse;
import com.ibm.gbs.va.grit.model.survey.Answer;
import com.ibm.gbs.va.grit.model.survey.Question;
import com.ibm.gbs.va.grit.model.survey.QuestionSet;
import com.ibm.gbs.va.grit.model.type.Badge;
import com.ibm.gbs.va.grit.model.type.Status;
import com.ibm.gbs.va.grit.rest.GritCallback;
import com.ibm.gbs.va.grit.rest.RetrofitApiServices;
import com.ibm.gbs.va.grit.room.AnswerDao;
import com.ibm.gbs.va.grit.room.AnswerDao.QuestionAnswerJoin;
import com.ibm.gbs.va.grit.room.QSetDao;
import com.ibm.gbs.va.grit.util.Consumer;
import com.ibm.gbs.va.grit.util.RemoteLog;

import java.util.ArrayList;
import java.util.List;

import retrofit2.Call;
import retrofit2.Response;


public class SurveyRepository {
    private static final String TAG = "SurveyRepository";

    private final QSetDao qSetDao;
    private final AnswerDao answerDao;
    private final BadgeRepository badgeRepo;
    private final NotificationRepository notificationRepo;
    private final RetrofitApiServices restServices;
    private final UserRepository userRepo;
    private final GritApplication gritApplication;

    private boolean newSession = true;

    private final LiveData<List<QuestionAnswerJoin>> questionsAndAnswers;

    private final MutableLiveData<Boolean> qSetNotComplete = new MutableLiveData<>();

    public SurveyRepository(RetrofitApiServices restServices, BadgeRepository badgeRepository, QSetDao qSetDao, AnswerDao answerDao, GritApplication gritApplication, NotificationRepository notificationRepo, UserRepository userRepository) {
        this.restServices = restServices;
        this.badgeRepo = badgeRepository;
        this.qSetDao = qSetDao;
        this.answerDao = answerDao;
        this.userRepo = userRepository;
        this.notificationRepo = notificationRepo;
        this.gritApplication = gritApplication;

        questionsAndAnswers = answerDao.getQuestionsAndAnswersForUser(userRepository.getCurrentUserEmail());
        questionsAndAnswers.observe(gritApplication, questsAndAns -> {
        });
    }

    private void getQuestionSetFromServer(@Nullable Consumer<Boolean> callback) {
        final Consumer<Boolean> finalCallback = callback != null ? callback : c -> {
        };
        qSetNotComplete.setValue(null);

        // do nothing if qSet already cached
        QuestionSet qSet = qSetDao.getQSetForUserNow(userRepo.getCurrentUserEmail());
        if (qSet != null) {
            finalCallback.accept(true);
            return;
        }

        // assert a survey Notification exists
        Notification notification = notificationRepo.getSurveyNotificationNow();
        if (notification == null) {
            notificationRepo.fetchNotifications(result -> {
                Notification n = notificationRepo.getSurveyNotificationNow();
                if (result && n != null)
                    getQuestionSetFromServer(finalCallback);
                else finalCallback.accept(false);
            });
            return;
        }

        restServices.getQuestionSet(new GetQuestionSetRequest(newSession, notification)).enqueue(new GritCallback<QuestionSet>(gritApplication) {
            @Override
            public void onSuccess(QuestionSet payload) {
                qSetDao.saveQuestionSet(payload);
                userRepo.setQSetForUser(payload.getQuestionSetId(), notification.getInternalId());

                qSetNotComplete.setValue(true);
                finalCallback.accept(true);
            }

            @Override
            public void onError(Status.StatusType statusType, String statusMessage, int displayMessage) {
                super.onError(statusType, statusMessage, displayMessage);

                qSetNotComplete.setValue(false);
                if (statusType == Status.StatusType.NO_QUESTIONS_AVAILABLE)
                    userRepo.setQSetForUser(null, null);
                finalCallback.accept(false);
            }
        });
    }

    //region Submit Survey

    /**
     * @param callback returns a pair where:
     *                 <p>
     *                 The 1st element represents <code>hasAdditionalQuestions</code> & is
     *                 <code>null</code> on error
     *                 <p>
     *                 The 2nd element is any badge that was received as a result of the call
     */
    public void submitQuestionSet(Consumer<Pair<Boolean, Badge>> callback) {
        String userEmail = userRepo.getCurrentUserEmail();
        qSetNotComplete.setValue(null);
        QuestionSet questionResponse = qSetDao.getQSetForUserNow(userEmail);
        List<Answer> answers = answerDao.getAnswersForUserNow(userEmail);
        assert questionResponse != null && questionsAndAnswers.getValue() != null;

        // check for refused questions with variants
        List<QuestionAnswerJoin> variantJoins = new ArrayList<>();
        for (QuestionAnswerJoin join : questionsAndAnswers.getValue()) {
            Answer answer = join.getAnswer();
            List<Integer> variantList = join.getQuestion().getVariantIdList();
            int currentIndex = variantList.indexOf(answer.getQuestionVariantId());
            if (answer.isRefused() && currentIndex < variantList.size() - 1) {
                answer.setQuestionVariantId(variantList.get(currentIndex + 1));
                variantJoins.add(join);
            }
        }
        if (!variantJoins.isEmpty()) {
            newSession = false;
            qSetNotComplete.setValue(true);
            updateAnswers(variantJoins);
            callback.accept(new Pair<>(true, null));
        } else
            callSubmitSurvey(new SubmitSurveyRequest(questionResponse, false, answers),
                    callback);
    }

    /**
     * @param abortCallback returns a pair where:
     *                      <p>
     *                      The 1st element represents if the survey was aborted as expected & is
     *                      <code>null</code> on error
     *                      <p>
     *                      The 2nd element is any badge that was received as a result of the call
     */
    public void abortCurrentSurvey(@NonNull Consumer<Pair<Boolean, Badge>> abortCallback) {
        String userEmail = userRepo.getCurrentUserEmail();
        qSetNotComplete.setValue(null);
        // initial setup to ready the answers for submitting (Answers default to refused state)
        QuestionSet questionResponse = qSetDao.getQSetForUserNow(userEmail);
        List<Answer> answers = answerDao.getAnswersForUserNow(userEmail);
        if (answers.isEmpty()) { // should only occur if there was an error submitting questions before
            for (Question curQuest : questionResponse.getQuestions()) {
                answers.add(new Answer(curQuest));
            }
        }
        assert questionResponse != null;

        // submit the survey in it's aborted state
        // will notify the callback with appropriate result at end of chain
        callSubmitSurvey(new SubmitSurveyRequest(questionResponse, true, answers), submitSurveyPair -> {
            if (submitSurveyPair.first != null) {
                if (submitSurveyPair.first)
                    RemoteLog.e(TAG, "aborted survey returned more question sets available");
                else
                    RemoteLog.d(TAG, "Submitted all question sets to abort, returning to dashboard");
            }
            Boolean abortResult = submitSurveyPair.first != null ? !submitSurveyPair.first : null;
            abortCallback.accept(new Pair<>(abortResult, submitSurveyPair.second));
        });
    }

    private void callSubmitSurvey(SubmitSurveyRequest request, @NonNull Consumer<Pair<Boolean, Badge>> callback) {
        notificationRepo.recordPreviousSurveyNotification();
        final Badge[] receivedBadge = {null};
        restServices.submitSurvey(request).enqueue(new GritCallback<SubmitSurveyResponse>(gritApplication) {
            @Override
            public void onResponse(@NonNull Call<BaseResponse<SubmitSurveyResponse>> call, @NonNull Response<BaseResponse<SubmitSurveyResponse>> response) {
                assert response.body() != null;
                List<Badge> badgeAward = badgeRepo.getBadgeListFromStrings(response.body().getBadgeAward());
                if (!badgeAward.isEmpty()) {
                    //process badge
                    receivedBadge[0] = badgeAward.get(0);
                    response.body().setBadgeAward(new ArrayList<>());
                }
                super.onResponse(call, response);
            }

            @Override
            public void onSuccess(SubmitSurveyResponse payload) {
                newSession = false;
                boolean hasAdditionalQuestions = payload.isAdditionalQuestions();
                unloadData(hasAdditionalQuestions); // Empty out preferences and data
                qSetNotComplete.setValue(hasAdditionalQuestions);
                newSession = !hasAdditionalQuestions;

                userRepo.setOnboardingInitialBaselineComplete(true);
                callback.accept(new Pair<>(hasAdditionalQuestions, receivedBadge[0]));
            }

            @Override
            public void onFailure(@NonNull Call<BaseResponse<SubmitSurveyResponse>> call, @NonNull Throwable t) {
                super.onFailure(call, t);
                qSetNotComplete.setValue(false);
                callback.accept(new Pair<>(null, receivedBadge[0]));
            }

            @Override
            public void onError(Status.StatusType statusType, String statusMessage, int displayMessage) {
                super.onError(statusType, statusMessage, displayMessage);
                if (statusType == Status.StatusType.INVALID_REQUEST) {
                    unloadData(false);
                }
                RemoteLog.d(TAG, "Failure when submitting survey");
                callback.accept(new Pair<>(null, receivedBadge[0]));
            }
        });
    }
    //endregion

    private void unloadData(boolean setHasAdditionalQuestions) {
        String userEmail = userRepo.getCurrentUserEmail();
        QuestionSet set = qSetDao.getQSetForUserNow(userEmail);
        answerDao.deleteAnswers(answerDao.getAnswersForUserNow(userEmail));
        userRepo.setQSetForUser(null, setHasAdditionalQuestions ?
                notificationRepo.getSurveyNotificationNow().getInternalId() :
                null);
        qSetDao.removeQuestionSet(set);
        RemoteLog.d(TAG, "answers&Questions: " + this.questionsAndAnswers.getValue());
        userRepo.setUserInProgressPage(0);
    }

    public LiveData<List<QuestionAnswerJoin>> getQuestionsAndAnswers() {
        RemoteLog.d(TAG, "answers&Questions: " + this.questionsAndAnswers.getValue());
        return questionsAndAnswers;
    }

    public LiveData<Boolean> getQuestionSetNotComplete() {
        return qSetNotComplete;
    }

    public void updateAnswer(Answer answer) {
        answer.setUserEmail(userRepo.getCurrentUserEmail());
        answerDao.saveAnswer(answer);
    }

    public void updateAnswers(List<QuestionAnswerJoin> list) {
        answerDao.setQuestionsAndAnswersForUser(list, userRepo.getCurrentUserEmail());
    }

    public void updateCurrentQuestionSet(Consumer<Boolean> updateCallback) {
        getQuestionSetFromServer(updateCallback);
    }
}
