package com.ibm.gbs.va.grit.repository;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.MutableLiveData;
import android.arch.lifecycle.Transformations;
import android.text.TextUtils;

import com.ibm.gbs.va.grit.GritApplication;
import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.model.notifications.Notification;
import com.ibm.gbs.va.grit.model.request.BaseRequest;
import com.ibm.gbs.va.grit.model.request.ExistsRequest;
import com.ibm.gbs.va.grit.model.request.ForgotPasswordRequest;
import com.ibm.gbs.va.grit.model.request.ResetPasswordRequest;
import com.ibm.gbs.va.grit.model.request.SignInRequest;
import com.ibm.gbs.va.grit.model.request.SignUpRequest;
import com.ibm.gbs.va.grit.model.request.UpdatePasswordRequest;
import com.ibm.gbs.va.grit.model.response.ExistsResponse;
import com.ibm.gbs.va.grit.model.response.ForgotPasswordResponse;
import com.ibm.gbs.va.grit.model.response.GetUserResponse;
import com.ibm.gbs.va.grit.model.response.SignInResponse;
import com.ibm.gbs.va.grit.model.response.TokenResponse;
import com.ibm.gbs.va.grit.model.type.User;
import com.ibm.gbs.va.grit.rest.GritCallback;
import com.ibm.gbs.va.grit.rest.RetrofitApiServices;
import com.ibm.gbs.va.grit.room.UserDao;
import com.ibm.gbs.va.grit.util.Keystore;
import com.ibm.gbs.va.grit.util.RemoteLog;
import com.ibm.gbs.va.grit.util.UserPrefs;


public class UserRepository {
    private static final String TAG = "UserRepository";

    private final LiveData<User> signedInUser;
    private final MutableLiveData<String> signedInUserEmail = new MutableLiveData<>();

    private final RetrofitApiServices restServices;
    private final UserPrefs userPrefs;
    private final AuthorizationRepository authorizationRepo;
    private final Keystore keystore;
    private final UserDao userDao;
    private final NotificationRepository messageRepo;
    private final GritApplication gritApplication;


    public UserRepository(RetrofitApiServices restServices, UserPrefs userPrefs, Keystore keystore, UserDao userDao, NotificationRepository messageRepo, GritApplication gritApplication, AuthorizationRepository authorizationRepo) {
        this.restServices = restServices;
        this.gritApplication = gritApplication;
        this.userPrefs = userPrefs;
        this.authorizationRepo = authorizationRepo;
        this.keystore = keystore;
        this.userDao = userDao;
        this.messageRepo = messageRepo;

        signedInUserEmail.setValue(authorizationRepo.getCurrentUserEmail());
        signedInUser = Transformations.switchMap(signedInUserEmail, userDao::getUser);
    }

    private void setUserPrefsForLogin(String email, String token, String name) {
        setCurrentUserEmail(email);
        setUserEmail(email);
        setUserLoggedIn(true);
        setUserHasLoggedIn(true);
        setUserToken(token);
        if (name != null && !name.isEmpty()) {
            setUserName(name);
        }
    }

    public LiveData<Boolean> signUp(final String name, final String email, final String password) {
        MutableLiveData<Boolean> result = new MutableLiveData<>();
        restServices.exists(new ExistsRequest(email))
                .enqueue(new GritCallback<ExistsResponse>(gritApplication) {
                    @Override
                    public void onSuccess(ExistsResponse payload) {
                        if (payload.getExist()) {
                            gritApplication.handleError(R.string.error_failed_login);
                            result.setValue(false);
                        } else {
                            restServices.signUp(new SignUpRequest(email, password, name))
                                    .enqueue(new GritCallback<TokenResponse>(gritApplication) {
                                        @Override
                                        public void onSuccess(TokenResponse payload) {
                                            saveNewUser(email, name, payload.getToken());
                                            setUserPrefsForLogin(email, payload.getToken(), name);
                                            result.setValue(true);
                                        }
                                    });
                        }
                    }
                });
        return result;
    }

    public LiveData<Boolean> login(final String email, final String password) {
        MutableLiveData<Boolean> result = new MutableLiveData<>();
        restServices.signIn(new SignInRequest(email, password))
                .enqueue(new GritCallback<SignInResponse>(gritApplication) {
                    @Override
                    public void onSuccess(SignInResponse payload) {
                        saveUserToken(email, payload.getToken());
                        setUserPrefsForLogin(email, payload.getToken(), payload.getName());
                        result.setValue(true);
                    }
                });
        return result;
    }

    /**
     * @param tempPassword The user's temporary password
     * @param newPassword  The user's new password
     * @return LiveData will be null until the current user's token is non-null
     */
    public LiveData<Boolean> resetTempPassword(String tempPassword, String newPassword) {
        MutableLiveData<Boolean> result = new MutableLiveData<>();
        String email = getCurrentUserEmail();
        restServices.resetPassword(new ResetPasswordRequest(email, tempPassword, newPassword))
                .enqueue(new GritCallback<SignInResponse>(gritApplication) {
                    @Override
                    public void onSuccess(SignInResponse payload) {
                        saveUserToken(email, payload.getToken());
                        setUserPrefsForLogin(email, payload.getToken(), payload.getName());
                        result.setValue(true);
                    }
                });
        return result;
    }

    public LiveData<User> getUserInfo() {
        if (signedInUserEmail.getValue() == null) {
            signedInUserEmail.setValue(authorizationRepo.getCurrentUserEmail());
        }
        if (signedInUser.getValue() == null) {
            restServices.getUser(new BaseRequest())
                    .enqueue(new GritCallback<GetUserResponse>(gritApplication) {
                        @Override
                        public void onSuccess(GetUserResponse payload) {
                            userDao.saveUserPersonalInfo(payload.getUser());
                            if (signedInUserEmail.getValue().isEmpty() || signedInUserEmail.getValue().equals(payload.getUser().getEmail())) {
                                //same user, so we want to make sure the UI is up to date with the user object. We do this to notify any observers
                                signedInUserEmail.setValue(payload.getUser().getEmail());
                            } else {
                                // This would cause all sorts of problems, if the server returned information for a different user
                                throw new RuntimeException("Server returned: " + payload.getUser().getEmail() + " info instead of: " + signedInUserEmail.getValue() + " info.");
                            }
                            // check to make sure it isn't already complete to avoid excessive writing
                            // would rather read a bunch and write once instead of write a bunch
                            if (!getOnboardingProfileComplete()) {
                                if (payload.getUser().isComplete())
                                    messageRepo.dismissNotification(Notification.Type.UPDATE_PROFILE);
                            }
                        }
                    });
            return signedInUser;
        }
        return signedInUser;
    }

    public LiveData<UPDATE_STATE> updateUser(User user) {
        String newEmail = user.getEmail(),
                oldEmail = signedInUserEmail.getValue();
        MutableLiveData<UPDATE_STATE> result = new MutableLiveData<>();

        RemoteLog.d(TAG, "User object being updated to: " + user);
        assert signedInUser.getValue() != null;
        restServices.updateProfile(user)
                .enqueue(new GritCallback<TokenResponse>(gritApplication) {
                    @Override
                    public void onSuccess(TokenResponse payload, boolean badgeAwarded) {
                        RemoteLog.d(TAG, "user has been updated");
                        if (!oldEmail.equals(newEmail)) {
                            userPrefs.renamePref(oldEmail, newEmail);
                            userPrefs.setCurrentUserEmail(newEmail);
                            userPrefs.setUserEmail(newEmail);
                            userPrefs.setUserBiometricsEmail(newEmail);
                        }
                        userPrefs.setUserToken(payload.getToken());
                        user.setToken(payload.getToken());
                        userDao.saveUserPersonalInfo(user);
                        if (badgeAwarded) result.setValue(UPDATE_STATE.UPDATED_WITH_BADGE);
                        else result.setValue(UPDATE_STATE.UPDATED_NO_BADGE);
                        // check to make sure it isn't already complete to avoid excessive writing
                        // would rather read a bunch and write once instead of write a bunch
                        if (!getOnboardingProfileComplete() && user.isComplete()) {
                            messageRepo.dismissNotification(Notification.Type.UPDATE_PROFILE);
                        }
                    }

                    @Override
                    public void onSuccess(TokenResponse payload) {
                        // Not needed, we handle above
                    }
                });
        return result;
    }

    public enum UPDATE_STATE {
        UPDATED_NO_BADGE, UPDATED_WITH_BADGE, UPDATED_FAILED
    }


    private void saveUserToken(String email, String token) {
        userDao.saveUserToken(email, token);
    }

    private void saveNewUser(String email, String name, String token) {
        User user = new User(email, token);
        user.setName(name);
        userDao.insertUser(user);
    }

    public void setQSetForUser(String qSetId, Long notificationId) {
        userDao.setQSetForUser(signedInUserEmail.getValue(), qSetId, notificationId);
    }

    private LiveData<String> getTokenForUser(String email) {
        LiveData<User> user = userDao.getUser(email);
        return Transformations.map(user, User::getToken);
    }

    public LiveData<String> getTokenForCurrentUser() {
        assert signedInUser.getValue() != null;
        return getTokenForUser(signedInUser.getValue().getEmail());
    }

    //region Change/Forgot Password
    public LiveData<Boolean> updateUserPassword(String oldPw, String newPw) {
        MutableLiveData<Boolean> result = new MutableLiveData<>();

        assert signedInUser.getValue() != null;
        RemoteLog.d(TAG, "User updating password");
        restServices.updatePassword(new UpdatePasswordRequest(oldPw, newPw))
                .enqueue(new GritCallback<TokenResponse>(gritApplication) {
                    @Override
                    public void onSuccess(TokenResponse payload) {
                        RemoteLog.d(TAG, "password has been updated");
                        String token = payload.getToken();
                        if (!TextUtils.isEmpty(token)) {
                            setUserToken(token);
                            saveUserToken(signedInUserEmail.getValue(), payload.getToken());
                            keystore.encryptAndSaveToUserPrefs(newPw, R.string.USERPREF_ENCRYPTED_PASSWORD_KEY_ALIAS, R.string.USERPREF_ENCRYPTED_PASSWORD_CIPHER_IV_KEY_ALIAS);
                            result.setValue(true);
                        } else {
                            result.setValue(false);
                        }
                    }
                });
        return result;
    }

    public LiveData<User> callForgotPassword(String email) {
        signUserOut();
        // assert no user is signed in
        if (!TextUtils.isEmpty(signedInUserEmail.getValue())) throw new AssertionError();

        // call api
        restServices.forgotPassword(new ForgotPasswordRequest(email)).enqueue(new GritCallback<ForgotPasswordResponse>(gritApplication) {
            @Override
            public void onSuccess(ForgotPasswordResponse payload) {
                // set current user
                setCurrentUserEmail(email);
                setUserEmail(email);
                setCurrentBiometricsEmail(email);
                saveUserToken(email, null);
                signedInUserEmail.setValue(getCurrentUserEmail());
            }
        });

        return signedInUser;
    }
    //endregion

    public void signUserOut() {
        userPrefs.setUserLoggedIn(false);
        signedInUserEmail.setValue(null);
    }

    public LiveData<Boolean> deactivateAccount() {
        MutableLiveData<Boolean> result = new MutableLiveData<>();
        RemoteLog.d(TAG, "Deactivating account");
        String userEmail = getCurrentUserEmail();

        // data purge remotely
        restServices.deleteAccount(new BaseRequest())
                .enqueue(new GritCallback<Object>(gritApplication) {
                    @Override
                    public void onSuccess(Object payload) {
                        // data purge locally
                        gritApplication.purgeUserData(userEmail);
                        userPrefs.deletePref(userEmail);
                        result.setValue(true);
                    }
                });
        return result;
    }

    ////////////////////////
    // Directly to UserPrefs

    // Current User Email (Defines User Scope as well)
    public String getCurrentUserEmail() {
        return authorizationRepo.getCurrentUserEmail();
    }

    public void setCurrentUserEmail(String newCurrentUserEmail) {
        authorizationRepo.setCurrentUserEmail(newCurrentUserEmail);
    }

    //region User Flags
    public boolean getUserAcceptedPolicy() {
        return userPrefs.getGlobalAcceptedPolicy();
    }

    public void setUserAcceptedPolicy(boolean flag) {
        userPrefs.setGlobalAcceptedPolicy(flag);
    }

    public boolean getUserLoggedIn() {
        return userPrefs.getUserLoggedIn();
    }

    private void setUserLoggedIn(boolean flag) {
        userPrefs.setUserLoggedIn(flag);
    }

    public boolean getUserHasLoggedIn() {
        return userPrefs.getUserHasLoggedIn();
    }

    private void setUserHasLoggedIn(boolean flag) {
        userPrefs.setUserHasLoggedIn(flag);
    }

    public int getUserInProgressPage() {
        return userPrefs.getInProgressPage();
    }

    public void setUserInProgressPage(int num) {
        userPrefs.setInProgressPage(num);
    }
    //endregion

    //region Onboarding Flags
    public boolean getOnboardingJobInfoComplete() {
        return userPrefs.getPrefJobInfoComplete();
    }

    public void setOnboardingJobInfoComplete(boolean flag) {
        userPrefs.setPrefJobinfoComplete(flag);
    }

    public boolean getOnboardingSquadComplete() {
        return userPrefs.getPrefSquadComplete();
    }

    public void setOnboardingSquadComplete(boolean flag) {
        userPrefs.setPrefSquadComplete(flag);
    }

    private boolean getOnboardingProfileComplete() {
        return userPrefs.getPrefProfileComplete();
    }

    public void setOnboardingProfileComplete(boolean flag) {
        userPrefs.setPrefProfileComplete(flag);
    }

    public boolean getOnboardingInitialBaselineComplete() {
        return userPrefs.getPrefInitialBaselineComplete();
    }

    public void setOnboardingInitialBaselineComplete(boolean flag) {
        userPrefs.setPrefInitialBaselineComplete(flag);
    }

    public boolean getOnboardingSurveyInfoComplete() {
        return userPrefs.getPrefSurveyInfo();
    }

    public void setOnboardingSurveyInfoComplete(boolean flag) {
        userPrefs.setPrefSurveyInfo(flag);
    }
    //endregion

    //region Info button auto-open flags
    public boolean getOnboardingJobInfoInfo() {
        return userPrefs.getPrefJobinfoInfo();
    }

    public void setOnboardingJobInfoInfo(boolean flag) {
        userPrefs.setPrefJobinfoInfo(flag);
    }

    public boolean getOnboardingSquadInfo() {
        return userPrefs.getPrefSquadInfo();
    }

    public void setOnboardingSquadInfo(boolean flag) {
        userPrefs.setPrefSquadInfo(flag);
    }

    public boolean getOnboardingSkillsInfo() {
        return userPrefs.getPrefSkillsInfo();
    }

    public void setOnboardingSkillsInfo(boolean flag) {
        userPrefs.setPrefSkillsInfo(flag);
    }

    public boolean getOnboardingDigitalAssistInfo() {
        return userPrefs.getPrefDigitalAssistInfo();
    }

    public void setOnboardingDigitalAssistInfo(boolean flag) {
        userPrefs.setPrefDigitalAssistInfo(flag);
    }

    public boolean getOnboardingDigitalJournalInfo() {
        return userPrefs.getPrefDsDigitaljournalInfo();
    }

    public void setOnboardingDigitalJournalInfo(boolean flag) {
        userPrefs.setPrefDsDigitaljournalInfo(flag);
    }
    //endregion

    public String getCurrentBiometricsEmail() {
        return userPrefs.getUserBiometricsEmail();
    }

    public void setCurrentBiometricsEmail(String biometricsEmail) {
        userPrefs.setUserBiometricsEmail(biometricsEmail);
    }

    public String getUserEmail() {
        return userPrefs.getUserEmail();
    }

    private void setUserEmail(String newUserEmail) {
        userPrefs.setUserEmail(newUserEmail);
    }

    public String getUserName() {
        return userPrefs.getUserName();
    }

    private void setUserName(String newUserName) {
        userPrefs.setUserName(newUserName);
    }

    public String getUserToken() {
        return authorizationRepo.getCurrentUserToken();
    }

    private void setUserToken(String newToken) {
        authorizationRepo.setCurrentUserToken(newToken);
    }

}
