package com.ibm.gbs.va.grit.rest;

import android.support.annotation.NonNull;

import com.ibm.gbs.va.grit.GritApplication;
import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.model.response.BaseResponse;
import com.ibm.gbs.va.grit.model.type.Status;
import com.ibm.gbs.va.grit.repository.BadgeRepository;
import com.ibm.gbs.va.grit.util.BadgeDelegate;
import com.ibm.gbs.va.grit.util.RemoteLog;

import java.util.List;

import okhttp3.Request;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public abstract class GritCallback<T> implements Callback<BaseResponse<T>> {
    private final String TAG = "GritCallback";

    // ALL parameters you want to be injected must be injected through @GritCallbackInjectionHelper
    // Injected
    private final NetworkErrorDelegate errorDelegate;
    // Injected
    private final BadgeDelegate badgeDelegate;
    // Injected
    private final BadgeRepository badgeRepository;

    public GritCallback(GritApplication gritApplication) {

        // Wrapper class for our injected parameters
        GritCallbackInjectionHelper parameterHolder = new GritCallbackInjectionHelper();
        //inject the desired parameters into our wrapper class
        gritApplication.getCurApplicationProvider().injectGritCallbackHelper(parameterHolder);
        //get the injected parameters we wanted from the wrapper
        this.errorDelegate = parameterHolder.errorDelegate;
        this.badgeDelegate = parameterHolder.badgeDelegate;
        this.badgeRepository = parameterHolder.badgeRepository;

    }
/*
    public GritCallback(@Nullable NetworkErrorDelegate networkErrorDelegate, BadgeRepository badgeRepository) {
        this.errorDelegate = networkErrorDelegate;
        this.badgeRepository = badgeRepository;
    }
/*
    public GritCallback(@Nullable NetworkErrorDelegate networkErrorDelegate,BadgeRepository badgeRepository, Consumer<Badge> customBadgeDisplay) {
        this.errorDelegate = networkErrorDelegate;
        //this.customBadgeDisplay = customBadgeDisplay;
        this.badgeRepository=badgeRepository;
    }*/

    /**
     * Invoked for a received HTTP response.
     * <p>
     * Note: An HTTP response may still indicate an application-level failure such as a 404 or 500.
     * Call {@link Response#isSuccessful()} to determine if the response indicates success. Inspect
     * response.body().getStatus().getStatusTypeEnum() to determine if data is good Call {@link
     * GritCallback#onSuccess(T)} if data is good
     *
     * @param call
     * @param response
     */
    @Override
    public void onResponse(@NonNull Call<BaseResponse<T>> call, @NonNull Response<BaseResponse<T>> response) {
        Request request = call.request();
        RemoteLog.d(TAG, "Response received for: " + getApiName(call));
        RemoteLog.d(TAG, "Request URL: " + request.url().toString());
        RemoteLog.d(TAG, "Request Headers: " + request.headers().toString());
        RemoteLog.d(TAG, "Request Body: " + RetrofitInterceptor.requestBodyToString(request.body()));

        if (response.body() != null) {
            RemoteLog.d(TAG, "Response: " + response.body());
        }

        if (response.body() != null && response.body().getStatus() != null && response.body().getStatus().getStatusTypeEnum() != null) {
            Status.StatusType statusType = response.body().getStatus().getStatusTypeEnum();
            String statusMessage = response.body().getStatus().getErrorMessage();
            switch (statusType) {
                case SUCCESS:
                    badgeDelegate.handleBadges(badgeRepository.getBadgeListFromStrings(response.body().getBadgeAward()));
                    onSuccess(response.body().getPayload(), (badgeRepository.getBadgeListFromStrings(response.body().getBadgeAward()).size() > 0));
                    break;
                case LOGIN_FAILED:
                    onError(statusType, statusMessage, R.string.error_failed_login);
                    break;
                case NO_QUESTIONS_AVAILABLE:
                    onError(statusType, statusMessage, R.string.no_questions_available);
                    break;
                case EMAIL_IN_USE:
                    onError(statusType, statusMessage, R.string.email_in_use_error);
                    break;
                case OFFLINE:
                    onError(statusType, statusMessage, R.string.no_connectivity);
                    break;
                case SQUAD_MATE_ALREADY_EXISTS:
                    onError(statusType, statusMessage, R.string.squad_mate_already_exists);
                    break;
                case SQUAD_MATES_MAX_REACHED:
                    onError(statusType, statusMessage, R.string.squad_mates_max_reached);
                    break;
                case HANDLE_NOT_VALID:
                    onError(statusType, statusMessage, R.string.twitter_handle_not_valid);
                    break;
                case NOT_UNIQUE:
                    onError(statusType, statusMessage, R.string.twitter_handle_not_unique);
                    break;
                case OLD_PASSWORD_MISMATCH:
                    onError(statusType, statusMessage, R.string.error_incorrect_password);
                    break;
                case TEMP_PASSWORD_MISMATCH:
                    onError(statusType, statusMessage, R.string.reset_password_error);
                    break;
                case NO_WEM_PROFILE:
                    onError(statusType, statusMessage, R.string.no_wem_profile);
                    break;
                case EMAIL_NOT_FOUND:
                    onError(statusType, statusMessage, R.string.email_not_found);
                    break;
                default:
                    onError(statusType, statusMessage, R.string.network_error);
                    break;
            }
        } else if (response.body() == null) {
            //No Body
            onError(Status.StatusType.UNEXPECTED_EXCEPTION, "No Body Exists", R.string.network_error);
        } else if (response.body().getStatus() == null) {
            //No Status
            onError(Status.StatusType.UNEXPECTED_EXCEPTION, "No Status Exists", R.string.network_error);
        } else {
            //UNKNOWN status type
            onError(Status.StatusType.UNEXPECTED_EXCEPTION, "Unknown status type: " + response.body().getStatus().getStatusType(), R.string.network_error);
        }
    }

    /**
     * Invoked when a network exception occurred talking to the server or when an unexpected
     * exception occurred creating the request or processing the response.
     *
     * @param call
     * @param t
     */
    @Override
    public void onFailure(@NonNull Call<BaseResponse<T>> call, @NonNull Throwable t) {
        RemoteLog.e(TAG, getApiName(call) + " FAILURE: " + t.getMessage(), t);
        if (errorDelegate != null)
            errorDelegate.handleError(R.string.network_error);
    }

    public void onError(Status.StatusType statusType, String statusMessage, int displayMessage) {
        RemoteLog.w(TAG, "ERROR: " + statusType + ": " + statusMessage);
        if (errorDelegate != null)
            errorDelegate.handleError(displayMessage);
    }

    // override this if you need to check for badges awarded, default behavoiur is to just ignore the boolean
    protected void onSuccess(T payload, boolean badgesAwarded) {
        onSuccess(payload);
    }

    protected abstract void onSuccess(T payload);

    private String getApiName(Call<BaseResponse<T>> call) {
        List<String> path = call.request().url().pathSegments();
        return path.get(path.size() - 1);
    }
}
