package com.ibm.gbs.va.grit.rest;


/* RetrofitInterceptor Notes:
    1) If offline, this RetrofitInterceptor will produce error and action can be taken
    2) Any other errors (including offline), will produce error on callback onFailure
 */


import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.support.annotation.NonNull;

import com.google.gson.Gson;
import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.model.response.BaseResponse;
import com.ibm.gbs.va.grit.model.type.Status;
import com.ibm.gbs.va.grit.util.RemoteLog;
import com.ibm.gbs.va.grit.util.UserPrefs;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;
import java.io.InputStream;

import javax.inject.Inject;

import okhttp3.Interceptor;
import okhttp3.MediaType;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;
import okhttp3.ResponseBody;
import okio.Buffer;


public class RetrofitInterceptor implements Interceptor {

    private final UserPrefs userPrefs;
    private final Context mContext;
    private static final String TAG = "RetrofitInterceptor";

    @Inject
    public RetrofitInterceptor(Context context, UserPrefs userPrefs) {
        this.mContext = context;
        this.userPrefs = userPrefs;
    }

    @Override
    public Response intercept(@NonNull Chain chain) throws IOException {
        if (!preRequestCheck()) {
            RemoteLog.w(TAG, "DEVICE IS OFFLINE");
            //Create New Response with offline status type
            BaseResponse<?> baseResponse = new BaseResponse(Status.StatusType.OFFLINE, mContext.getString(R.string.no_connectivity));
            retrofit2.Response<BaseResponse<?>> response = retrofit2.Response.success(baseResponse);
            MediaType contentType = MediaType.parse("application/json; charset=utf-8");
            Gson gson = new Gson();
            ResponseBody body = ResponseBody.create(contentType, gson.toJson(baseResponse));
            return response.raw().newBuilder().body(body).build();
        }
        Request request = chain.request();
        boolean useMockData = Boolean.parseBoolean(request.header("UseMockdata"));
        if (useMockData) {
            BaseResponse<?> baseResponse = new BaseResponse(Status.StatusType.SUCCESS, mContext.getString(R.string.mock_data));
            retrofit2.Response<BaseResponse<?>> mockResponse = retrofit2.Response.success(baseResponse);
            //Get Mock Data From File
            String mockdataJsonFileName = request.header("MockdataJsonFile");
            JSONObject jsonFromFileData = getMockDataFromFile(mockdataJsonFileName);
            //Create New Response with Mock Data
            MediaType contentType = MediaType.parse("application/json; charset=utf-8");
            ResponseBody body = ResponseBody.create(contentType, jsonFromFileData.toString());
            return mockResponse.raw().newBuilder().body(body).build();
        } else {
            //Do not use mock data
            Request.Builder builder = request.newBuilder();
            boolean useAuthorization = Boolean.parseBoolean(request.header("Authorization"));
            if (useAuthorization) {
                RemoteLog.d(TAG, "Adding auth token.");
                builder.header("Authorization", userPrefs.getUserToken());
            }
            builder.removeHeader("UseMockdata")
                    .removeHeader("MockdataJsonFile");
            request = builder.build();

            //Return and Continue with Altered Request
            return chain.proceed(request);
        }
    }

    public static String requestBodyToString(RequestBody requestBody) {
        try {
            Buffer buffer = new Buffer();
            requestBody.writeTo(buffer);
            return buffer.readUtf8();
        } catch (IOException e) {
            return "Request body can not be determined";
        }
    }

    //Performs Action before the request is processed
    private boolean preRequestCheck() {
        boolean preRequestCheckIsOk = false;
        ConnectivityManager connectivityManager = (ConnectivityManager) mContext.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo networkInfo = connectivityManager.getActiveNetworkInfo();
        if (networkInfo != null && networkInfo.isConnected()) {
            preRequestCheckIsOk = true;
        } else {
            RemoteLog.w(TAG, "ERROR checking for connectivity");
        }
        return preRequestCheckIsOk;
    }

    // Get Mock Data from a .json file in the assets/mockdata directory
    private JSONObject getMockDataFromFile(String fileName) {
        JSONObject mockDataJson = null;
        try {
            String fileLocation = "mockdata/" + fileName + ".json";
            String json;
            InputStream is = mContext.getAssets().open(fileLocation);
            int size = is.available();
            byte[] buffer = new byte[size];
            is.read(buffer);
            is.close();
            json = new String(buffer, "UTF-8");
            mockDataJson = new JSONObject(json);
        } catch (IOException | JSONException e) {
            RemoteLog.e(TAG, "ERROR getting mock data: " + e.getMessage(), e);
        }
        return mockDataJson;
    }
}
