package com.ibm.gbs.va.grit.room;

import android.arch.persistence.db.SupportSQLiteDatabase;
import android.arch.persistence.room.Database;
import android.arch.persistence.room.Room;
import android.arch.persistence.room.RoomDatabase;
import android.arch.persistence.room.TypeConverters;
import android.content.Context;
import android.support.annotation.NonNull;

import com.commonsware.cwac.saferoom.SafeHelperFactory;
import com.google.gson.Gson;
import com.ibm.gbs.va.grit.BuildConfig;
import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.model.notifications.Notification;
import com.ibm.gbs.va.grit.model.survey.Answer;
import com.ibm.gbs.va.grit.model.survey.Question;
import com.ibm.gbs.va.grit.model.survey.QuestionSet;
import com.ibm.gbs.va.grit.model.type.Badge;
import com.ibm.gbs.va.grit.model.type.EmploymentInfo;
import com.ibm.gbs.va.grit.model.type.SquadMember;
import com.ibm.gbs.va.grit.model.type.User;
import com.ibm.gbs.va.grit.util.Keystore;
import com.ibm.gbs.va.grit.util.RemoteLog;
import com.ibm.gbs.va.grit.util.UserPrefs;

import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Arrays;
import java.util.UUID;
import java.util.concurrent.Executors;

@Database(entities = {User.class, QuestionSet.class, EmploymentInfo.class, Question.class, Answer.class, Badge.class, SquadMember.class, Notification.class}, version = 1)
@TypeConverters(com.ibm.gbs.va.grit.room.TypeConverters.class)
public abstract class GritDatabase extends RoomDatabase {
    private static final String TAG = "GritDatabase";

    private static GritDatabase INSTANCE;

    public synchronized static GritDatabase getInstance(Context context, Keystore keystore, UserPrefs userPrefs) {
        if (INSTANCE == null) {
            INSTANCE = buildDatabase(context, keystore, userPrefs);
        }
        return INSTANCE;
    }

    public abstract UserDao userDao();

    public abstract QSetDao qSetDao();

    public abstract AnswerDao answerDao();

    public abstract BadgeDao badgeDao();

    public abstract SquadDao squadDao();

    public abstract NotificationDao notificationDao();

    public void purgeUser(String userEmail) {
        // purge all things related to a given user name.
        notificationDao().purgeUserNotifications(userEmail);
        squadDao().purgeUserSquadMembers(userEmail);
        answerDao().purgeUserAnswers(userEmail);
        userDao().purgeUser(userEmail);
    }

    private static GritDatabase buildDatabase(final Context context, final Keystore keystore, final UserPrefs userPrefs) {
        RoomDatabase.Builder<GritDatabase> builder = Room.databaseBuilder(context, GritDatabase.class, "grit-db")
                .allowMainThreadQueries()
                .addCallback(new RoomDatabase.Callback() { // inserts initial badges into db
                    @Override
                    public void onCreate(@NonNull SupportSQLiteDatabase db) {
                        super.onCreate(db);
                        Executors.newSingleThreadScheduledExecutor().execute(() -> {
                            GritDatabase gritDb = getInstance(context, keystore, userPrefs);
                            if (userPrefs.getGlobalPrefHighestAddorder() == 0) {
                                // get badges from asset file
                                Badge[] badges = null;
                                try {
                                    InputStreamReader inputStream = new InputStreamReader(context.getAssets().open("initial_app_badges.json"));
                                    Gson gson = new Gson();

                                    badges = gson.fromJson(inputStream, Badge[].class);

                                } catch (IOException ex) {
                                    RemoteLog.e(TAG, "database badge init error: " + ex.getMessage(), ex);
                                }

                                if (badges != null) {
                                    // insert badges into db
                                    gritDb.badgeDao().saveMultipleBadges(Arrays.asList(badges));
                                    userPrefs.setGlobalPrefHighestAddorder(1);
                                }
                            }
                        });
                    }
                });

        if (BuildConfig.DEBUG)
            return builder.build();
        else {
            // init db encryption
            if (!keystore.keystoreAliasExists(R.string.ROOM_KEY_ALIAS)) {
                keystore.encryptAndSaveToGlobalPrefs(UUID.randomUUID().toString(), R.string.ROOM_KEY_ALIAS, R.string.ROOM_CIPHER_IV_KEY_ALIAS);
            }
            SafeHelperFactory factory = new SafeHelperFactory(keystore.decryptTextFromGlobalPrefs(R.string.ROOM_KEY_ALIAS, R.string.ROOM_CIPHER_IV_KEY_ALIAS).toCharArray());
            return builder.openHelperFactory(factory).build();
        }
    }
}
