package com.ibm.gbs.va.grit.room;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.Transformations;
import android.arch.persistence.room.Dao;
import android.arch.persistence.room.Delete;
import android.arch.persistence.room.Insert;
import android.arch.persistence.room.Query;
import android.arch.persistence.room.Transaction;
import android.arch.persistence.room.Update;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;

import com.ibm.gbs.va.grit.model.notifications.Notification;
import com.ibm.gbs.va.grit.util.RemoteLog;

import org.joda.time.DateTime;

import java.util.Iterator;
import java.util.List;
import java.util.concurrent.ConcurrentSkipListSet;

@Dao
public abstract class NotificationDao {
    private static final String TAG = "NotificationDao";

    private final UserDao userDao;

    public NotificationDao(GritDatabase gritDatabase) {
        this.userDao = gritDatabase.userDao();
    }

    @Insert
    public abstract void insertNotification(Notification notification);

    @Update
    public abstract void updateNotification(Notification notification);

    @Delete
    public abstract void deleteNotifications(List<Notification> notification);

    @Transaction
    public void purgeUserNotifications(String userEmail) {
        deleteNotifications(_getNotificationsForUserNow(userEmail));
    }

    /**
     * Deletes notifications that are not included in the new list from the DB
     *
     * @param notifications the notifications to save
     * @param userEmail     user to save the notifications for
     */
    @Transaction
    public void saveNotificationsForUser(List<Notification> notifications, String userEmail) {
        userDao.stubUser(userEmail); // in case user DNE
        List<Notification> dbNotifications = _getNotificationsForUserNow(userEmail);

        for (Notification item : notifications) {
            item.setUserEmail(userEmail);
            int dbListIndex = dbNotifications.indexOf(item);
            if (dbListIndex == -1) insertNotification(item);
            else {
                Notification dbItem = dbNotifications.get(dbListIndex);
                // retain any transient fields here
                item.setInternalId(dbItem.getInternalId());
                updateNotification(item);
            }
        }

        // remove notifications that were not in the new list
        int priorListSize = dbNotifications.size();
        dbNotifications.removeAll(notifications);
        dbNotifications.remove(getSurveySpawningNotificationForUserNow(userEmail));
        RemoteLog.d(TAG, "Removing (" + dbNotifications.size() + " of " + priorListSize + ") " +
                "un-backed notifications:\n" + dbNotifications);
        deleteNotifications(dbNotifications);
    }

    public void setSurveySpawningNotificationForUser(String userEmail, Long notificationId) {
        userDao.setSurveySpawningNotificationForUser(userEmail, notificationId);
    }

    @Query("SELECT notification.* FROM user LEFT JOIN notification ON notification.user_email == user.email " +
            "WHERE user.email LIKE :userEmail AND user.survey_spawning_notification_id == notification.internal_id")
    public abstract Notification getSurveySpawningNotificationForUserNow(String userEmail);

    @Query("SELECT notification.* FROM user LEFT JOIN notification ON notification.user_email == user.email " +
            "WHERE user.email LIKE :userEmail AND user.survey_spawning_notification_id == notification.internal_id")
    public abstract LiveData<Notification> getSurveySpawningNotificationForUser(String userEmail);


    @Query("SELECT notification.* FROM notification INNER JOIN user ON notification.user_email == user.email WHERE user.email LIKE :userEmail")
    abstract List<Notification> _getNotificationsForUserNow(String userEmail);

    @Transaction
    @Nullable
    public ConcurrentSkipListSet<Notification> getNotificationsForUserNow(String userEmail) {
        List<Notification> list = _getNotificationsForUserNow(userEmail);
        return list != null ? new ConcurrentSkipListSet<>(list) : null;
    }

    //region Displayable Notifications
    @Query("SELECT notification.* FROM notification INNER JOIN user ON notification.user_email == user.email AND notification.internal_id IS NOT user.survey_spawning_notification_id " +
            "WHERE user.email LIKE :userEmail")
    abstract List<Notification> _getDisplayableNotificationsForUserNow(String userEmail);

    /**
     * Set has these conditions enforced:
     * <li>the current survey spawning notification is excluded</li>
     * <li>notifications effective dates are before now (evaluated at the time of the call)</li>
     *
     * @param userEmail owner of the notifications
     * @return set of notifications as described above
     */
    @Transaction
    public ConcurrentSkipListSet<Notification> getDisplayableNotificationsForUserNow(String userEmail) {
        return removeFutureNotifications(_getDisplayableNotificationsForUserNow(userEmail));
    }

    @Query("SELECT notification.* FROM notification INNER JOIN user ON notification.user_email == user.email AND notification.internal_id IS NOT user.survey_spawning_notification_id " +
            "WHERE user.email LIKE :userEmail")
    abstract LiveData<List<Notification>> _getDisplayableNotificationsForUser(String userEmail);

    /**
     * Set has these conditions enforced:
     * <li>the current survey spawning notification is excluded</li>
     * <li>notifications effective dates are before now (evaluated when the set changes)</li>
     *
     * @param userEmail owner of the notifications
     * @return set of notifications as described above
     */
    @Transaction
    public LiveData<ConcurrentSkipListSet<Notification>> getDisplayableNotificationsForUser(String userEmail) {
        return Transformations.map(_getDisplayableNotificationsForUser(userEmail), list -> {
            if (list == null) return null;
            else {
                return removeFutureNotifications(list);
            }
        });
    }

    private ConcurrentSkipListSet<Notification> removeFutureNotifications(@NonNull List<Notification> list) {
        Iterator<Notification> iterator = list.listIterator();
        while (iterator.hasNext()) {
            Notification item = iterator.next();
            if (new DateTime(item.getEffectiveDate()).isAfterNow()) iterator.remove();
        }
        return new ConcurrentSkipListSet<>(list);
    }
    //endregion
}
