package com.ibm.gbs.va.grit.util;

import android.content.Context;
import android.security.keystore.KeyGenParameterSpec;
import android.security.keystore.KeyProperties;
import android.support.annotation.NonNull;
import android.support.annotation.StringRes;
import android.util.Base64;

import java.io.IOException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.UnrecoverableEntryException;
import java.security.cert.CertificateException;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.KeyGenerator;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.spec.GCMParameterSpec;
import javax.inject.Inject;
import javax.inject.Singleton;

@Singleton
public class Keystore {
    private static final String TRANSFORMATION = "AES/GCM/NoPadding";
    private static final String ANDROID_KEY_STORE = "AndroidKeyStore";
    private final static String TAG = "Keystore";

    private final Context context;
    private final UserPrefs userPrefs;

    private Cipher cipher;
    private KeyStore keyStore;

    @Inject
    public Keystore(UserPrefs userPrefs, Context context) {
        this.userPrefs = userPrefs;
        this.context = context;
        try {
            initKeyStore();
        } catch (CertificateException | NoSuchAlgorithmException | KeyStoreException |
                IOException e) {
            RemoteLog.e(TAG, "ERROR initializing keystore: " + e.getMessage(), e);
        }
    }

    private void initKeyStore() throws KeyStoreException, CertificateException,
            NoSuchAlgorithmException, IOException {
        keyStore = KeyStore.getInstance(ANDROID_KEY_STORE);
        keyStore.load(null);
    }

    /**
     * Stores an encrypted string for the <b>current user</b>
     *
     * @param textToEncrypt            what to encrypt
     * @param aliasKeyResValue         alias key
     * @param cipherIvAliasKeyResValue initialization vector
     */
    public void encryptAndSaveToUserPrefs(final String textToEncrypt, @StringRes int aliasKeyResValue, @StringRes int cipherIvAliasKeyResValue) {
        final String aliasKey = context.getString(aliasKeyResValue);
        String cipherIvAliasKey = context.getString(cipherIvAliasKeyResValue);
        try {
            //Encrypt the Text
            final byte[] encryptedTextByteArray = encryptText(textToEncrypt, aliasKey);

            //Convert encryptedText byte[] into String
            String encryptedTextString = Keystore.convertByteArrayToString(encryptedTextByteArray);

            //Save Encrypted Text String To Prefs
            userPrefs.setUserPrefString(aliasKey, encryptedTextString);

            //Save cipher.iv
            //Convert iv byte[] into String
            String iVString = convertByteArrayToString(cipher.getIV());
            //Save iVString To Prefs
            userPrefs.setUserPrefString(cipherIvAliasKey, iVString);
        } catch (NoSuchAlgorithmException | NoSuchProviderException | IOException |
                NoSuchPaddingException | InvalidKeyException e) {
            RemoteLog.e(TAG, "ERROR Accessing encryption method: " + e.getMessage(), e);
        } catch (InvalidAlgorithmParameterException | IllegalBlockSizeException |
                BadPaddingException e) {
            RemoteLog.e(TAG, "ERROR encrypting value: " + e.getMessage(), e);
        }
    }

    /**
     * Retrieves a decrypted string for the <b>current user</b>
     *
     * @param aliasKeyResValue         alias key
     * @param cipherIvAliasKeyResValue initialization vector
     * @return the decrypted string
     */
    public String decryptTextFromUserPrefs(@StringRes int aliasKeyResValue, @StringRes int cipherIvAliasKeyResValue) {
        final String aliasKey = context.getString(aliasKeyResValue);
        String cipherIvAliasKey = context.getString(cipherIvAliasKeyResValue);
        try {
            //  Get Saved encryptedTextString and convert back to Byte when decrypting
            String savedEncryptedPassword = userPrefs.getUserPrefString(aliasKey);
            if (savedEncryptedPassword != null) {
                byte[] encryptedPasswordByteArray = Keystore.convertStringToByteArray(savedEncryptedPassword);

                // Get Saved encrypted iVString and convert back to Byte when decrypting
                String savedIvString = userPrefs.getUserPrefString(cipherIvAliasKey);
                byte[] iVStringByteArray = Keystore.convertStringToByteArray(savedIvString);

                // Decrypt the encryptedPasswordByteArray and get the plain text result
                return decryptData(aliasKey, encryptedPasswordByteArray, iVStringByteArray);
            } else {
                RemoteLog.d(TAG, "decryptText - no user text to decrypt");
                return null;
            }
        } catch (UnrecoverableEntryException | NoSuchAlgorithmException |
                KeyStoreException | NoSuchPaddingException | IOException | InvalidKeyException e) {
            RemoteLog.e(TAG, "ERROR Accessing decryption method:" + e.getMessage(), e);
            return null;
        } catch (IllegalBlockSizeException | BadPaddingException | InvalidAlgorithmParameterException e) {
            RemoteLog.e(TAG, "ERROR decrypting value: " + e.getMessage(), e);
            return null;
        }
    }

    /**
     * @param textToEncrypt            what to encrypt
     * @param aliasKeyResValue         alias key
     * @param cipherIvAliasKeyResValue initialization vector
     */
    public void encryptAndSaveToGlobalPrefs(final String textToEncrypt, @StringRes int aliasKeyResValue, @StringRes int cipherIvAliasKeyResValue) {
        final String aliasKey = context.getString(aliasKeyResValue);
        String cipherIvAliasKey = context.getString(cipherIvAliasKeyResValue);
        try {
            //Encrypt the Text
            final byte[] encryptedTextByteArray = encryptText(textToEncrypt, aliasKey);

            //Convert encryptedText byte[] into String
            String encryptedTextString = Keystore.convertByteArrayToString(encryptedTextByteArray);

            //Save Encrypted Text String To Prefs
            userPrefs.setGlobalString(aliasKey, encryptedTextString);

            //Save cipher.iv
            //Convert iv byte[] into String
            String iVString = convertByteArrayToString(cipher.getIV());
            //Save iVString To Prefs
            userPrefs.setGlobalString(cipherIvAliasKey, iVString);
        } catch (NoSuchAlgorithmException | NoSuchProviderException | IOException |
                NoSuchPaddingException | InvalidKeyException e) {
            RemoteLog.e(TAG, "ERROR Accessing encryption method: " + e.getMessage(), e);
        } catch (InvalidAlgorithmParameterException | IllegalBlockSizeException |
                BadPaddingException e) {
            RemoteLog.e(TAG, "ERROR encrypting value: " + e.getMessage(), e);
        }
    }

    /**
     * @param aliasKeyResValue         alias key
     * @param cipherIvAliasKeyResValue initialization vector
     * @return the decrypted string
     */
    public String decryptTextFromGlobalPrefs(@StringRes int aliasKeyResValue, @StringRes int cipherIvAliasKeyResValue) {
        final String aliasKey = context.getString(aliasKeyResValue);
        String cipherIvAliasKey = context.getString(cipherIvAliasKeyResValue);
        try {
            //  Get Saved encryptedTextString and convert back to Byte when decrypting
            String savedEncryptedPassword = userPrefs.getGlobalString(aliasKey);

            if (savedEncryptedPassword != null) {
                byte[] encryptedPasswordByteArray = Keystore.convertStringToByteArray(savedEncryptedPassword);

                // Get Saved encrypted iVString and convert back to Byte when decrypting
                String savedIvString = userPrefs.getGlobalString(cipherIvAliasKey);
                byte[] iVStringByteArray = Keystore.convertStringToByteArray(savedIvString);

                // Decrypt the encryptedPasswordByteArray and get the plain text result
                return decryptData(aliasKey, encryptedPasswordByteArray, iVStringByteArray);
            } else {
                RemoteLog.d(TAG, "decryptText - no user text to decrypt");
                return null;
            }
        } catch (UnrecoverableEntryException | NoSuchAlgorithmException |
                KeyStoreException | NoSuchPaddingException | IOException | InvalidKeyException e) {
            RemoteLog.e(TAG, "ERROR Accessing decryption method:" + e.getMessage(), e);
            return null;
        } catch (IllegalBlockSizeException | BadPaddingException | InvalidAlgorithmParameterException e) {
            RemoteLog.e(TAG, "ERROR decrypting value: " + e.getMessage(), e);
            return null;
        }
    }

    /*
     ********  ENCRYPT - This encrypts a String and keeps part of the key on the device.
     * In order to Decrypt, you need the encrypted String and the iVString created during the encryption
     * The encrypted text is not saved to the device, but the key to decrypt it is
     */
    private byte[] encryptText(final String textToEncrypt, final String aliasKey)
            throws NoSuchAlgorithmException, NoSuchProviderException, NoSuchPaddingException,
            InvalidKeyException, IOException, InvalidAlgorithmParameterException,
            BadPaddingException, IllegalBlockSizeException {
        return getCipher(aliasKey).doFinal(textToEncrypt.getBytes("UTF-8"));
    }

    private Cipher getCipher(String aliasKey) throws NoSuchPaddingException, NoSuchAlgorithmException,
            NoSuchProviderException, InvalidAlgorithmParameterException, InvalidKeyException {
        cipher = Cipher.getInstance(TRANSFORMATION);
        cipher.init(Cipher.ENCRYPT_MODE, getSecretKeyForEncrypt(aliasKey));
        return cipher;
    }

    @NonNull
    private SecretKey getSecretKeyForEncrypt(final String alias) throws NoSuchAlgorithmException,
            NoSuchProviderException, InvalidAlgorithmParameterException {
        final KeyGenerator keyGenerator = KeyGenerator
                .getInstance(KeyProperties.KEY_ALGORITHM_AES, ANDROID_KEY_STORE);
        keyGenerator.init(new KeyGenParameterSpec.Builder(alias,
                KeyProperties.PURPOSE_ENCRYPT | KeyProperties.PURPOSE_DECRYPT)
                .setBlockModes(KeyProperties.BLOCK_MODE_GCM)
                .setEncryptionPaddings(KeyProperties.ENCRYPTION_PADDING_NONE)
                .build());
        return keyGenerator.generateKey();
    }

    //  DECRYPT - Decrypts an encrypted String with the cipher.iv settings and the key stored on the device
    private String decryptData(final String aliasKey, final byte[] encryptedData, final byte[] encryptionIv)
            throws UnrecoverableEntryException, NoSuchAlgorithmException, KeyStoreException,
            NoSuchPaddingException, InvalidKeyException, IOException, BadPaddingException,
            IllegalBlockSizeException, InvalidAlgorithmParameterException {
        if (!keystoreAliasExists(aliasKey)) {
            return null;
        }
        final Cipher cipher = Cipher.getInstance(TRANSFORMATION);
        final GCMParameterSpec spec = new GCMParameterSpec(128, encryptionIv);
        cipher.init(Cipher.DECRYPT_MODE, getSecretKeyForDecrypt(aliasKey), spec);
        return new String(cipher.doFinal(encryptedData), "UTF-8");
    }

    private SecretKey getSecretKeyForDecrypt(final String alias) throws NoSuchAlgorithmException,
            UnrecoverableEntryException, KeyStoreException {
        return ((KeyStore.SecretKeyEntry) keyStore.getEntry(alias, null)).getSecretKey();
    }

    //region Utilities
    public boolean keystoreAliasExists(@StringRes int aliasKeyResValue) {
        return keystoreAliasExists(context.getString(aliasKeyResValue));
    }

    private boolean keystoreAliasExists(String aliasKey) {
        try {
            return keyStore.containsAlias(aliasKey);
        } catch (KeyStoreException e) {
            RemoteLog.e(TAG, "ERROR checking for keystoreAlias: " + e.getMessage(), e);
            return false;
        }
    }

    public void deleteKeystoreAliasEntry(@StringRes int aliasKeyResValue){
        deleteKeystoreAliasEntry(context.getString(aliasKeyResValue));
    }

    private void deleteKeystoreAliasEntry(String aliasKey){
        try {
            keyStore.deleteEntry(aliasKey);
        } catch (KeyStoreException e) {
            RemoteLog.e(TAG, "ERROR deleting keystoreAlias: " + e.getMessage(), e);
        }
    }

    private static String convertByteArrayToString(byte[] byteArray) {
        return Base64.encodeToString(byteArray, Base64.NO_WRAP);
    }

    private static byte[] convertStringToByteArray(String string) {
        return Base64.decode(string, Base64.NO_WRAP);
    }
    //endregion
}
