package com.ibm.gbs.va.grit.util;

import android.content.Context;
import android.graphics.drawable.Drawable;
import android.support.design.widget.TextInputLayout;
import android.support.v7.content.res.AppCompatResources;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.View;
import android.widget.EditText;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.model.ValidationRule;

import java.util.List;

public class LocalValidationHelper {
    private final TextInputLayout inputLayout;
    private final EditText inputEditText;
    private ValidationRule validationRule;
    private final Drawable ic_valid;
    private final Drawable ic_invalid;
    private final Drawable originalDrawable;
    private final Context context;
    private final String TAG = this.getClass().getName();

    public LocalValidationHelper(TextInputLayout inputLayout, ValidationRule validationRule) {
        this.inputLayout = inputLayout;
        this.inputEditText = inputLayout.getEditText();
        this.validationRule = validationRule;
        this.context = inputLayout.getContext();
        this.ic_valid = this.context.getDrawable(R.drawable.ic_success);
        this.ic_invalid = this.context.getDrawable(R.drawable.ic_invalid);

        boolean isPasswordField = inputLayout.isPasswordVisibilityToggleEnabled();
        this.originalDrawable = isPasswordField ? inputLayout.getPasswordVisibilityToggleDrawable() : inputEditText.getCompoundDrawables()[2];
    }

    public void setValidationRule(ValidationRule validationRule) {
        this.validationRule = validationRule;
    }

    public void validateAfterChange() {
        inputEditText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
            }

            @Override
            public void afterTextChanged(Editable s) {
                validate();
            }
        });
    }

    public TextInputLayout getInputLayout() {
        return inputLayout;
    }

    private View validateNow() {
        validate();
        if (!isValid()) {
            return this.inputLayout;
        } else {
            return null;
        }
    }

    public void setErrorIcon(Boolean isError) {
        boolean isPasswordField = inputLayout.isPasswordVisibilityToggleEnabled();
        if (isError == null) {
            clearIcons();
        } else if (isError) {
            if (isPasswordField)
                inputLayout.setPasswordVisibilityToggleTintList(AppCompatResources.getColorStateList(context, R.color.colorError));
            else
                inputEditText.setCompoundDrawablesWithIntrinsicBounds(null, null, ic_invalid, null);
        } else {
            if (isPasswordField)
                inputLayout.setPasswordVisibilityToggleTintList(AppCompatResources.getColorStateList(context, R.color.colorSuccess));
            else
                inputEditText.setCompoundDrawablesWithIntrinsicBounds(null, null, ic_valid, null);
        }
    }

    public String getText() {
        return inputEditText.getText().toString();
    }

    public void setError(String error) {
        RemoteLog.d(TAG, "setting custom error: " + error);
        setErrorIcon(true);
        inputLayout.setError(error);
        inputLayout.requestFocus();
    }

    private void validate() {
        String error = silentlyValidate();
        if (error == null) {
            setErrorIcon(false);
            inputLayout.setError(null);
        } else if (error.isEmpty()) {
            clearIcons();
            inputLayout.setError(null);
        } else {
            setErrorIcon(true);
            inputLayout.setError(error);
        }
    }

    private String silentlyValidate() {
        Editable s = inputEditText.getText();
        if (!validationRule.canBeEmpty() && TextUtils.isEmpty(s)) {
            RemoteLog.d(TAG, s.toString() + " violates isEmpty");
            return validationRule.getEmptyErr(this.context);
        } else if (validationRule.canBeEmpty() && TextUtils.isEmpty(s)) { // optional field is empty
            return "";
        } else if (validationRule.hasMaxLength() && s.length() > validationRule.getMaxLength()) {
            RemoteLog.d(TAG, s.toString() + " violates maxLength");
            return validationRule.getMaxLengthErr(this.context);
        } else if (validationRule.hasMinLength() && s.length() < validationRule.getMinLength()) {
            RemoteLog.d(TAG, s.toString() + " violates minLength");
            return validationRule.getMinLengthErr(this.context);
        } else if (validationRule.hasRequiredRegex() && !s.toString().matches(validationRule.getRequiredRegex())) {
            RemoteLog.d(TAG, s.toString() + " violates requiredRegex");
            return validationRule.getRequiredRegexErr(this.context);
        } else {
            return null;
        }
    }

    public void clearIcons() {
        boolean isPasswordField = inputLayout.isPasswordVisibilityToggleEnabled();
        if (isPasswordField)
            inputLayout.setPasswordVisibilityToggleDrawable(originalDrawable);
        else
            inputEditText.setCompoundDrawablesWithIntrinsicBounds(null, null, null, null);
    }

    private Boolean isValid() {
        return inputLayout.getError() == null;
    }

    public static void validateInputs(List<LocalValidationHelper> validationHelpers, Consumer<View> validFunction) {
        View focusView = null;
        for (LocalValidationHelper validationHelper : validationHelpers) {
            View thisView = validationHelper.validateNow();
            focusView = focusView != null ? focusView : thisView;
        }
        if (focusView != null) {
            // There was an error; don't attempt login and focus the first
            // form field with an error.
            focusView.requestFocus();
        } else {
            validFunction.accept(focusView);
        }
    }

    public static void validateInputsSilently(List<LocalValidationHelper> validationHelpers, Consumer<Void> validFunction) {
        boolean allValid = true;
        for (LocalValidationHelper item : validationHelpers) {
            allValid = allValid && TextUtils.isEmpty(item.silentlyValidate());
        }
        if (allValid) validFunction.accept(null);
    }
}
