package com.ibm.gbs.va.grit.util;

import android.content.Context;
import android.content.SharedPreferences;
import android.text.TextUtils;

import com.ibm.gbs.va.grit.GritApplication;

import java.io.File;
import java.util.Map;
import java.util.Set;

import javax.inject.Inject;
import javax.inject.Singleton;

@Singleton
public class UserPrefs {
    private static final String TAG = "UserPrefs";

    private static final String PREFERENCES_FILE = "user_preferences";

    private static final String PREF_GLOBAL_FIRST_TIME = "global_first_time";
    private static final String PREF_USER_CURRENT = "user_current";
    private static final String PREF_GLOBAL_HIGHEST_ADDORDER = "global_highest_add_order";
    private static final String PREF_GLOBAL_ACCEPTED_PRIVACY = "user_accepted_privacy";

    private final GritApplication ctx;

    private String currentUserEmail;

    @Inject
    public UserPrefs(GritApplication ctx) {
        this.ctx = ctx;
    }

    //region Global Helpers
    void setGlobalString(String key, String value) {
        SharedPreferences.Editor editor = ctx.getSharedPreferences(PREFERENCES_FILE, Context.MODE_PRIVATE).edit();
        editor.putString(key, value);
        editor.apply();
    }

    String getGlobalString(String key) {
        SharedPreferences sharedPref = ctx.getSharedPreferences(PREFERENCES_FILE, Context.MODE_PRIVATE);
        return sharedPref.getString(key, "");
    }
    //endregion

    public String getCurrentUserEmail() {
        return getGlobalString(PREF_USER_CURRENT);
    }

    public void setCurrentUserEmail(String currentUserEmail) {
        if (currentUserEmail == null) {
            ctx.resetCurrentUserScope();
            this.currentUserEmail = null;
            setGlobalString(PREF_USER_CURRENT, "null");
            RemoteLog.d(TAG, "Setting current userEmail to null");
            return;
        }
        RemoteLog.d(TAG, "Changing CurrentUser from: " + this.currentUserEmail + " to: " + currentUserEmail);
        if (!currentUserEmail.equals(this.currentUserEmail)) {
            ctx.resetCurrentUserScope();
            this.currentUserEmail = currentUserEmail;
            setGlobalString(PREF_USER_CURRENT, currentUserEmail);
        }
    }

    public boolean getGlobalFirstTime() {
        SharedPreferences sharedPref = ctx.getSharedPreferences(PREFERENCES_FILE, Context.MODE_PRIVATE);
        return sharedPref.getBoolean(PREF_GLOBAL_FIRST_TIME, true);
    }

    public void setGlobalFirstTime(boolean value) {
        SharedPreferences.Editor editor = ctx.getSharedPreferences(PREFERENCES_FILE, Context.MODE_PRIVATE).edit();
        editor.putBoolean(PREF_GLOBAL_FIRST_TIME, value);
        editor.apply();
    }

    public int getGlobalPrefHighestAddorder() {
        SharedPreferences sharedPref = ctx.getSharedPreferences(PREFERENCES_FILE, Context.MODE_PRIVATE);
        return sharedPref.getInt(PREF_GLOBAL_HIGHEST_ADDORDER, 0);
    }

    public void setGlobalPrefHighestAddorder(int value) {
        SharedPreferences.Editor editor = ctx.getSharedPreferences(PREFERENCES_FILE, Context.MODE_PRIVATE).edit();
        editor.putInt(PREF_GLOBAL_HIGHEST_ADDORDER, value);
        editor.apply();
    }

    public boolean getGlobalAcceptedPolicy() {
        SharedPreferences sharedPref = ctx.getSharedPreferences(PREFERENCES_FILE, Context.MODE_PRIVATE);
        return sharedPref.getBoolean(PREF_GLOBAL_ACCEPTED_PRIVACY, false);
    }

    public void setGlobalAcceptedPolicy(boolean value) {
        SharedPreferences.Editor editor = ctx.getSharedPreferences(PREFERENCES_FILE, Context.MODE_PRIVATE).edit();
        editor.putBoolean(PREF_GLOBAL_ACCEPTED_PRIVACY, value);
        editor.apply();
    }


    //region User Pref Helpers
    private String getCurrentUser() {
        // To reduce excessive file-checking, check if we already have it
        if (TextUtils.isEmpty(currentUserEmail)) currentUserEmail = getCurrentUserEmail();
        return currentUserEmail;
    }

    void setUserPrefString(String key, String value) {
        SharedPreferences.Editor editor = ctx.getSharedPreferences(getCurrentUser(), Context.MODE_PRIVATE).edit();
        editor.putString(key, value);
        editor.apply();
    }

    String getUserPrefString(String key) {
        SharedPreferences sharedPref = ctx.getSharedPreferences(getCurrentUser(), Context.MODE_PRIVATE);
        return sharedPref.getString(key, "");
    }

    private boolean getUserPrefBoolean(String key, boolean defaultVal) {
        SharedPreferences sharedPref = ctx.getSharedPreferences(getCurrentUser(), Context.MODE_PRIVATE);
        return sharedPref.getBoolean(key, defaultVal);
    }

    private void setUserPrefBoolean(String key, boolean value) {
        SharedPreferences.Editor editor = ctx.getSharedPreferences(getCurrentUser(), Context.MODE_PRIVATE).edit();
        editor.putBoolean(key, value);
        editor.apply();
    }

    @SuppressWarnings("SameParameterValue")
    private int getUserPrefInt(String key) {
        SharedPreferences sharedPref = ctx.getSharedPreferences(getCurrentUser(), Context.MODE_PRIVATE);
        return sharedPref.getInt(key, 0);
    }

    @SuppressWarnings("SameParameterValue")
    private void setUserPrefInt(String key, int value) {
        SharedPreferences.Editor editor = ctx.getSharedPreferences(getCurrentUser(), Context.MODE_PRIVATE).edit();
        editor.putInt(key, value);
        editor.apply();
    }
    //endregion

    //region User Prefs
    private static final String PREF_USER_LOGGED_IN = "user_logged_in";
    private static final String PREF_USER_HAS_LOGGED_IN = "user_has_logged_in";
    private static final String PREF_USER_EMAIL = "user_email";
    private static final String PREF_USER_NAME = "user_name";
    private static final String PREF_USER_BIOMETRICS_EMAIL = "user_biometrics_email";
    private static final String PREF_USER_TOKEN = "user_token";
    private static final String PREF_IN_PROGRESS_PAGE = "in_progress_page";


    public boolean getUserLoggedIn() {
        return getUserPrefBoolean(PREF_USER_LOGGED_IN, false);
    }

    public void setUserLoggedIn(boolean value) {
        setUserPrefBoolean(PREF_USER_LOGGED_IN, value);
    }

    public boolean getUserHasLoggedIn() {
        return getUserPrefBoolean(PREF_USER_HAS_LOGGED_IN, false);
    }

    public void setUserHasLoggedIn(boolean value) {
        setUserPrefBoolean(PREF_USER_HAS_LOGGED_IN, value);
    }

    public String getUserEmail() {
        return getUserPrefString(PREF_USER_EMAIL);
    }

    public void setUserEmail(String value) {
        setUserPrefString(PREF_USER_EMAIL, value);
    }

    public String getUserName() {
        return getUserPrefString(PREF_USER_NAME);
    }

    public void setUserName(String value) {
        setUserPrefString(PREF_USER_NAME, value);
    }

    public String getUserBiometricsEmail() {
        return getUserPrefString(PREF_USER_BIOMETRICS_EMAIL);
    }

    public void setUserBiometricsEmail(String value) {
        setUserPrefString(PREF_USER_BIOMETRICS_EMAIL, value);
    }

    public String getUserToken() {
        return getUserPrefString(PREF_USER_TOKEN);
    }

    public void setUserToken(String value) {
        setUserPrefString(PREF_USER_TOKEN, value);
    }

    public int getInProgressPage() {
        return getUserPrefInt(PREF_IN_PROGRESS_PAGE);
    }

    public void setInProgressPage(int value) {
        setUserPrefInt(PREF_IN_PROGRESS_PAGE, value);
    }

    //endregion

    //region Local User Goal Notifications
    private static final String PREF_JOBINFO_COMPLETE = "jobinfo_complete";
    private static final String PREF_SQUAD_COMPLETE = "squad_complete";
    private static final String PREF_PROFILE_COMPLETE = "profile_complete";
    private static final String PREF_INITIAL_BASELINE_COMPLETE = "initial_baseline_complete";

    public boolean getPrefJobInfoComplete() {
        return getUserPrefBoolean(PREF_JOBINFO_COMPLETE, false);
    }

    public void setPrefJobinfoComplete(boolean value) {
        setUserPrefBoolean(PREF_JOBINFO_COMPLETE, value);
    }

    public boolean getPrefSquadComplete() {
        return getUserPrefBoolean(PREF_SQUAD_COMPLETE, false);
    }

    public void setPrefSquadComplete(boolean value) {
        setUserPrefBoolean(PREF_SQUAD_COMPLETE, value);
    }

    public boolean getPrefProfileComplete() {
        return getUserPrefBoolean(PREF_PROFILE_COMPLETE, false);
    }

    public void setPrefProfileComplete(boolean value) {
        setUserPrefBoolean(PREF_PROFILE_COMPLETE, value);
    }

    public boolean getPrefInitialBaselineComplete() {
        return getUserPrefBoolean(PREF_INITIAL_BASELINE_COMPLETE, false);
    }

    public void setPrefInitialBaselineComplete(boolean value) {
        setUserPrefBoolean(PREF_INITIAL_BASELINE_COMPLETE, value);
    }
    //endregion

    //region Info Dialog User Prefs
    private static final String PREF_JOBINFO_INFO = "jobinfo_info";
    private static final String PREF_SQUAD_INFO = "squad_info";
    private static final String PREF_SURVEY_INFO = "survey_info";
    private static final String PREF_SKILLS_INFO = "skills_info";
    private static final String PREF_DIGITALASSIST_INFO = "digital_assistant_info";
    private static final String PREF_DS_DIGITALJOURNAL_INFO = "digital_journal_info";

    public boolean getPrefJobinfoInfo() {
        return getUserPrefBoolean(PREF_JOBINFO_INFO, true);
    }

    public void setPrefJobinfoInfo(boolean value) {
        setUserPrefBoolean(PREF_JOBINFO_INFO, value);
    }

    public boolean getPrefSquadInfo() {
        return getUserPrefBoolean(PREF_SQUAD_INFO, true);
    }

    public void setPrefSquadInfo(boolean value) {
        setUserPrefBoolean(PREF_SQUAD_INFO, value);
    }

    public boolean getPrefSurveyInfo() {
        return getUserPrefBoolean(PREF_SURVEY_INFO, true);
    }

    public void setPrefSurveyInfo(boolean value) {
        setUserPrefBoolean(PREF_SURVEY_INFO, value);
    }

    public boolean getPrefSkillsInfo() {
        return getUserPrefBoolean(PREF_SKILLS_INFO, true);
    }

    public void setPrefSkillsInfo(boolean value) {
        setUserPrefBoolean(PREF_SKILLS_INFO, value);
    }

    public boolean getPrefDigitalAssistInfo() {
        return getUserPrefBoolean(PREF_DIGITALASSIST_INFO, true);
    }

    public void setPrefDigitalAssistInfo(boolean value) {
        setUserPrefBoolean(PREF_DIGITALASSIST_INFO, value);
    }

    public boolean getPrefDsDigitaljournalInfo() {
        return getUserPrefBoolean(PREF_DS_DIGITALJOURNAL_INFO, true);
    }

    public void setPrefDsDigitaljournalInfo(boolean value) {
        setUserPrefBoolean(PREF_DS_DIGITALJOURNAL_INFO, value);
    }
    //endregion

    public void renamePref(String oldFile, String newFile) {
        if (!oldFile.equals(newFile)) {
            SharedPreferences oldSharedPref = ctx.getSharedPreferences(oldFile, Context.MODE_PRIVATE);
            SharedPreferences.Editor newSharedPrefEdit = ctx.getSharedPreferences(newFile, Context.MODE_PRIVATE).edit();

            for (Map.Entry<String, ?> entry : oldSharedPref.getAll().entrySet()) {
                String key = entry.getKey();
                Object value = entry.getValue();
                if (value instanceof Boolean)
                    newSharedPrefEdit.putBoolean(key, (Boolean) value);
                else if (value instanceof String)
                    newSharedPrefEdit.putString(key, (String) value);
                else if (value instanceof Integer)
                    newSharedPrefEdit.putInt(key, (Integer) value);
                else if (value instanceof Float)
                    newSharedPrefEdit.putFloat(key, (Float) value);
                else if (value instanceof Long)
                    newSharedPrefEdit.putLong(key, (Long) value);
                else if (value instanceof Set) {
                    newSharedPrefEdit.putStringSet(key, (Set<String>) value);
                }
            }
            newSharedPrefEdit.apply();
            oldSharedPref.edit().clear().apply();
        }
    }

    public void deletePref(String userName) {
        SharedPreferences sharedPrefToDelete = ctx.getSharedPreferences(userName, Context.MODE_PRIVATE);
        sharedPrefToDelete.edit().clear().apply();
        File file = new File(ctx.getFilesDir().getParent() + "/shared_prefs/" + userName + ".xml");
        file.delete();
        setCurrentUserEmail(null);
    }

}
