package com.ibm.gbs.va.grit.viewmodel;

import android.app.Application;
import android.arch.lifecycle.AndroidViewModel;
import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.MutableLiveData;
import android.arch.lifecycle.Transformations;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.LayerDrawable;
import android.support.annotation.DrawableRes;
import android.support.annotation.IdRes;
import android.support.annotation.StringRes;
import android.support.graphics.drawable.VectorDrawableCompat;
import android.support.v4.graphics.drawable.DrawableCompat;
import android.support.v4.util.Pair;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.model.notifications.Notification;
import com.ibm.gbs.va.grit.model.type.Badge;
import com.ibm.gbs.va.grit.model.type.User;
import com.ibm.gbs.va.grit.repository.JobRepository;
import com.ibm.gbs.va.grit.repository.NotificationRepository;
import com.ibm.gbs.va.grit.repository.SquadRepository;
import com.ibm.gbs.va.grit.repository.SurveyRepository;
import com.ibm.gbs.va.grit.repository.UserRepository;
import com.ibm.gbs.va.grit.util.Consumer;

import java.util.Arrays;
import java.util.List;
import java.util.concurrent.ConcurrentSkipListSet;

import javax.inject.Inject;

public class DashboardViewModel extends AndroidViewModel {

    private final MutableLiveData<List<MenuData>> menuData = new MutableLiveData<>();
    private final LiveData<ConcurrentSkipListSet<Notification>> messages;
    public final LiveData<Boolean> hasMessages;

    private final NotificationRepository messageRepo;
    private final SurveyRepository surveyRepo;

    private final LiveData<User> user;

    public final List<TileData> tiles = Arrays.asList(
            new TileData(R.string.digital_self, R.id.digitalSelf, R.drawable.card_digital_self),
            new TileData(R.string.employment, R.id.jobs, R.drawable.card_jobs),
            new TileData(R.string.skills, R.id.skills, R.drawable.card_skills),
            new TileData(R.string.squad, R.id.squad, R.drawable.card_squad)
    );

    @Inject
    public DashboardViewModel(Application application, NotificationRepository messageRepo, UserRepository userRepo, SurveyRepository surveyRepo, JobRepository jobRepository, SquadRepository squadRepository) {
        super(application);
        user = userRepo.getUserInfo();
        //VG-928 - Calling getSquad in case notification needs to be dismissed at new install
        squadRepository.getSquad();
        this.messageRepo = messageRepo;
        this.surveyRepo = surveyRepo;
        jobRepository.getTargetIndustries();
        messages = messageRepo.getNotificationsToDisplay();
        hasMessages = Transformations.map(messages, set -> set != null && !set.isEmpty());

        menuData.setValue(Arrays.asList(
                new MenuData(R.string.dashboard, R.id.dashboard, R.id.dashboard, R.drawable.ic_home),
                new MenuData(R.string.digital_self, R.id.digitalSelf, R.id.digitalSelf, R.drawable.ic_screen),
                new MenuData(R.string.employment, R.id.jobs, R.id.jobs, R.drawable.ic_briefcase),
                new MenuData(R.string.skills, R.id.jobs, R.id.skills, R.drawable.ic_ribbon),
                new MenuData(R.string.filter, R.id.jobs, R.id.jobFilter, R.drawable.ic_filter_white, true),
                new MenuData(R.string.squad, R.id.squad, R.id.squad, R.drawable.ic_squad),
                new MenuData(R.string.chat, R.id.chat, R.id.chat, R.drawable.ic_chat),
                new MenuData(R.string.immediate_help, R.id.immediateHelp, R.id.immediateHelp, R.drawable.ic_warning, true),
                new MenuData(R.string.settings, R.id.settings, R.id.settings, R.drawable.ic_settings, true),
                new MenuData(R.string.edit_profile, R.id.settings_editProfile, R.id.settings_editProfile, R.drawable.ic_settings, true),
                new MenuData(R.string.about_this_app, R.id.aboutThisApp, R.id.aboutThisApp, R.drawable.ic_info, true),
                new MenuData(R.string.sign_out, R.id.signOut, R.id.signOut, R.drawable.ic_power)
        ));
    }

    public LiveData<User> getUser() {
        return user;
    }

    public LiveData<ConcurrentSkipListSet<Notification>> getMessages() {
        return messages;
    }

    public LiveData<List<MenuData>> getMenuData() {
        return menuData;
    }

    public MenuData getMenuItem(@IdRes int id) {
        List<MenuData> list = menuData.getValue();
        assert list != null;
        for (MenuData item : list) {
            if (item.menuId == id) return item;
        }
        return null;
    }

    public void dismissNotification(Notification.Type notificationType) {
        messageRepo.dismissNotification(notificationType);
    }

    public void dismissNotification(Notification notificationToDismiss) {
        messageRepo.dismissNotification(notificationToDismiss, null);
    }

    public void setMenuHasBadge(@IdRes int id, boolean hasBadge) {
        List<MenuData> list = menuData.getValue();
        MenuData item = getMenuItem(id);
        list.get(list.indexOf(item)).showBadge = hasBadge;
        menuData.setValue(list);
    }

    public LiveData<Notification> getCurrentSurveyNotification() {
        return messageRepo.getSurveySpawningNotificationForUser();
    }

    public void updateMessages() {
        messageRepo.fetchNotifications(null);
    }

    /**
     * @param notification The survey to open after aborting the current one
     * @see SurveyRepository#abortCurrentSurvey(Consumer)
     */
    public void abortCurrentSurvey(Notification notification, Consumer<Pair<Boolean, Badge>> callback) {
        surveyRepo.abortCurrentSurvey(booleanBadgePair -> {
            if (booleanBadgePair.first == null) { // error occurred
                callback.accept(booleanBadgePair);
            } else {
                messageRepo.setSurveyNotification(notification, dismissed -> {
                    if (dismissed && booleanBadgePair.first) {
                        // abort succeeded
                        // setup for starting new survey
                        surveyRepo.updateCurrentQuestionSet(result -> callback.accept(new Pair<>(result, booleanBadgePair.second)));
                    }
                });
            }
        });
    }

    public void setSurveyNotification(Notification notification, Consumer<Boolean> callback) {
        messageRepo.setSurveyNotification(notification, callback);
    }

    //region Isolated Data Models
    public class TileData {
        public final String title;
        @IdRes
        public final int id;
        @DrawableRes
        public final int imageId;

        TileData(@StringRes int title, @IdRes int id, @DrawableRes int imageId) {
            this.title = getApplication().getString(title);
            this.id = id;
            this.imageId = imageId;
        }
    }

    public class MenuData {
        public final String title;
        @IdRes
        public final int fragmentId,
                menuId;
        public final LayerDrawable iconDrawable;
        public boolean blocksMenuAccess = false;
        public boolean showBadge = false;

        MenuData(@StringRes int title, @IdRes int fragmentId, @IdRes int menuId) {
            this.title = getApplication().getString(title);
            this.menuId = menuId;
            this.fragmentId = fragmentId;
            iconDrawable = (LayerDrawable) getApplication().getDrawable(R.drawable.badged_nav_menu_item);
            iconDrawable.setDrawableByLayerId(R.id.menuIcon, getApplication().getDrawable(R.drawable.ic_invalid));
        }

        MenuData(@StringRes int title, @IdRes int fragmentId, @IdRes int menuId, @DrawableRes int iconRes) {
            this(title, fragmentId, menuId);
            VectorDrawableCompat menuIcon = VectorDrawableCompat.create(getApplication().getResources(), iconRes, null);
            //noinspection ConstantConditions
            menuIcon.mutate();
            menuIcon.setTintList(getApplication().getColorStateList(R.color.active_menu_item));
            iconDrawable.setDrawableByLayerId(R.id.menuIcon, menuIcon);
            Drawable badge = DrawableCompat.wrap(iconDrawable.findDrawableByLayerId(R.id.badgeIcon));
            badge.mutate();
            badge.setAlpha(showBadge ? 255 : 0);
            badge.setTintList(getApplication().getColorStateList(R.color.active_menu_item_badge));
        }

        @SuppressWarnings("SameParameterValue")
        MenuData(@StringRes int title, @IdRes int fragmentId, @IdRes int menuId, @DrawableRes int icon, boolean blocksMenuAccess) {
            this(title, fragmentId, menuId, icon);
            this.blocksMenuAccess = blocksMenuAccess;
        }

        @Override
        public boolean equals(Object obj) {
            if (obj instanceof MenuData)
                return ((MenuData) obj).title.equals(title);
            else
                return super.equals(obj);
        }
    }
    //endregion
}


