package com.ibm.gbs.va.grit.viewmodel;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.MutableLiveData;
import android.arch.lifecycle.Transformations;
import android.arch.lifecycle.ViewModel;
import android.support.annotation.IdRes;
import android.text.TextUtils;
import android.util.Range;
import android.util.SparseArray;
import android.view.View;
import android.widget.SeekBar;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.model.request.SearchSkillsRequest;
import com.ibm.gbs.va.grit.model.request.UploadResumeRequest;
import com.ibm.gbs.va.grit.model.response.SkillsResponse;
import com.ibm.gbs.va.grit.model.response.UploadResumeInfoResponse;
import com.ibm.gbs.va.grit.model.type.EmploymentInfo;
import com.ibm.gbs.va.grit.model.type.Job;
import com.ibm.gbs.va.grit.model.type.Role;
import com.ibm.gbs.va.grit.model.type.SavedJob;
import com.ibm.gbs.va.grit.repository.JobRepository;
import com.ibm.gbs.va.grit.repository.LocationRepository;
import com.ibm.gbs.va.grit.repository.UserRepository;
import com.ibm.gbs.va.grit.util.Consumer;
import com.ibm.gbs.va.grit.util.Converter;
import com.ibm.gbs.va.grit.util.RemoteLog;

import java.util.LinkedList;
import java.util.List;

import javax.inject.Inject;

import static com.ibm.gbs.va.grit.viewmodel.EmploymentViewModel.Tab.Jobs;

public class EmploymentViewModel extends ViewModel {

    private final MutableLiveData<Tab> currentTab = new MutableLiveData<>();

    private final GritMutableLiveData<EmploymentInfo> employmentInfo = new GritMutableLiveData<>();
    private final LiveData<EmploymentInfo> currentEmploymentInfo;

    public final LiveData<List<Job>> currentJobs;
    public final LiveData<Integer> jobSize;

    public final LiveData<List<SavedJob>> savedJobs;

    private final LiveData<SkillsResponse> currentSkills;
    public final LiveData<Integer> skillsSize;

    public final LiveData<Boolean> haveEditsBeenMade;
    public final LiveData<Integer> showResumeGroup;

    public final Range<Integer> salaryRange = new Range<>(10000, 100000);
    public final List<String> citiesList;

    private final JobRepository jobRepo;
    private final UserRepository userRepo;

    private static final String TAG = "Employment";

    @Inject
    public EmploymentViewModel(LocationRepository locationRepository, JobRepository jobRepo, UserRepository userRepo) {
        this.jobRepo = jobRepo;
        this.userRepo = userRepo;

        currentTab.setValue(Jobs);
        citiesList = locationRepository.getCities();

        //Employment Info
        currentEmploymentInfo = jobRepo.getEmploymentInfo();
        currentEmploymentInfo.observeForever(currentEmploymentInfo -> {
            // append resume filename/timestamp if updated
            EmploymentInfo editingInfo = employmentInfo.getValue();
            if (editingInfo != null &&
                    currentEmploymentInfo != null &&
                    !TextUtils.equals(editingInfo.getUploadTimestamp(), currentEmploymentInfo.getUploadTimestamp())) {
                editingInfo.setResumeFilename(currentEmploymentInfo.getResumeFilename());
                editingInfo.setUploadTimestamp(currentEmploymentInfo.getUploadTimestamp());
                employmentInfo.setValue(editingInfo);
            } else {
                employmentInfo.setValue(currentEmploymentInfo != null ? currentEmploymentInfo.duplicate() : null);
            }
        });

        // set up observations
        haveEditsBeenMade = Transformations.map(employmentInfo, employmentInfo -> {
            String prevVal = currentEmploymentInfo.getValue() != null ? currentEmploymentInfo.getValue().toString() : null;
            String newVal = "";
            if (employmentInfo != null) {
                newVal = employmentInfo.toString();
            }
            return !newVal.equals((prevVal));
        });
        showResumeGroup = Transformations.map(employmentInfo, employmentInfo -> {
            boolean show = employmentInfo != null && !TextUtils.isEmpty(employmentInfo.getResumeFilename()) && !TextUtils.isEmpty(employmentInfo.getUploadTimestamp());
            return show ? View.VISIBLE : View.GONE;
        });

        //Jobs
        currentJobs = jobRepo.getJobs();
        jobSize = Transformations.map(currentJobs, jobs -> {
            RemoteLog.d(TAG, "in jobs transformation");
            return jobs != null ? jobs.size() : 0;
        });

        //Saved Jobs
        savedJobs = jobRepo.getSavedJobs();

        //Skills
        currentSkills = jobRepo.getSkills();
        skillsSize = Transformations.map(currentSkills, currentSkills -> {
            RemoteLog.d(TAG, "in currentSkills transformation");
            if (currentSkills != null && currentSkills.getRoles() != null) {
                RemoteLog.d(TAG, "returning data for size: " + currentSkills.getRoles().size());
                return currentSkills.getRoles().size();
            }
            RemoteLog.d(TAG, "returning 0 for size");
            return 0;
        });
    }

    public void onResume() {
        EmploymentInfo info = employmentInfo.getValue();
        if (info != null) info.setUploadTimestamp(info.getUploadTimestamp());
    }

    public void toggleJobType(EmploymentInfo.JobType type, boolean checked) {
        EmploymentInfo info = employmentInfo.getValue();
        if (info != null) {
            List<EmploymentInfo.JobType> list = new LinkedList<>(info.getJobTypes());
            if (!list.contains(type) && checked) {
                list.add(type);
                info.setJobTypes(list);
            } else if (list.contains(type)) {
                list.remove(type);
                info.setJobTypes(list);
            }
        }
    }

    public LiveData<List<String>> getTargetIndustries() {
        return jobRepo.getTargetIndustries();
    }

    public void checkDataAvailable() {
        if (currentEmploymentInfo.getValue() == null) {
            jobRepo.setEmploymentInfo(null);
        } else if (currentSkills.getValue() == null) {
            jobRepo.setSkills();
        } else if (currentJobs.getValue() == null) {
            jobRepo.setJobs();
        }
    }

    public void discardUnsavedResumeInfo() {
        EmploymentInfo currentInfo = currentEmploymentInfo.getValue();
        employmentInfo.setValue(currentInfo != null ? currentInfo.duplicate() : null);
    }

    public LiveData<Boolean> isJobsOrSkillsLoading() {
        return jobRepo.isJobsOrSkillsLoading();
    }

    public LiveData<UploadResumeInfoResponse> updateInfo() {
        return jobRepo.updateEmploymentInfo(employmentInfo.getValue());
    }

    public LiveData<Boolean> uploadedResume(UploadResumeRequest.UploadType uploadType, String img, String filename) {
        return this.jobRepo.uploadedResume(uploadType, img, filename);
    }

    public LiveData<String> uploadComplete() {
        return this.jobRepo.uploadComplete();
    }

    //Jobs
    public Job getJobAt(int index) {
        //noinspection ConstantConditions
        return currentJobs.getValue().get(index);
    }

    public List<Job> getJobs() {
        //noinspection ConstantConditions
        return currentJobs.getValue();
    }

    public boolean isJobsFirstTimeInfo() {
        return userRepo.getOnboardingJobInfoInfo();
    }

    public void setJobsFirstTimeInfo(boolean firstTimeInfo) {
        userRepo.setOnboardingJobInfoInfo(firstTimeInfo);
    }

    //Saved Jobs
    public List<SavedJob> getSavedJobs() {
        return savedJobs.getValue();
    }

    public void addSavedJob(SavedJob savedJob, Consumer<Void> callback) {
        jobRepo.addSavedJob(savedJob, callback);
    }

    public void addSavedJob(Job job, Consumer<Void> callback) {
        jobRepo.addSavedJob(job, callback);
    }

    public void deleteSavedJob(SavedJob savedJob, Consumer<Void> callback) {
        jobRepo.deleteSavedJob(savedJob, callback);
    }

    public void deleteSavedJob(Job job, Consumer<Void> callback) {
        jobRepo.deleteSavedJob(job, callback);
    }

    //Skills
    public Role getRoleAt(int index) {
        //noinspection ConstantConditions
        return currentSkills.getValue().getRoles().get(index);
    }

    public List<Role> getRoles() {
        //noinspection ConstantConditions
        return currentSkills.getValue().getRoles();
    }

    public LiveData<Boolean> updateSkills() {
        return Transformations.map(jobRepo.setSkills(), result -> result != null ? true : null);
    }

    public boolean isSkillsFirstTimeInfo() {
        return userRepo.getOnboardingSkillsInfo();
    }

    public void setSkillsFirstTimeInfo(boolean firstTimeInfo) {
        userRepo.setOnboardingSkillsInfo(firstTimeInfo);
    }

    public SearchSkillsRequest getSkillsRequest() {
        return jobRepo.getSkillsRequest();
    }

    public LiveData<EmploymentInfo> getEmploymentInfo() {
        return employmentInfo;
    }

    public LiveData<EmploymentInfo> getCurrentEmploymentInfo() {
        return currentEmploymentInfo;
    }


    public void onProgressChangedSalary(SeekBar s, int index, boolean fromUser) {
        if (fromUser) {
            if (employmentInfo.getValue() != null) {
                employmentInfo.getValue().setMinimumSalary(Converter.storeRounded(Converter.roundTo1000s(index)));
                s.setProgress(Converter.storeRounded(Converter.roundTo1000s(index)));
            }
        }
    }

    //region Tab Management
    public LiveData<Tab> getCurrentTab() {
        return currentTab;
    }

    public void setCurrentTab(@IdRes int tabId) {
        setCurrentTab(Tab.fromId(tabId));
    }

    public void setCurrentTab(Tab requestedTab) {
        if (requestedTab != currentTab.getValue())
            currentTab.setValue(requestedTab);
    }

    public enum Tab {
        Jobs(0, R.id.jobs),
        SavedJobs(1, R.id.jobs),
        Skills(2, R.id.skills);

        public final int value;
        @IdRes
        public final int menuId;
        private final static SparseArray<Tab> CONSTANTS = new SparseArray<>();

        static {
            for (Tab item : values()) CONSTANTS.put(item.value, item);
        }

        Tab(int value, int menuId) {
            this.value = value;
            this.menuId = menuId;
        }

        public static Tab fromId(@IdRes int tabId) {
            switch (tabId) {
                case R.id.skills:
                    return Skills;
                case R.id.savedJobs:
                    return SavedJobs;
                case R.id.jobs:
                    return Jobs;
                case R.id.jobFilter:
                    return Jobs;
                default:
                    throw new IllegalArgumentException(String.valueOf(tabId));
            }
        }

        public static Tab fromValue(int value) {
            Tab constant = CONSTANTS.get(value);
            if (constant == null) {
                throw new IllegalArgumentException(String.valueOf(value));
            } else {
                return constant;
            }
        }
    }
    //endregion
}
