package com.ibm.gbs.va.grit.viewmodel;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.MutableLiveData;
import android.arch.lifecycle.Transformations;
import android.arch.lifecycle.ViewModel;
import android.support.annotation.Nullable;
import android.support.v4.util.Pair;
import android.util.SparseArray;

import com.ibm.gbs.va.grit.model.notifications.Notification;
import com.ibm.gbs.va.grit.model.survey.Answer;
import com.ibm.gbs.va.grit.model.type.Badge;
import com.ibm.gbs.va.grit.repository.NotificationRepository;
import com.ibm.gbs.va.grit.repository.SurveyRepository;
import com.ibm.gbs.va.grit.repository.UserRepository;
import com.ibm.gbs.va.grit.room.AnswerDao.QuestionAnswerJoin;
import com.ibm.gbs.va.grit.util.Consumer;

import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

public class SurveyViewModel extends ViewModel {

    private final LiveData<List<QuestionAnswerJoin>> questionsAndAnswers;
    /**
     * Guaranteed to be non-null
     */
    private final LiveData<Integer> size;

    private final SurveyRepository surveyRepo;
    private final UserRepository userRepo;
    private final NotificationRepository notificationRepo;

    @Inject
    public SurveyViewModel(SurveyRepository surveyRepo, NotificationRepository notificationRepo, UserRepository userRepository) {
        this.surveyRepo = surveyRepo;
        this.userRepo = userRepository;
        this.notificationRepo = notificationRepo;

        questionsAndAnswers = Transformations.map(surveyRepo.getQuestionsAndAnswers(), qAndAs -> {
            if (qAndAs == null || qAndAs.isEmpty()) {
                return null;
            }

            // filter items for the highest variant id number
            SparseArray<List<QuestionAnswerJoin>> sortingList = new SparseArray<>();
            int largestVariantId = 0;
            for (QuestionAnswerJoin join : qAndAs) {
                int index;
                if (join.getAnswer() != null) {
                    index = join.getAnswer().getQuestionVariantId();
                    //Fix to move to next variant question
                    join.getQuestion().setVariantIndex(join.getAnswer().getQuestionVariantId() - 1);
                } else {
                    index = join.getQuestion().getVariantIdList().get(0);
                }
                largestVariantId = Math.max(largestVariantId, index);
                if (sortingList.get(index, null) == null) {
                    sortingList.put(index, new ArrayList<>());
                }
                sortingList.get(index).add(join);
            }
            return sortingList.get(largestVariantId);
        });

        size = Transformations.map(questionsAndAnswers, list -> {
            if (list != null) return list.size();
            return 0;
        });
    }

    public LiveData<QuestionAnswerJoin> getQuestionAnswerJoinAt(int index) {
        return Transformations.switchMap(questionsAndAnswers, set -> {
            if (set == null) {
                return null;
            }
            MutableLiveData<QuestionAnswerJoin> item = new MutableLiveData<>();
            item.setValue(set.size() > index ? set.get(index) : null);
            return item;
        });
    }

    public void saveAnswer(Answer answer) {
        surveyRepo.updateAnswer(answer);
    }

    public void saveAllAnswers() {
        surveyRepo.updateAnswers(questionsAndAnswers.getValue());
    }

    public void updateCurrentUserQuestions(@Nullable Consumer<Boolean> callback) {
        surveyRepo.updateCurrentQuestionSet(callback);
    }

    public void getNextPlausibleSurvey(Consumer<String> callback) {
        notificationRepo.popSurveyNotification(fetchSucceeded -> {
            if (fetchSucceeded) {
                Notification nextPlausibleSurvey = notificationRepo.getNextAvailableSurveyNotification();
                callback.accept(nextPlausibleSurvey != null ?
                        nextPlausibleSurvey.getTitle().getEn() :
                        null);
            }
        });
    }

    public void queueNextPlausibleSurvey(Consumer<Boolean> callback) {
        notificationRepo.setSurveyNotification(notificationRepo.getNextAvailableSurveyNotification(), callback);
    }

    /**
     * @see SurveyRepository#submitQuestionSet(Consumer)
     */
    public void submitQuestionSet(Consumer<Pair<Boolean, Badge>> callback) {
        surveyRepo.submitQuestionSet(callback);
    }

    //region Getters/Setters
    public boolean isInitialBaselineComplete() {
        return userRepo.getOnboardingInitialBaselineComplete();
    }

    public boolean isSurveyInfoComplete() {
        return userRepo.getOnboardingSurveyInfoComplete();
    }

    public void setSurveyInfoComplete(boolean firstTimeInfo) {
        userRepo.setOnboardingSurveyInfoComplete(firstTimeInfo);
    }

    public LiveData<Integer> getSize() {
        return size;
    }

    public LiveData<List<QuestionAnswerJoin>> getQuestionsAndAnswers() {
        return questionsAndAnswers;
    }

    public int getInProgressPage() {
        return userRepo.getUserInProgressPage();
    }

    public void setInProgressPage(int inProgressPage) {
        userRepo.setUserInProgressPage(inProgressPage);
    }

    public LiveData<String> getCurrentSurveyName() {
        return notificationRepo.getCurrentSurveyName();
    }

    public String getPreviousSurveyName() {
        Notification n = notificationRepo.getPreviousSurveySpawningNotification();
        return n != null ? n.getTitle().getEn() : null;
    }
    //endregion
}
