/**
 * Model for creating and loading Industry collections and Jobs
 */

'use strict';
const Queue = require('better-queue');
var DiscoveryV1 = require('watson-developer-cloud/discovery/v1');
var db = require('../config/db');
var services = require('../config/config').services;
var fs = require('fs');
var async = require('async');
var JSONStream = require('JSONStream');
var path = require('path');
var mkdirp = require('mkdirp');
var fs = require('fs-extra')

var JD_FILE_PATH = "./UploadedJobs/";


var discovery = new DiscoveryV1({
    url: services.watson_discovery.url,
    version: services.watson_discovery.version,
    iam_apikey: services.watson_discovery.apikey
    // iam_url: '<iam_url>', // optional - the default value is https://iam.bluemix.net/identity/token
});

module.exports = function (WEMAdmin) {

    /**
     * Create a Job Industry collection to uploads Jobs
     * API Type: POST
     */
    WEMAdmin.remoteMethod(
        'createCollection', {
            description: "Create a Job Industry collection to uploads Jobs",
            http: {
                path: '/createCollection',
                verb: 'post'
            },
            accepts: [{
                arg: 'collectionName',
                description: "Name of the WDS collection",
                type: 'string'
            }],
            returns: {
                arg: 'status',
                type: 'object'
            }
        }
    );

    /**
     * Create collection in WDS
     */
    WEMAdmin.createCollection = function (collectionName, environmentId, cb) {
        var params = {
            environment_id: services.watson_discovery.environment_id,
            name: collectionName

        };

        discovery.createCollection((params), function (error, data) {
            console.log("result:: ", data, " Error:: ", error);
            cb(null, data);
        });

    };

    /**
     * Upload jobs into a collection. in MongoDB
     * API Type: POST
     */

    WEMAdmin.remoteMethod(
        'uploadJobs', {
            description: "Create a job document in WDS collection",
            notes: [
                "Upload the contents in Content-Type: multipart/form-data.",
                "Form attribute for job description file is 'jdFile'. \n ",
                "Example: \n",
                "curl -X POST --header 'Content-Type: multipart/form-data' --header 'Accept: application/json' \
                -F jdFile=@BurningGlassJobs_2018-10-23.json \
                'http://localhost:3000/api/WEMAdmins/BurningGlass/uploadJobs'"
            ],
            http: {
                path: '/:jobIndustry/uploadJobs',
                verb: 'post'
            },
            accepts: [
                {arg: 'req', type: 'object', 'http': {source: 'req'}},
                {arg: 'res', type: 'object', 'http': {source: 'res'}},
                {arg: 'jobIndustry', type: 'string', description: "Name of the Job Industry"}
            ],
            returns: {
                arg: 'status',
                type: 'string'
            }
        }
    );

    /**
     * Uploads jobs in a Job industry to MongoDB and write each job to a .json for upload to WDS
     * API Type: POST
     */
    WEMAdmin.uploadJobs = function (req, res, jobIndustry, cb) {
        var StorageContainer = WEMAdmin.app.models.UploadJobsContainer;
        StorageContainer.getContainers(function (err, containers) {
            if (containers.some(function (e) {
                return e.name == jobIndustry;
            })) {

                StorageContainer.upload(req, res, {container: jobIndustry}, cb);

            } else {

                StorageContainer.createContainer({name: jobIndustry}, function (err, c) {
                    StorageContainer.upload(req, res, {container: c.name}, cb);
                });
            }
        });
    };


    /**
     * Extract details from uploadJobs POST req
     */
    WEMAdmin.afterRemote('uploadJobs', function (context, remoteMethodOutput, next) {
        var fileName = remoteMethodOutput.status.files.jdFile[0].name;
        var jobIndustry = context.args.jobIndustry;
        var jobCollnParam = {

            "fileName": fileName,
            "envId": services.watson_discovery.environment_id,
            "collectionId": services.watson_discovery.collection_id,
            "jobIndustry": jobIndustry
        };

        createDocsInCollection(jobCollnParam, next);

    });

    /**
     * Upload Jobs to WDS collection
     * API Type: POST
     */
    WEMAdmin.remoteMethod(
        'createDocuments', {
            description: "Upload Jobs from directory to WDS",
            http: {
                path: '/createDocuments',
                verb: 'post'
            },
            accepts: [{
                arg: 'directory',
                description: "directory containing each job as a json",
                type: 'string'
            }],
            returns: {
                arg: 'status',
                type: 'object'
            }
        }
    );

    /**
     * Upload Documents to WDS
     */
    WEMAdmin.createDocuments = function (directory, cb) {
        console.log(JD_FILE_PATH);
        // Make file to write uploaded jobs to
        // const processDir = JD_FILE_PATH + "processDir";
        // fs.mkdirSync(processDir);
        try {
            const files = fs.readdirSync(directory);
            if (files && files.length > 0) {
                console.log(`Queing up ${files.length} files`);
                files.forEach((file) => {
                    const filePath = path.join(directory, file);
                    const stat = fs.statSync(filePath);
                    if (stat.isFile()) {
                        console.log(`Adding ${filePath} to queue`);
                        queue.push(filePath).on('finish', function (data) {
                            if(data){
                            console.log(`WDS Document: ${JSON.stringify(data)}`);
                            let jobKey = file.substring(0, file.lastIndexOf('.'));
                            console.log(`JOBKEY ${jobKey}`);
                            let jobLookup = {"JobID":jobKey , "document_id" : data.document_id};
                            console.log("JOBLOOKUP", JSON.stringify(jobLookup));
                            // Save JobID and WDS Document ID in a lookup table in MongoDB for job deletion
                            db.WDSJoblookupModel.findOneAndUpdate({JobID: jobLookup.JobID}, jobLookup, {upsert: true, new: true}, function (err) {
                                if (err) {
                                    console.log('Error saving job: ' + err);
                                } else {
                                    console.log('Success saving job in DB ' + jobLookup._id);
                                }
                            });
                            // Write uploaded files to new directory
                            // fs.rename(filePath, processDir + "/" + file, function (err) {
                            //     if (err) throw err;
                            //     console.log(`Successfully moved! ${file}`)
                            // });
                            // Delete file after upload
                            fs.unlinkSync(filePath);
                            // fs.rmdir(directory);
                            console.log(`Successfully processed ${filePath}`);
                        }
                        }).on('failed', function (err) {
                            console.log(`Failed to process ${filePath} error reported: ${err}`);
                        });
                    }
                });
            }
        } catch (e) {
            console.error(`Error Reading ${directory} Thrown error: ${e}`);
            cb(null, "failure ");
        }
        cb(null, "success");
    };

    const queue = new Queue(function (file, cb) {
        console.log(`Chomping on file ${file}`);
        const processDir = JD_FILE_PATH + "processDir";
        const myFile = fs.readFileSync(file);
        const params = {
            "environment_id": services.watson_discovery.environment_id,
            "collection_id": services.watson_discovery.collection_id,
            "file": {value: myFile, options: {contentType: 'application/json'}},
            "filename": file
        };
        discovery.addDocument(params, function (error, data) {
            if(error){
                console.log(`Error: ${error}`);
            }
            //console.log(`Added WDS Document ${data.document_id}`);
            //if (error) throw error;
            if(data){
                cb(null, data);
            }
            
        });
    });

    /**
     * Remove jobs old jobs from database and WDS
     * API Type: POST
     */

    WEMAdmin.remoteMethod(
        'removeJobs', {
            description: "Remove state jobs from DB and WDS",
            notes: [
                "Upload the contents in Content-Type: multipart/form-data.",
                "Form attribute for job description file is 'jdFile'. \n ",
                "Example: \n",
                "curl -X POST --header 'Content-Type: multipart/form-data' --header 'Accept: application/json' \
                -F jdFile=@DeletedJobs.json \
                'http://localhost:3000/api/WEMAdmins/removeJobs'"
            ],
            http: {
                path: '/:jobIndustry/removeJobs',
                verb: 'post'
            },
            accepts: [
                {arg: 'req', type: 'object', 'http': {source: 'req'}},
                {arg: 'res', type: 'object', 'http': {source: 'res'}},
                {arg: 'jobIndustry', type: 'string', description: "Name of the Job Industry"}
            ],
            returns: {
                arg: 'status',
                type: 'string'
            }
        }
    );

    /**
     * Remove jobs from Mongo
     */
    WEMAdmin.removeJobs = function (req, res, jobIndustry, cb) {
        var StorageContainer = WEMAdmin.app.models.UploadJobsContainer;
        StorageContainer.getContainers(function (err, containers) {
            if (containers.some(function (e) {
                return e.name == jobIndustry;
            })) {
                StorageContainer.upload(req, res, {container: jobIndustry}, cb);
            } else {
                StorageContainer.createContainer({name: jobIndustry}, function (err, c) {
                    StorageContainer.upload(req, res, {container: c.name}, cb);
                });
            }
        });
    };


    /**
     * Extract details from removeJobs POST req
     */
    WEMAdmin.afterRemote('removeJobs', function (context, remoteMethodOutput, next) {
        var fileName = remoteMethodOutput.status.files.jdFile[0].name;
        var jobIndustry = context.args.jobIndustry;
        var jobCollnParam = {

            "fileName": fileName,
            "envId": services.watson_discovery.environment_id,
            "collectionId": services.watson_discovery.collection_id,
            "jobIndustry": jobIndustry
        };

        removeDocsFromCollection(jobCollnParam, next);

    });

};

/**
 * Create docs in collections
 * @param jobCollnParam
 * @param next
 * @returns
 */
function createDocsInCollection(jobCollnParam, next) {

    var filePath = JD_FILE_PATH + jobCollnParam.jobIndustry + '/' + jobCollnParam.fileName;
    var stream = fs.createReadStream(filePath);

    stream.pipe(JSONStream.parse('*')).on('data', function (data) {
        // console.log('received:', data);
        // stream.pause();

        var reRange = /^[0-9]+-[0-9]+$/;
        if (reRange.test(data.CanonYearsOfExperienceCanonLevel) === true) {
            var expNum = data.CanonYearsOfExperienceCanonLevel.split("-");
            var expMin = expNum[0];
            var expMax = expNum[1];

            function range(start, end) {
                return Array(end - start + 1).fill().map((_, idx) => start + idx);
            }
            
            data.CanonYearsOfExperienceCanonLevel = range(Number(expMin), Number(expMax)).toString();
        } 
        if (data.CanonYearsOfExperienceCanonLevel && data.CanonYearsOfExperienceCanonLevel.includes("+")) {
            var expMin = data.CanonYearsOfExperienceCanonLevel.split("+")[0];
            var expMax = 25;

            function range(start, end) {
                return Array(end - start + 1).fill().map((_, idx) => start + idx);
            }
            
            data.CanonYearsOfExperienceCanonLevel = range(Number(expMin), Number(expMax)).toString();
        }

        if (data.CanonYearsOfExperienceCanonLevel == "") {
            data.CanonYearsOfExperienceCanonLevel = "-1";
        }

        if (data.MaxAnnualSalary != "") {
            data.MaxAnnualSalary = Number(data.MaxAnnualSalary);
        }
        if (data.MaxHourlySalary != "") {
            data.MaxHourlySalary = Number(data.MaxHourlySalary);
        }
        if (data.MinAnnualSalary != "") {
            data.MinAnnualSalary = Number(data.MinAnnualSalary);
        }
        if (data.MinHourlySalary != "") {
            data.MinHourlySalary = Number(data.MinHourlySalary);
        }
        if (data.MaxAnnualSalary == "" || data.MaxAnnualSalary == null) {
            data.MaxAnnualSalary = 1;
        }
        if (data.MaxHourlySalary == "" || data.MaxHourlySalary == null) {
            data.MaxHourlySalary = 1;
        }
        if (data.MinAnnualSalary == "" || data.MinAnnualSalary == null) {
            data.MinAnnualSalary = 1;
        }
        if (data.MinHourlySalary == "" || data.MinHourlySalary == null) {
            data.MinHourlySalary = 1;
        }
        

        // only add jobs to DB/WDS that are in the 5 industries we trained the model on (IT, Engineering, Construction, Management, Logisitics)
        try {
            var naicsSub2 = data.ConsolidatedInferredNAICS.substring(0,2);
            var naicsSub3 = data.ConsolidatedInferredNAICS.substring(0,3);
            var industryCode = ["23", "48", "49", "31", "32", "33", "54", "55", "56", "518", "519"];
            if (data.ConsolidatedInferredNAICS !== undefined && (industryCode.includes(naicsSub2) === true || industryCode.includes(naicsSub3) === true) && data.JobText != "" && data.CanonJobTitle != "") {
                var job = {
    
                    JobID: data.JobID,
                    CleanJobTitle: data.CleanJobTitle,
                    JobDomain: data.JobDomain,
                    CanonCity: data.CanonCity,
                    CanonCountry: data.CanonCountry,
                    CanonState: data.CanonState,
                    JobDate: data.JobDate,
                    JobText: data.JobText,
                    JobURL: data.JobURL,
                    PostingHTML: data.PostingHTML,
                    Source: data.Source,
                    JobReferenceID: data.JobReferenceID,
                    Email: data.Email,
                    CanonEmployer: data.CanonEmployer,
                    Latitude: data.Latitude,
                    Longitude: data.Longitude,
                    CanonIntermediary: data.CanonIntermediary,
                    Telephone: data.Telephone,
                    CanonJobTitle: data.CanonJobTitle,
                    CanonCounty: data.CanonCounty,
                    DivisionCode: data.DivisionCode,
                    MSA: data.MSA,
                    LMA: data.LMA,
                    InternshipFlag: data.InternshipFlag,
                    ConsolidatedONET: data.ConsolidatedONET,
                    CanonCertification: data.CanonCertification,
                    CanonSkillClusters: data.CanonSkillClusters,
                    CanonSkills: data.CanonSkills,
                    IsDuplicate: data.IsDuplicate,
                    IsDuplicateOf: data.IsDuplicateOf,
                    CanonMaximumDegree: data.CanonMaximumDegree,
                    CanonMinimumDegree: data.CanonMinimumDegree,
                    CanonOtherDegrees: data.CanonOtherDegrees,
                    CanonPreferredDegrees: data.CanonPreferredDegrees,
                    CanonRequiredDegrees: data.CanonRequiredDegrees,
                    CIPCode: data.CIPCode,
                    StandardMajor: data.StandardMajor,
                    MaxExperience: data.MaxExperience,
                    MinExperience: data.MinExperience,
                    ConsolidatedInferredNAICS: data.ConsolidatedInferredNAICS,
                    BGTOcc: data.BGTOcc,
                    MaxAnnualSalary: data.MaxAnnualSalary,
                    MaxHourlySalary: data.MaxHourlySalary,
                    MinAnnualSalary: data.MinAnnualSalary,
                    MinHourlySalary: data.MinHourlySalary,
                    YearsOfExperience: data.YearsOfExperience,
                    CanonJobHours: data.CanonJobHours,
                    CanonJobType: data.CanonJobType,
                    CanonPostalCode: data.CanonPostalCode,
                    CanonYearsOfExperienceCanonLevel: data.CanonYearsOfExperienceCanonLevel,
                    CanonYearsOfExperienceLevel: data.CanonYearsOfExperienceLevel,
                    ConsolidatedTitle: data.ConsolidatedTitle,
                    Language: data.Language,
                    BGTSubOcc: data.BGTSubOcc,
                    ConsolidatedDegreeLevels: data.ConsolidatedDegreeLevels,
                    MaxDegreeLevel: data.MaxDegreeLevel,
                    MinDegreeLevel: data.MinDegreeLevel,
                    industry: jobCollnParam.jobIndustry
                    // dbModel : db.gritJobsModel
                };
        
                writeJobJson(jobCollnParam, job, function (err, status) {
                    if (err) {
                        console.log(`ERROR writing job to JSON: ${err}`)
                    }
        
                });
                console.log("Sending Job to DB")
                saveJobsinDB(job, function(err, status) {
                        if (err){
                            console.log(`ERROR parsing JSON: ${err}`);
                        }
        
                    });
            } else {
                console.log("Job Excluded from DB as it is not in one of the 5 industries");
            } 
        } catch (e) {
            console.log("Error reading NAICS");
        }
        
    });

    stream.on('end', function () {
        console.log('end of saving file');
        //Delete the jobs file
        // var filePath = JD_FILE_PATH + jobCollnParam.jobIndustry + '/' + jobCollnParam.fileName;
        // try {
        // 	fs.unlinkSync(filePath);
        // } catch (err) {
        // 	console.log("Error in deleting JD json file");
        // }
        next();
    });
};


/**
 * Save Jobs in DB
 * @param job
 * @param job
 * @param callback
 * @returns
 */


// Upload the doc into MongoDB
function saveJobsinDB(job, callback) {

    var today = new Date().toISOString().split('T')[0];

    if (job.JobID !== undefined) {

        var doc = new db.gritJobsModel();

        doc._id = job.JobID;
        doc.JobID = job.JobID;
        doc.CleanJobTitle = job.CleanJobTitle;
        doc.JobDomain = job.JobDomain;
        doc.CanonCity = job.CanonCity;
        doc.CanonCountry = job.CanonCountry;
        doc.CanonState = job.CanonState;
        doc.JobDate = job.JobDate;
        doc.JobText = job.JobText;
        doc.JobURL = job.JobURL;
        doc.PostingHTML = job.PostingHTML;
        doc.Source = job.Source;
        doc.JobReferenceID = job.JobReferenceID;
        doc.Email = job.Email;
        doc.CanonEmployer = job.CanonEmployer;
        doc.Latitude = job.Latitude;
        doc.Longitude = job.Longitude;
        doc.CanonIntermediary = job.CanonIntermediary;
        doc.Telephone = job.Telephone;
        doc.CanonJobTitle = job.CanonJobTitle;
        doc.CanonCounty = job.CanonCounty;
        doc.DivisionCode = job.DivisionCode;
        doc.MSA = job.MSA;
        doc.LMA = job.LMA;
        doc.InternshipFlag = job.InternshipFlag;
        doc.ConsolidatedONET = job.ConsolidatedONET;
        doc.CanonCertification = job.CanonCertification;
        doc.CanonSkillClusters = job.CanonSkillClusters;
        doc.CanonSkills = job.CanonSkills;
        doc.IsDuplicate = job.IsDuplicate;
        doc.IsDuplicateOf = job.IsDuplicateOf;
        doc.CanonMaximumDegree = job.CanonMaximumDegree;
        doc.CanonMinimumDegree = job.CanonMinimumDegree;
        doc.CanonOtherDegrees = job.CanonOtherDegrees;
        doc.CanonPreferredDegrees = job.CanonPreferredDegrees;
        doc.CanonRequiredDegrees = job.CanonRequiredDegrees;
        doc.CIPCode = job.CIPCode;
        doc.StandardMajor = job.StandardMajor;
        doc.MaxExperience = job.MaxExperience;
        doc.MinExperience = job.MinExperience;
        doc.ConsolidatedInferredNAICS = job.ConsolidatedInferredNAICS;
        doc.BGTOcc = job.BGTOcc;
        doc.MaxAnnualSalary = job.MaxAnnualSalary;
        doc.MaxHourlySalary = job.MaxHourlySalary;
        doc.MinAnnualSalary = job.MinAnnualSalary;
        doc.MinHourlySalary = job.MinHourlySalary;
        doc.YearsOfExperience = job.YearsOfExperience;
        doc.CanonJobHours = job.CanonJobHours;
        doc.CanonJobType = job.CanonJobType;
        doc.CanonPostalCode = job.CanonPostalCode;
        doc.CanonYearsOfExperienceCanonLevel = job.CanonYearsOfExperienceCanonLevel;
        doc.CanonYearsOfExperienceLevel = job.CanonYearsOfExperienceLevel;
        doc.ConsolidatedTitle = job.ConsolidatedTitle;
        doc.Language = job.Language;
        doc.BGTSubOcc = job.BGTSubOcc;
        doc.ConsolidatedDegreeLevels = job.ConsolidatedDegreeLevels;
        doc.MaxDegreeLevel = job.MaxDegreeLevel;
        doc.MinDegreeLevel = job.MinDegreeLevel;
        doc.UploadDate = today;

        db.gritJobsModel.findOneAndUpdate({JobID: job.JobID}, doc, {upsert: true, new: true}, function (err) {
            if (err) {
                console.log('Error saving job: ' + err);
                callback(null, err);
            } else {
                console.log('Success saving job in DB ' + doc._id);
                callback(null, doc); //Return db obj
            }
        });
    }
}

/**
 * Write jobs to Json directory
 * @param jobCollnParam
 * @param job
 * @param callback
 * @returns
 */

function writeJobJson(jobCollnParam, job, callback) {
    var fileName = job.JobID + '.json';
    var jobDate = new Date().toISOString().split('T')[0];;
    var dailyJobsDir = JD_FILE_PATH + jobDate;
    var filePath = JD_FILE_PATH + jobDate + '/' + fileName;

    try {
        if (fileName !== "undefined.json") {
            fs.writeFileSync(filePath, JSON.stringify(job));
            console.log(`${filePath} written to file`)
        }
    } catch (err) {
        mkdirp(dailyJobsDir, function (err) {
            if (err) {
                console.log(`ERROR making directory: ${err}`)
            }
            fs.writeFileSync(filePath, JSON.stringify(job));
            console.log(`${filePath} written to file`)
        });
    }
}

/**
 * Remove Jobs from MongoDB and WDS
 * @param jobCollnParam
 * @param job
 * @param callback
 * @returns
 */

function removeDocsFromCollection(jobCollnParam, next) {
    var filePath = JD_FILE_PATH + jobCollnParam.jobIndustry + '/' + jobCollnParam.fileName;
    var stream = fs.createReadStream(filePath);

    stream.pipe(JSONStream.parse('*')).on('data', function (data) {
        // console.log('received:', data);
        var job = {

            JobID: data.JobID,
            JobAddDate: data.JobAddDate,
            JobDeleteDate: data.JobDeleteDate,
            Is60DayOlderJob: data.Is60DayOlderJob

        };
        removeData(jobCollnParam, job, function (err, status) {
            if (err) {
                console.log(`ERROR parsing JSON: ${err}`);
            }
        });
    });

    stream.on('end', function () {
        console.log('end of saving file');
        //Delete the jobs file
        var filePath = JD_FILE_PATH + jobCollnParam.jobIndustry + '/' + jobCollnParam.fileName;
        try {
            fs.unlinkSync(filePath);
        } catch (err) {
            console.log("Error in deleting JD json file");
        }
        next();
    });
};

function removeData(jobCollnParam, job, csvCallback) {
    console.log('inside removing');
    async.waterfall([
            //#1 Remove the document into MongoDB
            function (callback) {
                // db.gritJobsModel.remove({JobID: job.JobID}, function (err) {
                //     if (err) {
                //         console.log('Error removing job: ' + err);
                //     } else {

                //         console.log('Success removing job in DB ' + job.JobID);
                //         callback(null, job); //Return db obj
                //     }
                // });
                console.log('Skip removing job in DB ' + job.JobID);
                callback(null, job); //Return db obj
            },
            function (job, callback) {
                //  find doc in a WDS collection
                var envId = jobCollnParam.envId;
                var collectionId = jobCollnParam.collectionId;

                db.WDSJoblookupModel.findOne({JobID: job.JobID}, function (err, result) {
                    if (err) {
                        console.log('Error finding job: ' + err);
                    } else {
                        if (result == null) {
                            console.log("Error removing Job from WDS");
                        } else {
                            var wdsDocLookup = result;
                            console.log('Query result:', JSON.stringify(wdsDocLookup));
                            var params = {
                                "environment_id": envId,
                                "collection_id": collectionId,
                                "document_id": wdsDocLookup.document_id
                            };
    
                            // search wds based on document_id and remove from wds
                            discovery.deleteDocument(params, function (error, data) {
                                console.log(`Document successfully removed from WDS`);
                                console.log(JSON.stringify(data, null, 2));
                            });
                        }
                    }
                });
            }
        ],
        function (err, result) {

            csvCallback(null, "success");

        });
};


