
/**
 * Model for User profile creation, Job search and Skills extraction
 */

'use strict';

var async = require('async');
//var Sync = require('sync');
var watson = require('watson-developer-cloud');
var fs = require('fs');
var RESUME_PATH = "./UploadedJobs/CandidateResume/";
var request = require('request');
var parseString = require('xml2js').parseString;
var xmldoc = require('xmldoc');
var db = require('../config/db');
var services  = require('../config/config').services;
var DiscoveryV1 = require('watson-developer-cloud/discovery/v1');
var util = require('util');
var NaturalLanguageUnderstandingV1 = require('watson-developer-cloud/natural-language-understanding/v1.js');
var textract = require('textract');
var http = require("http");
var HashMap = require('hashmap');



var discovery = new DiscoveryV1({
    url: services.watson_discovery.url,
    version: services.watson_discovery.version,
    iam_apikey:services.watson_discovery.apikey
// iam_url: '<iam_url>', // optional - the default value is https://iam.bluemix.net/identity/token
});

module.exports = function(WEMuser) {

    /**
     * API - createUserProfile
     */
    WEMuser.remoteMethod(
        'createUserProfile', {
            description : "Create a User profile in the application",
            notes : [
                "Upload the User profile in below form-data format \n",
                "Content-Type: multipart/form-data \n",
                "-F 'candidateName=Marc-Jimerson' \n",
                "-F 'veteranID=unique ID generated from midtier when user creates profile' \n",
                "-F 'careerHistory=[] (optional)' \n",
                "-F 'previousCompany=XYZ (optional)' \n",
                "-F 'jobRole=some job role (optional)' \n",
                "-F 'moc=25N (military MOS)' \n",
                "-F 'qualifications=some qualifications (optional)' \n",
                "-F 'location=[] (desired location as an array)' \n",
                "-F 'availableIn=30 (optional)' \n",
                "-F 'experience=12 (optional)' \n",
                "-F 'targetedIndustries=[]' \n",
                //"-F 'editprofile=false' \n",
                "-F 'minSalary' \n",
                "-F 'jobType=fulltime' \n",
                "-F 'militaryBranch=Army' \n",
                "-F 'militaryRank=private' \n",
                "-F resumeFile=@'Marc-Jimerson.pdf' \n"
            ],
            http: {
                path: '/createUserProfile',
                verb: 'post'
            },
            accepts: [
                {arg: 'req', type: 'object', 'http': {source: 'req'}},
                {arg: 'res', type: 'object', 'http': {source: 'res'}}
            ],
            returns: {
                arg: 'status',
                type: 'object'
            }
        }
    );

    /**
     * Storing Candidate resume to upload into MongoDB
     */
    WEMuser.createUserProfile = function(req, res, cb) {

        var StorageContainer = WEMuser.app.models.UploadJobsContainer;
        var collectionName = "CandidateResume";

        StorageContainer.getContainers(function (err, containers) {
            if (containers.some(function(e) { return e.name == collectionName; })) {
                StorageContainer.upload(req, res, {container: collectionName}, cb);
            } else {
                StorageContainer.createContainer({name: collectionName}, function(err, c) {
                    StorageContainer.upload(req, res, {container: c.name}, cb);
                });
            }
        });

    };

    /**
     * Extracting candidate's information from the POST request
     */
    WEMuser.afterRemote('createUserProfile', function(context, remoteMethodOutput, next) {
        // validateInput(remoteMethodOutput, profile)
        // if (profile) {
        //     saveCandidateProfile;
        // } else {
        //     next()
        // }

        var resumeFileName = '';
        var moc = '';
        if (remoteMethodOutput.status){
            if (remoteMethodOutput.status.files.resumeFile) {
                resumeFileName = remoteMethodOutput.status.files.resumeFile[0].name;
                console.log("RESUME FILE NAME "+resumeFileName);
            } else {
                if (!remoteMethodOutput.status.fields.moc[0]) {
                    console.log("Either Resume or MOC is needed to create Employment Profile");
                }
            }
        }
        moc = remoteMethodOutput.status.fields.moc[0];
        var reMoc = /^\w+$/;
        if (reMoc.test(moc) === false) {
            next("MOC must be alphanumeric like 11b");
        }
        var veteranID = remoteMethodOutput.status.fields.veteranID[0];
        var candidateName = remoteMethodOutput.status.fields.candidateName[0];
        var careerHistory = remoteMethodOutput.status.fields.careerHistory ? remoteMethodOutput.status.fields.careerHistory[0] : '';
        var previousCompany = remoteMethodOutput.status.fields.previousCompany ? remoteMethodOutput.status.fields.previousCompany[0] : '';
        var jobRole = remoteMethodOutput.status.fields.jobRole[0] ? remoteMethodOutput.status.fields.jobRole[0] : '';
        var qualifications = remoteMethodOutput.status.fields.moc[0] ? remoteMethodOutput.status.fields.qualifications[0] : '';

        // validate location is City, St format
        var location = remoteMethodOutput.status.fields.location[0];
        var reLoc = /([A-Za-z]+(?: [A-Za-z]+)*), ((A[LKSZR])|(C[AOT])|(D[EC])|(F[ML])|(G[AU])|(HI)|(I[DLNA])|(K[SY])|(LA)|(M[EHDAINSOT])|(N[EVHJMYCD])|(MP)|(O[HKR])|(P[WAR])|(RI)|(S[CD])|(T[NX])|(UT)|(V[TIA])|(W[AVIY]))/;
        if (reLoc.test(location) === false) {
            next("Location must be in City, St format");
        }
        var availableIn = remoteMethodOutput.status.fields.availableIn[0] ? remoteMethodOutput.status.fields.availableIn[0] : '';
        var experience = remoteMethodOutput.status.fields.experience[0] ? remoteMethodOutput.status.fields.experience[0] : '';
        var reExp = /^-?\d*(\.\d+)?$/;
        if (reExp.test(experience) === false) {
            next("Experience must be number such as 5 or 5.5");
        }
        var targetedIndustries = remoteMethodOutput.status.fields.targetedIndustries[0];
        if (targetedIndustries.toLowerCase() !== "information technology" && targetedIndustries.toLowerCase() !== "it" && targetedIndustries.toLowerCase() !== "technology" && targetedIndustries.toLowerCase() !== "tech"
        && targetedIndustries.toLowerCase() !== "engineering" && targetedIndustries.toLowerCase() !== "logistics" && targetedIndustries.toLowerCase() !== "management" && targetedIndustries.toLowerCase() !== "construction"
        && targetedIndustries.toLowerCase() !== "") {
            next("Targeted industry must be Information Technology, Engineering, Logistics, Management or Construction. Please select one of these options.");
        }

        if (targetedIndustries.toLowerCase() === "information technology" || targetedIndustries.toLowerCase() === "it" 
        || targetedIndustries.toLowerCase() === "technology" || targetedIndustries === "tech") {
            targetedIndustries = "information technology";
        } 

        
        var minSalary = remoteMethodOutput.status.fields.minSalary[0];
        // var reSal = /^-?\d*(\.\d+)?$/;
        // if (reSal.test(minSalary)) {
        //     next("Minimum Salaray must be a number such as 10000 or 45000.50");
        // }


        var jobType = remoteMethodOutput.status.fields.jobType[0];
        if (jobType.toLowerCase() !== "parttime" && jobType.toLowerCase() !== "part time" && jobType.toLowerCase() !== "part-time" && jobType.toLowerCase() !== "part_time"
        && jobType.toLowerCase() !== "full-time" && jobType.toLowerCase() !== "fulltime" && jobType.toLowerCase() !== "full time" && jobType.toLowerCase() !== "full_time"
        && jobType.toLowerCase() !== "") {
            next("Job Type must be fulltime or parttime.");
        }

        if (jobType.toLowerCase() === "part time" || jobType.toLowerCase() === "part-time" 
        || jobType.toLowerCase() === "part_time" || jobType === "PARTTIME") {
            jobType = "parttime";
        } 

        if (jobType.toLowerCase() === "full time" || jobType.toLowerCase() === "full-time" 
        || jobType.toLowerCase() === "full_time" || jobType === "FULLTIME") {
            jobType = "fulltime";
        } 

        var militaryBranch = remoteMethodOutput.status.fields.militaryBranch[0];
        var militaryRank = remoteMethodOutput.status.fields.militaryRank[0];

        var userProfile = {
            "resumeFileName" : resumeFileName,
            "veteranID" : veteranID,
            "candidateName" : candidateName,
            "careerHistory" : careerHistory,
            "previousCompany" : previousCompany,
            "jobRole" : jobRole,
            "moc" : moc,
            "qualifications" : qualifications,
            "location" : location,
            "availableIn" : availableIn,
            "experience" : experience,
            "targetedIndustries" : targetedIndustries,
            //"editprofile" : editprofile,
            "minSalary" : minSalary,
            "jobType" : jobType,
            "militaryBranch" : militaryBranch,
            "militaryRank" : militaryRank
        };
        // console.log("PROFILE:", userProfile);
        saveCandidateProfile(userProfile, context, next);

    });

    /**
     * API - Get user profile
     */
    WEMuser.remoteMethod(
        'getUserProfile', {
            description : "Get user profile",
            http: {
                path: '/getUserProfile/:veteranId',
                verb: 'get'
            },
            accepts: [{
                arg: 'veteranId',
                type: 'string'
            }],
            returns: {
                arg: 'userProfile',
                type: 'object'
            }
        }
    );

    /**
     * Get user profile stored in MongoDB
     */
    WEMuser.getUserProfile = function(veteranId, cb) {
        console.log(`Fetching profile information for ${veteranId}`);
        getUserProfile(veteranId, cb);
    };


    /**
     * API - Deletes user profile
     */
    WEMuser.remoteMethod(
        'deleteUserProfile', {
            description : "Delete user profile",
            http: {
                path: '/deleteUserProfile/:veteranId',
                verb: 'get'
            },
            accepts: [{
                arg: 'veteranId',
                type: 'string'
            }],
            returns: {
                arg: 'status',
                type: 'object'
            }
        }
    );

    /**
     * Get user profile stored in MongoDB
     */
    WEMuser.deleteUserProfile = function(veteranId, cb) {
        console.log(`Deleting profile information for ${veteranId}`);
        removeProfile(veteranId, cb);
    };

    /**
     * API - Job Search
     */
    WEMuser.remoteMethod(
        'performJobSearch', {
            description : "Perform conceptual job search against the key skills",
            http: {
                path: '/performJobSearch',
                verb: 'post'
            },
            accepts: [
                {arg: 'query', type: 'string', description: 'Key skills of the candidate', required: false },
                {arg: 'isMOS', type: 'boolean', description: 'Job search is based on MOS code or not', required: true, default:'false'},
                {arg: 'isLite', type: 'boolean', description: 'Client needs less attributes on the search result', required: true, default:'false'},
                {arg: 'veteranId', type: 'string', description: 'Pass veteran id for personalized job match', required: false},
                {arg: 'noOfJobs', type: 'number', description: 'No of job search results to be returned', required: true}
            ],
            returns: {
                arg: 'searchresult',
                type: 'object'
            }
        }
    );

    /**
     * Job search based on MOS as well as skills
     */
    WEMuser.performJobSearch = function(query,isMOS, isLite, veteranId, noOfJobs, cb) {
        console.log("recived a performJobSearch req", query);
        var defaultReturnAttr = "JobID,CanonEmployer, CanonJobTitle, JobText,enriched_JobText.entities,CanonYearsOfExperienceCanonLevel, JobURL, CanonJobType,CanonJobHours, CanonCity,CanonState,MSA,MinAnnualSalary,MaxAnnualSalary";
        var liteReturnAttr = "JobID,JobURL, CanonEmployer,CanonJobTitle";
        var filterParam ='';

        async.waterfall([
            function(callback) {
                //#1 Process Custom Query Param
                console.log("VETID INPUT", veteranId);
                getCustomQueryParam(veteranId,callback);
            },
            function (profileWithFilter, callback) {
            console.log("PROFILE WITH FILTER:", profileWithFilter);
            if (profileWithFilter !== null) {
                var profile = profileWithFilter;
                var customQParam = profile.filterParam;
        console.log("****Filter Param 2 ******", customQParam);
                var param = {
                    "environment_id" : services.watson_discovery.environment_id,
                    "collection_id" : services.watson_discovery.collection_id,
                    "natural_language_query" : query,
                    "count" : noOfJobs,
                    "filter" : customQParam,
                    "deduplicate.field" :"JobID",
                    "return": defaultReturnAttr
                };

                if(isLite) {
                    param.return = liteReturnAttr;
                }
                //When Job search is based on MOS, need to extract skills from MOS code from Onet web service
                
                if(isMOS || (!query && !profile)) {
                    async.waterfall([
                        function(callback) {
                            getMOStechnonolgy(profile.moc, callback);
                        }
                    ],function (err, mosSkills) {
                        if(err) {
                            cb(err, null);
                        } else {
                            var mosSkillsQuery = '';
                            for (var i = 0; i < mosSkills.length; i++) {
                              //  query += mosSkills[i] + ', ' ;
                              mosSkillsQuery += mosSkills[i] + ', ' ;
                            }

                            param.natural_language_query = mosSkillsQuery;

                            discovery.query((param), function(error, data) {
                                if(error) {
                                    cb("Error! Unable to get job search result. Please try later.", null);
                                } else {
                                    cb(null, data);
                                }
                            });
                        }
                    });
                } else{
                    console.log("PARAMATERS: ", param);
                    // Check if query input is null or undefined
                    if(param.natural_language_query == null | param.natural_language_query == undefined){
                        console.log(veteranId);
                        // if query is empty extract skills from resume and save to natural_language_query
                     
                        var vetSkillList = [];
                        extractKeySkills(veteranId, function(skills, response) {
                                    
                                    var vetSkillsArray = response.entities;
                                    vetSkillsArray.forEach(function(vetSkill){
                                        vetSkillList.push(String(vetSkill.text.toLowerCase()));
                                    });
                                    console.log("CANDIDATE SKILL LIST: ", vetSkillList);
                       
                        if(vetSkillList.length >0){
                                var vetSkillsQuery = '';
                                for (var i = 0; i < vetSkillList.length; i++) {
                                vetSkillsQuery += vetSkillList[i] + ', ' ;
                                }
                                param.natural_language_query = vetSkillsQuery;

                        }
                    console.log("*** DISCOVERY PARAM **" + param.filter + "vetQuery" +param.natural_language_query);
                    //Skill based job search from user input/resume
                    discovery.query((param), function(error, data) {
                        if(error) {
                            cb("Error! Unable to get job search result.", null);
                        } else {
                            console.log("*** DISCOVERY CALLED **"+JSON.stringify(data));
                            cb(null, data);
                        }

                        });
                    });
                    }
                    else{

                    
                    console.log("*** DISCOVERY PARAM **" + param.filter + "query" +param.natural_language_query);
                    //Skill based job search from user input/resume
                    discovery.query((param), function(error, data) {
                        if(error) {
                            cb("Error! Unable to get job search result.", null);
                        } else {
                            cb(null, data);
                        }

                    });
                }
                
            } 
            } else {
                cb("NO_WEM_PROFILE",null);
            }
                
            // function end
            }
            
        ]);
    };


    /**
     * API - Get Personality Insights of the User
     */
    WEMuser.remoteMethod(
        'getPersonalityInsights', {
            description : "Get Personality Insights of the User",
            http: {
                path: '/getPersonalityInsights/:veteranId',
                verb: 'get'
            },
            accepts: [{
                arg: 'veteranId',
                type: 'string'
            }],
            returns: {
                arg: 'result',
                "description" : "Personality Insights from Watson Personality Service",
                type: 'object'
            }
        }
    );


    /**
     * Getting personality insights from Watson Service
     */
    WEMuser.getPersonalityInsights = function(veteranId, cb) {
        //Fetch resume text from user profile
        db.gritCandidateModel.findOne({ veteranID: veteranId }, 'resumeText').exec(function (err, profile) {
            if(err) {
                console.error(err.reason);
                cb(null, "Error in finding profile. Please try after sometime");
            } else {
                var PersonalityInsightsV3 = require('watson-developer-cloud/personality-insights/v3');
                var personality_insights = new PersonalityInsightsV3({
                    username: services.personality_insights.username,
                    password: services.personality_insights.password,
                    version_date: '2016-10-20'
                });

                var params = {
                    // Get the content items from the JSON file.
                    text: profile.resumeText,
                    consumption_preferences: true,
                    raw_scores: true,
                    headers: {
                        'accept-language': 'en'
                    }
                };

                personality_insights.profile(params, function(error, response) {
                        if (error) {
                            console.log('Error:', error);
                        } else {
                            cb(null, response);
                        }
                    }
                );
            }
        });
    };

    /**
     * API - Get Candidate's key skills from resume
     */
    WEMuser.remoteMethod(
        'getCandidateSkills', {
            description : "Get Candidate's key skills from resume",
            http: {
                path: '/getCandidateSkills/:veteranId',
                verb: 'get'
            },
            accepts: [{
                arg: 'veteranId',
                type: 'string'
            }],
            returns: {
                arg: 'result',
                "description" : "Personality Insights from Watson Personality Service",
                type: 'object'
            }
        }
    );

    /**
     * Extracting candidate's key skill using Natural Language Understanding(NLU) service
     */
    WEMuser.getCandidateSkills = function(veteranId, cb) {

        //Fetch resume text from user profile
        console.log(`Fetching skills for veteranId ${veteranId}`);
        extractKeySkills(veteranId, cb);
    };

    /**
     * API - Perform Skill Gap Analysis
     */
    WEMuser.remoteMethod(
        'performSkillGapAnalysis', {
            description : "Perform skill gap analysis comparing veteran's extracted skills to Hot Skills in market",
            http: {
                path: '/performSkillGapAnalysis',
                verb: 'post'
            },
            accepts: [
                {arg: 'veteranId', type: 'string', description: 'Pass veteran id for personalized job match', required: true},
                {arg: 'desiredIndustry', type: 'string', description: 'Collect desired Industry for skill gap analysis', required: false},
                {arg: 'yearsOfExp', type: 'string', description: 'Collect total number of years of work experience to narrow suggested roles', required: false},
                {arg: 'minSalaryExpectation', type: 'string', description: 'Collect salary expectation to narrow suggested roles', required: false},
                {arg: 'preferredLocation', type: 'string', description: 'Collect salary expectation to narrow suggested roles', required: false}
            ],
            returns: {
                arg: 'skillGapResult',
                type: 'object'
            }
        }
    );

    /**
     * Job search based on MOS as well as skills
     */
    WEMuser.performSkillGapAnalysis = function(veteranId, desiredIndustry, yearsOfExp, minSalaryExpectation, preferredLocation, cb) {
        console.log("Received a performSkillGapAnalysis req");


        extractKeySkills(veteranId, function(skills, response) {
            if (response !== null) {
                var vetSkillList = [];
            var vetSkillsArray = response.entities;
            vetSkillsArray.forEach(function(vetSkill){
                vetSkillList.push(vetSkill.text.toLowerCase());
            });
            console.log("CANDIDATE SKILL LIST: ", vetSkillList);

            //Fetch user profile except resume text
            db.gritCandidateModel.findOne({ veteranID: veteranId }, "-resumeText").exec(function (err, profile) {
                if(err) {
                    console.error(err.reason);
                } else {
                    // build veteran profile to query BG API with skills
                    var vetProfile = {
                        "veteranId" : veteranId
                    };
                    vetProfile.desiredIndustry = !desiredIndustry ? profile.targetedIndustries[0] : desiredIndustry;
                    vetProfile.yearsOfExp = !yearsOfExp ? profile.experience : yearsOfExp;
                    vetProfile.minSalaryExpectation = !minSalaryExpectation ? profile.minSalary : minSalaryExpectation;
                    vetProfile.preferredLocation = !preferredLocation ? profile.location[0] : preferredLocation;


                     var skillGapResults = [];
                     var skillsArray =[];
                     var simSkillsArray =[];

                    // Start of Retrieve Job Roles and their required skills

                    fetchJobRoleAndSkills(vetProfile, function(err, BGdata) {
                    // var BGdata = JSON.parse(fs.readFileSync("/Users/kristakinnard/sb/ibm/va-grit-cognitive/WEMapp/wem-master/BurningGlassResponse.json"));
                    console.log("Invoke BG API");
                    // console.log("Data from BG***" +JSON.stringify(BGdata));
                    // console.log("BGdata.response" + BGdata.response);
		    
		    if(!err & !BGdata.error){ 
                        skillsArray = BGdata.response.data;
                        // console.log ("SKILLS Array Length "+ JSON.stringify(skillsArray));

                        fetchSimilarSkills(vetSkillList, function(err, BGdata) {
                           var simSkills = BGdata.response.result;
                           simSkills.forEach(function(name) {
                               simSkillsArray.push(name.name);
                           });
                            var vetAugSkillsList = vetSkillList.concat(simSkillsArray);
                            console.log("VET COMBINED SKILLS: " + JSON.stringify(vetAugSkillsList));
                            skillsArray.forEach(function(elem) {
                                // console.log("ELEMENTS: ", JSON.stringify(elem));
                                var role = {};
                                role.jobRoleName = !elem.name ? "" : elem.name;
                                role.jobRoleDesc = !elem.description ? "" : elem.description;
                                role.industry = !elem.careerArea.name ? "" : elem.careerArea.name;
                                role.meanExperience = !elem.experience ? "" : elem.experience.mean;
                                role.meanSalary = !elem.salary ? "" : elem.salary.mean;
                                role.demandLevel = !elem.demand ? "" : elem.demand.level;
                                role.employmentGrowth = !elem.employment ? "" : elem.employment.growth;

                                var baselineSkillsArray = elem.skills.baselineSkills;
                                var baselineSkills = [];
                                baselineSkillsArray.forEach(function(baselineSkill) {
                                    var oneSkill = {};
                                    var isVetSkill = [];
                                    vetAugSkillsList.forEach(function(vetSkill){
                                        var vetComp = vetSkill.includes(baselineSkill.name.toLowerCase());
                                        if (vetComp === true) {
                                            isVetSkill.push(vetComp);
                                        }
                                    });
                                    oneSkill.name = !baselineSkill.name ? "" : baselineSkill.name;
                                    oneSkill.description = !baselineSkill.description ? "" : baselineSkill.description;
                                    oneSkill.isVeteranSkill = isVetSkill.length > 0;
                                    baselineSkills.push(oneSkill);
                                });

                                var specializedSkillsArray = elem.skills.specializedSkills;
                                var specializedSkills = [];
                                specializedSkillsArray.forEach(function(specializedSkill) {
                                    var oneSkill = { };
                                    var isVetSkill = [];
                                    vetAugSkillsList.forEach(function(vetSkill){
                                        var vetComp = vetSkill.includes(specializedSkill.name.toLowerCase());
                                        if (vetComp === true) {
                                            isVetSkill.push(vetComp);
                                        }
                                    });
                                    oneSkill.name = !specializedSkill.name ? "" : specializedSkill.name;
                                    oneSkill.description = !specializedSkill.description ? "" : specializedSkill.description;
                                    oneSkill.isVeteranSkill = isVetSkill.length > 0;
                                    specializedSkills.push(oneSkill);
                                });

                                role.skills = {
                                    "baselineSkills" : baselineSkills,
                                    "specializedSkills" : specializedSkills
                                };

                                skillGapResults.push(role);
                            });
                            cb(null, skillGapResults);
                    });
		    
		    //End of if Error condition from BG
		    }
                       else{ cb("ERROR in RETRIEVING JOB ROLES", skillGapResults);}
			
                    // End for JobRole Retreival function
                    });
                }
            });
            } else {
                cb("NO_WEM_PROFILE", null);
            }
            
            //end of function
        });
    };
};


/**
 * Save Candidate's keys skills into Data base
 * @param profile
 * @param context
 * @param next
 * @returns
 */
function saveCandidateProfile(profile, context, next) {

    var userProfile = profile;

    async.waterfall([
            function(callback) {
                //#1 MOS processing
                getMOStechnonolgy(userProfile.moc, callback);
            },
            function (mosHotSkills, callback) {
                //Append mosHotSkills in profile
                userProfile.mocSkills = mosHotSkills;
                //#2 Convert resume
                if(userProfile.resumeFileName != ''){
                    var fileExtentionSplit = userProfile.resumeFileName.split(/[\s.]+/);
                    var fileExtention = fileExtentionSplit[fileExtentionSplit.length-1];
                    var resumePath = RESUME_PATH + userProfile.resumeFileName;
                    if (fileExtention === "pdf" | fileExtention === "doc" | fileExtention === "docx" | fileExtention === "txt") {
                        textract.fromFileWithPath(resumePath, function(err, text) {
                            if (err) {
                                // ERROR Handling here
                                console.log('Error extracting text:', err);
                                // callback(err);
                            } else {
                                console.log(`Read Resume data for: ${text}`);
                                callback(null, text);
                            }
                        });
                    } else {
                        console.log("File type not supported");
                        callback("File Type not supported");
                    }
                }
                else{
                    callback(null, '');
                }
            },
            function(resumeText, callback) {
                //#3. Save/Update candidate's profile into MongoDB
                saveProfileInDB(resumeText, userProfile, callback);
            }
        ],
        function (err, result) {
            if(err) {
                console.log("Veteran registration failed!: " + result);
                var response = {
                    "status" : "error",
                    "veteranId" : "Unable to create or update candidate profile. Check the Resume file type and check that the MOS is valid"
                };
            } else {
                console.log("Candidate saved successfully! veteranId: " + result);
                var response = {
                    "status" : "success",
                    "veteranId" : result
                };
            }
            context.result = response;
            try {
                next()
            } catch (e) {
                console.log("An Error Occurred while saving profile. Please check input criteria.");
            }
            ;
        });
}

/**
 * Save profile into MongoDB
 * @param resumeText
 * @param profile
 * @param parentCallback
 * @returns
 */
function saveProfileInDB(resumeText, profile, parentCallback) {

    var doc = new db.gritCandidateModel();
    // db.gritCandidateModel.insert({veteranID: 1}, {unique: true});

    doc.candidateName = profile.candidateName;
    doc.veteranID = profile.veteranID;
    console.log(doc._id);
    doc.resumeText = resumeText;
    doc.careerHistory = profile.careerHistory;
    doc.previousCompany = profile.previousCompany;
    doc.jobRole = profile.jobRole;
    doc.moc = profile.moc;
    doc.mocSkills = profile.mocSkills;
    doc.qualifications = profile.qualifications;
    doc.location = profile.location;
    doc.availableIn = profile.availableIn;
    doc.experience = profile.experience;
    doc.targetedIndustries = profile.targetedIndustries;
    doc.minSalary = profile.minSalary;
    doc.jobType = profile.jobType;
    doc.militaryBranch = profile.militaryBranch;
    doc.militaryRank = profile.militaryRank;

    var msaLookup = JSON.parse(fs.readFileSync('./UploadedJobs/msa_lookup/lookup_MSA.json'));
    var msaSearch = msaLookup.filter(function(msaLookup){
        return(doc.location === msaLookup.City);
    });

    try {
        doc.msa = `${msaSearch[0]["CBSA"]}: Metropolitan Statistical Area`;
    } catch (e) {
        doc.msa = "";
    }

    console.log("MSA RESULT: ", doc);

    //If document exists satisfying the find criteria, it will update existing record.
    //Else insert a new document

    db.gritCandidateModel.findOneAndUpdate({veteranID:profile.veteranID},doc,{upsert: true, new:true},function (err,doc) {
        if (err) {
            console.log('Error saving veteran: ' + err);
            callback(err);
        } else {
            console.log('Success saving veteran');
            parentCallback(null, doc.veteranID);
        }
    });


}

/**

 Build Custom Discovery Filter based on Profile Paramters
 **/

function getCustomQueryParam( veteranId, parentCallback) {

    var filterParam ='';
    console.log("VETERANID", veteranId);
    if(veteranId){
        
        db.gritCandidateModel.findOne({ veteranID: veteranId }).exec(function (err, profile) {
            if(err) {
                console.error(err.reason);
            } else { 
        console.log("PROFILE~~~~~~", profile);              
		if(profile !=null){

                    // Create Experience Range

                    
                    if(profile.experience != null && profile.experience != ""){
                        var yearsOfExp = parseInt(profile.experience);
                        var maxExpRange =yearsOfExp+1;
                        var minExpRange = 0;
                       if(yearsOfExp > 0){
                            minExpRange =yearsOfExp-1;}

                              filterParam =filterParam +'CanonYearsOfExperienceCanonLevel:'+yearsOfExp+'|'+maxExpRange+'|'+minExpRange+',';
                        }

                    // If MSA Available
                     if(profile.msa !=null && profile.msa != ""){filterParam =filterParam +'MSA:"'+profile.msa+'",';}


                    // Use Location only when MSA isn't available.
		            // Location value in profile has state as well for e.g Dallas, TX : so extract city & state 
                   else {
                        if(profile.location != null && profile.location != "")
                            {
                            filterParam =filterParam +'CanonCity:"'+profile.location[0].split(",")[0]+'",';
                            filterParam =filterParam +'CanonState:"'+profile.location[0].split(",")[1]+'",';
                            }
                        }
                    // JobType in profile has hours fulltime/parttime hence map it
                    if(profile.jobType != null && profile.jobType != ""){filterParam =filterParam +'CanonJobHours:"'+profile.jobType[0]+'",';}

                    console.log("****Filter Param ******", filterParam);
                    // Removes the last "," which is any following filed is empty or null
                    filterParam = filterParam.replace(/,\s*$/, "");
                      profile.filterParam =filterParam;
                      console.log("PROFILE2", profile.filterParam);
        }
        parentCallback(null,profile);
       
		
                // filterParam = 'CanonCity:'+profile.location+','+',CanonJobType:'+profile.CanonJobType+',CanonYearsOfExperienceLevel:'+profile.CanonYearsOfExperienceLevel+';         }
            }});
    }
    else{
        console.log("****Filter Param ****** EMPTY", filterParam);
        parentCallback(null,filterParam);
    }
}



/**
 * Extracting candidate's key skill using Natural Language Understanding(NLU) service - Local function
 */
function extractKeySkills(veteranId, cb) {

    //Fetch resume text from user profile
    db.gritCandidateModel.findOne({ veteranID: veteranId }, 'resumeText mocSkills').exec(function (err, profile) {
        if(err) {
            console.error(err.reason);
            cb(null, "Error in finding profile. Please try after sometime");
        } else {
            var natural_language_understanding = new NaturalLanguageUnderstandingV1({
                'url': services.nlu.url,
                'version': services.nlu.version_date,
                'iam_apikey':services.nlu.apikey
                // iam_url: '<iam_url>', // optional - the default value is https://iam.bluemix.net/identity/token
            });
           if (profile !== null) {
            var skillsText ='';
            if(profile.resumeText !=null && profile.resumeText !=''){
                skillsText = profile.resumeText;
            }
            else{
 
                 skillsText = profile.mocSkills.toString();
 
            }
            console.log("SKILLS TEXT TO BE EXTRACTED"+skillsText);
             var parameters = {
                 'text': skillsText,
                 'features': {
                     'entities': {
                         'emotion': false,
                         'sentiment': false,
                         'limit': 20,
                         'model' : services.nlu.model_id
                     }
                 }
             };
             natural_language_understanding.analyze(parameters, function(err, response) {
                 if (err) {
                     console.log('error:', err);
                 } else {
                     //console.log(JSON.stringify(response, null, 2));
                     cb(null, response);
                 }
             });
           } else {
               cb("NO_WEM_PROFILE", null);
           }
            // end
        }
    });
}
/**
 Get ONET Technology list for a MOS code
 */

function getMOStechnonolgy(MOS, parentcallback) {

    async.waterfall([

        function(callback) {
            //1. Get job position for MOS
            console.log("1. Get job position for MOS");
            console.log("MOS", MOS);
            var service = "veterans/military?keyword=" + MOS;
            var url = services.onetcenter.onet_ws + service;
            var separator = service.indexOf('?') !== -1 ? '&' : '?';
            url += separator + 'client=employment_manager';
            var body = '';
            //console.log("URL", url);

            request.get(url).on('data', function(data) {
                body += data;
            }).on('response', function(response) {
                console.log(response.statusCode);
                console.log(response.headers['content-type']);
            }).on('end', function() {
                parseString(body, function (err, result) {
                    //console.log(util.inspect(result));
                    var totalResult = parseInt(result.careers.$.total);
                    //console.log("totalResult::: ", totalResult);
                    if(totalResult === 0) {
                        var error = "Invalid MOS code";
                        callback(error, null);
                    } else {
                        var jobCode = result.careers.career[0].code[0];
                        console.log(jobCode);
                        callback(null, jobCode);
                    }
                });
            });
        }, function(jobCode, callback) {
            //2. Get technology from job code
            console.log("2. Get technology from job code");
            var mosSkills = [];
            var service = "veterans/careers/" + jobCode + '/technology';
            var url = services.onetcenter.onet_ws + service;
            var separator = service.indexOf('?') !== -1 ? '&' : '?';
            url += separator + 'client=employment_manager';
            var body = '';

            request.get(url).on('data', function(data) {

                body += data;
            }).on('response', function(response) {

                console.log(response.statusCode); // 200
                console.log(response.headers['content-type']) // 'image/png'
            }).on('end', function() {

                var hotSkills = [];
                var document = new xmldoc.XmlDocument(body);
                document.eachChild(function(child, index, array){

                    var myTech = child.childrenNamed("example");
                    for (var i = 0; i < myTech.length; i++) {
                        var skills = myTech[i].val;
                        hotSkills.push(skills);
                    }
                });
                console.log("HOTSKILLS", hotSkills);
                callback(null, hotSkills); //Return skills
            });

        }
            // console.log("*************MOS SKILLS", mosSkills.concat(hotSkills, genSkills));
    ],function (err, result) {
        console.log("::MOS Hotskills:: " + JSON.stringify(result) );
        if(err) {
            parentcallback(err, null);
        } else {
            parentcallback(null, result);
        }
    });
}

function getUserProfile(veteranId, cb){
    //Fetch user profile except resume text
    db.gritCandidateModel.findOne({ veteranID: veteranId }, "-resumeText").exec(function (err, profile) {
        if(err) {
            console.error(err.reason);
            cb(null, "Error in finding profile. Please try after sometime");
        } else {
            cb(null, profile);
        }
    });
}

// Deletes all records associated with the user profile
function removeProfile(veteranId, cb){
    //Delete user profile records
    db.gritCandidateModel.deleteMany({ veteranID: veteranId }).exec(function (err, profile) {
        if(err) {
            console.error(err.reason);
            cb(null, "Error in deleting profile. Please try after sometime");
        } else {
          //  cb(null, profile);
           cb(null, "Profile associated with VeteranID "+veteranId+" sucessfully deleted.");
        }
    });
}

// Fetches Job Roles and their required skills matching veteran's skills & preferences.

function fetchJobRoleAndSkills(profile, parentcallback) {

    var vetProfile = profile;
    console.log("VETPROFILE:", vetProfile);
    async.waterfall([

        function(callback) {
            //1. Get oAuth Token for BG API
              console.log("1. Get Burning Glass API Access Token");
           
              getAccessToken(callback);


        }, function(accessToken, callback) {
            //2. Get JobRoles & Skills matching veteran profile
	    
	    console.log("2. Get JobRoles & Skills matching veteran profile");
            var minSalaryExpectation = 10000;
            var vetExp = 1;
            if(vetProfile.minSalaryExpectation !=null && vetProfile.minSalaryExpectation !=''){
               minSalaryExpectation = parseInt(vetProfile.minSalaryExpectation);
               var maxSalaryRange = minSalaryExpectation+20000;
            }
            if(vetProfile.yearsOfExp !=null && vetProfile.yearsOfExp !=''){
             vetExp = parseInt(vetProfile.yearsOfExp);
            }
            var maxExpRange =vetExp+1;
            var minExpRange =vetExp-1;

            //Mapping between Application Industries to BG Career Areas

            var industryMap = new HashMap();
            industryMap.set("IT","Information Technology");
            industryMap.set("Logistics","Transportation");
            industryMap.set("Engineering","Engineering");
            industryMap.set("Construction","Construction, Extraction, and Architecture");
            industryMap.set("Management","Business Management and Operations");
            industryMap.set("Information Technology","Information Technology");

           console.log ("VETERAN DESIRED INDUSTRY:::"+vetProfile.desiredIndustry);

           // To Do Add profile location in the body - vetProfile.preferredLocation

            var occupationBody = '{"careerArea":["'+industryMap.get(vetProfile.desiredIndustry)+'"],"salary":{"ge":'+minSalaryExpectation+'},"experience":{"le":'+maxExpRange+',"ge":'+minExpRange+'}}';
	    
	    
	    var options = { method: 'POST',
                             url: services.burningGlass.occupation_API_URL,
                             headers: 
                            {
                                'cache-control': 'no-cache',
                                authorization: 'Bearer '+accessToken,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           
                                'content-type': 'application/json',
                                accept: 'application/json' },
                            //  body: '{"careerArea":["Information Technology"],"salary":{"le":90000,"ge":50000},"experience":{"le":6,"ge":4}}' };
                           
                                body: occupationBody
                            };


                console.log("occupationBody****"+occupationBody);
                request(options, function (error, response, body) {
                if (error) throw new Error(error);

                    // console.log(body);
                    var jsonBody = JSON.parse(body);
                    callback(null, jsonBody);

    });

    }
    ],function (err, result) {
        console.log("::Job Roles:: " + JSON.stringify(result) );
        if(err) {
            parentcallback(err, null);
        } else {
            parentcallback(null, result);
        }
    });
}

// Get API Token for Burning Glass

function getAccessToken(callback){

            //1. Get oAuth Token for BG API
            console.log("1. Get Burning Glass API Access Token");
            var body ='';
            var tokenFormParam = "{form:{'username':'"+services.burningGlass.username+"', 'password':'"+services.burningGlass.password+"', 'grant_type':'"+services.burningGlass.grant_type+"','client_id':'"+services.burningGlass.client_id+"'}}";
           // var tokenPOST = "{'url:"+services.burningGlass.oAuth_URL+",body:"+tokenFormParam+"}";


            var options = { method: 'POST',
             url: services.burningGlass.oAuth_URL,
             headers: 
                {'cache-control': 'no-cache' },
            form: 
            { grant_type: services.burningGlass.grant_type,
              client_id: services.burningGlass.client_id,
              username: services.burningGlass.username,
              password: services.burningGlass.password } };

            request(options, function (error, response, body) {
                if (error) throw new Error(error);

                    console.log(body);
                    var jsonBody = JSON.parse(body);
                    console.log("***ACCESS TOKEN ***"+jsonBody.access_token);
                    var accessToken = jsonBody.access_token;
                  //  callback(null, vetProfile, accessToken);
                  callback(null, accessToken);

    });
}


// Fetch Similar Skills

function fetchSimilarSkills(skillsArray, parentcallback) {

    var skillsArrayInput = skillsArray;

    async.waterfall([

        function(callback) {

            getAccessToken(callback);
            
        }, function(accessToken, callback) {
            //2. Get Similar Skills matching input skills
            console.log("2. Get Similar skills");

            //var skillsArray = ["Java","Python"];
            var similarityBody = `{"input": {"skill": ${JSON.stringify(skillsArrayInput)}}}`;

            //body: '{"input":{"skill":["java","java","java"]}}'


            var options = { method: 'POST',
                             url: services.burningGlass.similarity_API_URL,
                             headers: 
                            {
                                'cache-control': 'no-cache',
                                authorization: 'Bearer '+accessToken,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           
                                'content-type': 'application/json',
                                accept: 'application/json' },
                            //  body: '{"careerArea":["Information Technology"],"salary":{"le":90000,"ge":50000},"experience":{"le":6,"ge":4}}' };
                           
                                body: similarityBody
                            };


                console.log("*******" + JSON.stringify(options));
                request(options, function (error, response, body) {
                if (error) throw new Error(error);

                console.log(body);
                var jsonBody = JSON.parse(body);
                callback(null, jsonBody);

                });
        }
    ],function (err, result) {
        console.log("::Job Roles:: " + JSON.stringify(result) );
        if(err) {
            parentcallback(err, null);
        } else {
            parentcallback(null, result);
        }
    });
}

