# Risk Service Project

## Tech Stack
- System-wide Components
    - Oracle JDK 1.8, update 192
    - Maven 3.5.2
    - JUnit 5.3.0
    - MongoDB Java Driver 3.82
    - MongoDB 4.0.4
- Rules Engine
    - Drools 7.14
    - KIE API 7.14
    - Apache Commons Math
- REST API
    - Jersey 2.27
        - Grizzly HTTP Server
        - MOXy 
- HTTP Client
    - OkHttp 3.11.0
    - Gson 2.8.5

## Development Environment
- Eclipse configuration:
    - Download and install Eclipse.
    - From the Eclipse marketplace, install either JBoss Tools or Red Hat Developer Studio.
    - Download plugin called, Drools and jBPM tools, from http://drools.org/download/download.html.
    - Extract the folder and install the plugin in Eclipse using Help -> Install new software... -> Add. See https://www.tutorialspoint.com/drools/drools_eclipse_plugin.htm for screenshots and more details.
    - Use the Drools perspective from Window->Perspective->Open Perspective->Other...->Drools.
    - Clone the repository and then open the project by going to File->Open Project from File System->Directory and selecting the repository directory.
- Other IDEs or text editors can be used for Drools but possibly without some convenience features for rules development such as syntax highlighting and rules browsers.
- Maven is the preferred way to manage dependencies for this project.
    - Use the Maven Eclipse plugin commands below to use Eclipse.
        - `mvn eclipse:clean`
        - `mvn eclipse:eclipse`
    - Use jar packaging instead of kjar packaging as kjar packaging can cause errors with the Eclipse IDE m2e plugin.
- Testing Environment
    - Install Docker
        - Use httpbin to catch all notifications: `sudo docker run -p 80:80 kennethreitz/httpbin`
    - Install MongoDB
        - Start the MongoDB instance locally after installing using the default port.
        - Use the mongo shell to verify results
    - Use the config.properties file located in src/main/resources to change between production and test settings. Set `production` to false to make API calls to 0.0.0.0 otherwise the calls will go to the mid-tier. Set the mlab setting to false to send MongoDB calls to 0.0.0.0 otherwise the calls will go to the MLAB hosted MongoDB instance.
        - The boolean `PRODUCTION` in `RiskServiceResource` allows for easy switching between production and developmental testing environments for API calls to the GRIT internal services at the mid-tier.
        - The boolean `MLAB` in `RiskServiceResource` allows for easy switching between a local MongoDB instance and the mLab hosted instance.

## Project Structure
- main/java/com/ibm/grit/rest: Contains REST API related classes
    - ScoreResource: This is the class that provides the REST API service when the end point is called. Ensure that the correct MongoDB connection string is being used for production.
        - Ensure that the correct MongoClientSettings parameter is used. Use testSettings for testing and development and productionSettings for deployment and production.
- main/java/com/ibm/grit/scoring: Contains risk scoring related classes
- main/java/com/ibm/grit/utils: Contains utility classes and functions
    - FlagsUtils: Utilities for flags rules including parsing of the score request data fields.
    - NotificationsUtils: Utilities for handling notification creation and updates
        - sendNotification: Change REST end point to production server when deploying
        - updateNotification: Change REST end point to production server when deploying
    - ScoringUtils: Utilities primarily for unit testing scoring computations
- main/resources: Contains the packages for the rules files. See the rules section for more information.
    - META-INF: This folder contains the kmodule.xml that defines which rules files to load during a KnowledgeSession
- test/resources: Contains JSON test files as well test shell scripts. The test JSON files are useful for testing and debugging with data similar to what will be used in production. Multiple JSONs are available for different unique users.
    - The naming convention on the JSONs are as follows:
        - Prefix with question set name
        - Suffix with testing vetId
            - The letters in the vetId indicate risk level. For example:
                - LRM - low risk male
                - MRF - medium risk female
                - HRM - high risk male
                - MHRM - medium to high risk male
        - Life changes numbers indicates which answer options were selected. For example:
            - `LIFE_CHANGES_1_HRM` means `Life_Changes_Ans1` was selected. 
            - `LIFE_CHANGES_12_HRM` means `Life_Changes_Ans1` and `Life_Changes_Ans2` were both selected.
        - Mood faces surveys follow a similar naming convention as life changes surveys.

## Docker
- Base image: maven:3.5.2-jdk-8
- Image name: "risk"
- risk-service image runs on port 5500 mapped to 5500
- Build with Dockerfile from CLI
    - Command to build: `sudo docker build --tag risk .`
    - Command to run detached and network: `sudo docker run -td --name risk-service -p 5500:5500 risk`
    - Command to run and show console (use to debug): `sudo docker run -t --name risk-service -p 5500:5500 risk`
    - Command to see running containers: `sudo docker ps`
    - Command to stop container: `sudo docker stop risk-service`
    - Command to remove container: `sudo docker rm {CONTAINER ID}` `CONTAINER ID` is obtained from the `ps` command output
- Build with docker-compose
    - Command to build: `sudo docker-compose -p risk-service up --build --detach`
    - Command to recreate: `sudo docker-compose -p risk-service up --force-recreate --build --detach`
    - Command to start: `sudo docker-compose -p risk-service start --detach`
    - Command to stop: `sudo docker-compose -p risk-service stop`

## Design Choices
- Rules Engine
    - Drools was chosen because it is the most powerful and well-maintained open-source rule engine implementation available. It is actively developed and supported by JBoss and Red Hat.
    - Fact Classes
        - Question set fact classes are defined in accordance with the associated question sets that are asked to users during the baseline assessment. The questions are converted to metrics and sent to risk service which will parse metrics to populate associated fact classes. Each fact class has a personId field which identifies the user who answered the question set. This allows each question set to stored independently in the database without depending on the other question sets.
            - `BasicNeeds`
            - `Employment`
            - `Health`
            - `Mental`
            - `Military`
            - `Social`
            - `Mood`
            - `Tweet`
        - The `RiskAssessment` fact class is used to store the risk scores and risk levels for each user. The users are uniquely identified with the `personId` field which is the same as the `personId` field in the question set classes. Each `RiskAssessment` object is timestamped during instantiation in the `timestamp` field. The risk assessment scores aren't actually computed at instantiation time, but rather just afterwards during risk scoring rules execution. Hence, the timestamp indicates when the risk assessment was started not when it was completed and may be off from the completion time by about one second.
        - `Category` is an enumerative type which allows unique identification of data types used in the scoring rules.
        - `Score` is fact class that is used to accumulate and sum scores during rules execution.
        - `Weights` is a fact class used to store weights for scoring.
        - `ScoreRequestData` is a fact class that matches the incoming JSON data that is sent from the mid-tier to the risk service.
        - `Notification` is a fact class supporting sending notifications to the REST API internal/notifications/createNotification.
    - Stateless execution
        - The rules sessions are executed using the default stateful KIE session but are effectively used as stateless sessions. State is inserted into the session, the rules are executed and the session is immediately disposed of. No long running sessions occur.
        - Rules execution flow:
            - create session
            - set global variables
            - insert facts into the session
            - fire rules
            - dispose session 
        - To handle stateless rules execution, state must be persisted somewhere. All state necessary for rules execution is stored in a MongoDB database called `risk`. State data must be read from the database and inserted into the rules session and updates and new state must be written to the database. This design choice allows for more robust stateless execution as the risk service can go down without losing valuable state data. Once the risk server is back online, it can easily read the state from the database back into memory and continue execution. See the section on database design for more information.
- Database Design (MongoDB)
    - MongoDB allows for persistent storage of BSON objects which are JSON-like objects. Many Java classes can be cleanly represented as a BSON object in MongoDB, hence storage is simply a of matter of having the right Java classes to store and way to convert between JSON, Java, and BSON.
    - CRUD operations, including object conversion, are handled by the official [MongoDB Java Driver](http://mongodb.github.io/mongo-java-driver/).
        - In accordance with the driver recommendation, a single MongoClient instance is used for DB operations. We achieve this by using static objects of DB operations. This may help prevent race conditions from occurring during DB operations.
    - Database schema (*Note: number of objects stored per user differs depending on the collection*)
        - `risk` database
            - `data`: collection of all `ScoreRequestData` objects per user
            - `basicneeds`: collection of one `BasicNeeds` object per user
            - `employment`: collection of one `Employment` object per user
            - `health`: collection of one `Health` object per user
            - `mental`: collection of one `Mental` object per user
            - `military`: collection of one `Military` object per user
            - `mood`: collection of all `Mood` objects per user
            - `assessment`: collection of all `RiskAssessment` object per user
            - `notification`: collection of one `Notification` object per type per user
            - `sms`: collection of one (most recent) `Sms` object per user
            - `tweet`: collection of all `Tweet` objects per user

- REST API
    - Endpoints are located at http://risk-service:5500/risk-service/.
    - The calls should be made from the mid-tier with a POST containing JSON data. See SwaggerHub for more details.
    - /score
        - The `score` API is the primary API for the risk service and will update flags, perform risk assessments, and create notifications.
        - The JSON data contains the unique `vetId`, `surveyType`, `questionSetId`, and `metrics`.
        - Metrics must be parsed from the JSON data and written to MongoDB.
        - The JSON is data is passed directly into the rules engine as a `ScoreRequestData` object using the flags.drl rule set. The flags are retrieved from the database if they exist for that user and also inserted into the rules engine session.
        - The different rules files makes heavy use of global variables for interactions with the different MongoDB collections. Global variables are listed below:
            - `MongoCollection<BasicNeeds> bnCollection`
            - `MongoCollection<Employment> empCollection`
            - `MongoCollection<Health> heaCollection`
            - `MongoCollection<Mental> mentCollection`
            - `MongoCollection<Military> milCollection`
            - `MongoCollection<Social> socCollection`
            - `MongoCollection<Notification> notifCollection`
            - `List<Flag> nextSessionFlags`
            - `MongoCollection<Mood> moodCollection`
            - `MongoCollection<Sms> smsCollection`
            - `MongoCollection<Tweet> tweetCollection`
    - /purgeData
        - The `purgeData` API is used to delete all data stored in the risk database for the specified user. This action is irreversable.
    - /getMoodScores
        - The `getMoodScores` API will return all previous mood scores for the user.
    
- Control Flow
    - The flow of control for the risk service score API -> flags rules -> scoring rules -> notification rules.


## Flags
- Flags are sent back to the mid-tier to indicate which question sets should be asked in the session.
    - Two sets session flags are tracked:
        - `currentSessionFlags` are sent to the mid-tier in response to the `score` API call by the mid-tier.
        - `nextSessionFlags` are sent to the mid-tier in the `createNotifications` API call by the risk service in the string form of a JSON array in the `metadata` field of the request.
    - Generally, only the currentSessionFlags are manipulated.
    - List of available flags ordered by index. The session flags are implemented as `List`s of `Flag`s and thus manipulation of the flags is most easily achieved by treating each list as a tuple and getting and setting elements by index position.
        -  0: `reassess`
        -  1: `afterBL`
        -  2: `askBLCustom2`
        -  3: `askBLCustom3`
        -  4: `askBLCustom4`
        -  5: `askRABN`
        -  6: `askRAEmp`
        -  7: `askRAEmpMoUnemp`
        -  8: `askRAHea`
        -  9: `askRAEmp`
        - 10: `askRAEmpMoUnemp`
        - 11: `askRAMent`
        - 12: `askRAMentCond`
        - 13: `askRASoc`
        - 14: `askRASocMoMaritalChg`

## Rules
Rules files are defined for each type of risk category as well as notifications rules and flag rules. 

### Rules general considerations
The developer writing rules should keep the following points in mind:
- Rules have a left-hand side (LHS) which is the conditional portion of the rule. The LHS uses Drools Rules Language (DRL).
- Rules have a right-hand side (RHS) which is the action taken if all conditions in LHS are met. The RHS is written in plain Java with some additional Drools functions and variables allowed.
- Rules can only operate on objects in working memory. Hence, the facts necessary for rules operations must be inserted into memory either before running the rules or during a knowledge session.
- Global variables are not accessible for rules to reason over (LHS). However, global variables are useful for rules actions (RHS).
- In general, rules should be atomic. That is, rules should do one thing and not able to be broken down into further sets of smaller rules.
- In general, rules should be independent. That is, rules should be loosely coupled and have the ability to be added or removed from a rules file without affecting the operation of other rules. Some exceptions to this guiding principle are necessary (e.g. to prepare working memory by performing preprocessing tasks).
- Rules cannot directly reason over the contents of Java Collections. To reason over Collections, the LHS of the rule must use something like `collect`, `from`, or `accumulate`. See Drools documentation for details on these functions.
- Many of the functions called in the rules are defined in the corresponding utility class as `public static` methods and imported into the rules files.
- Some of the rules files declare their own classes and functions. Generally, if rules files declare their own classes and functions, it is because those classes or functions are specific only to that rules file.
- Rules names must be unique within the session they are running. Because all of the different scoring rules run within the same session, each scoring rule must be unique from each other despite being in different files.
- The kmodule.xml file contained in src/main/resources/META-INF is used to define the rules files to use and the operational mode of the knowledge sessions.
- The `salience` section of the rule helps control the flow of rule execution.
- When the knowledge session starts, all rules are immediately evaluated for execution by checking the LHS of each rule. `salience` does not control rule LHS evaulation time which usually occurs immediately at the start of a session.
- Generally, much of the state that is inserted into the working memory of each knowledge session is passed through the writing and reading from MongoDB.
- By convention, variables bound in LHS are prefixed with `$` to make it easy for developers to distinguish between fields and bound variables.

### flags rules
Flags rules handle the following:

- The setting and unsetting of `currentSessionFlags` and `nextSessionFlags`
- Updating metrics data in the database.

The flags rules rely heavily on the FlagsUtils class to provide functions used in the rules. The use of functions provides much cleaner and easier to maintain rules.

The update methods in the FlagsUtils class map all of the metrics to a `Map` and checks for the existence of `String` keys which correspond to the Question IDs defined in the GRIT questions and scoring spreadsheet. Hence, undefined keys are ignored during operation which allows for robustness against undefined Question IDs during operation. Additionally, the update methods only overwrite the metrics that were actually parsed from the metrics. Hence, previously existing data that was not sent in the metrics remain intact and can be used in the scoring rules.

The flags rules start by detecting what type of survey was provided and then reads and parses the metrics into the appropriate fact classes. The classes are then added or updated in the database and inserted into working memory.

Generally, after the fact classes are inserted into working memory, the rules handle updating the `currentSessionFlags` and `nextSessionFlags`. `nextSessionFlags` is a global variable. `nextSessionFlags` starts as an empty `List` and `Flag` objects are added to in during the rules session. Once flags rules execution halts, the `nextSessionFlags` should be populated with the appropriate `Flag`s. The `Flag`s in `nextSessionFlags` will be individually inserted into working memory for the notifications rules session.

### scoring rules
The risk service has six different rules files for each of different types or risk scores.

- basic-needs-risk.drl
- employability-risk.drl
- health-risk.drl
- mood-risk.drl
- social-risk.drl
- suicide-risk.drl

The `AllKS` ksession in the kmodule.xml file allows all scoring rules to be run simultaneously.

Each set (rule file) of risk rules must have the appropriate scoring fact classes inserted into working memory for proper scoring. For example, the basic-needs-risk depends on the BasicNeeds fact class to score. Therefore the KnowledgeSession needs the appropriate BasicNeeds object for the user to be inserted into working memory.

The scoring rules generally compute risk as the sum of various weighted risk components. The more complicated scoring equations have an associated equation text file showing the mathematical equivalence of the summation-based scoring used by the scoring rules.

### notifications rules
Notifications rules handle the following:

- Scheduling of BASELINE surveys using the nextSessionFlags that were set by flags rules.
- Scheduling new LIFE_CHANGES and MOOD_FACES surveys based on risk levels that were set by the scoring rules.
- Updating existing LIFE_CHANGES and MOOD_FACES surveys based on increased risk levels.

Notifications rules relies heavily on the NotificationsUtils class to provide functions used by the rules.

Working memory goes through multiple changes during notifications rules execution.
- To start, the `ScoreRequestData`, all previous `Notification`s, the last 180 `RiskAssessment`s, all `Flag`s in `nextSessionFlags` are inserted into working memory.
- During execution, a declared type `LowRisk` is inserted into working memory if the user had low risk in all categories over the last 90 days.
- Next, all but the most recent two `RiskAssessment`s are deleted from working memory.
- Next, a declared type `PreviousRisk` is populated with the risk levels and employment status from the previous (not current) `RiskAssessment`. 
- Next, a declared type `AllowNotification` with `notificationType = DIGITAL_ASSISTANT` is logically* inserted into working memory if a DIGITAL_ASSISTANT notification type was not generated in the last 14 days.
- Next, a declared type `AllowNotification` with `notificationType = NOTIFY_SQUAD` is logically* inserted into working memory if the squad was not notified in the last 14 days.
- If an IMMEDIATE_HELP notification is sent, then a NotificationSent object is inserted into working memory to prevent multiple IMMEDIATE_HELP being sent in the same session.

*logical insertion is inserting a fact and deleting that fact from working memory when the conditions for insertion no longer exist. See Drools documentation on `insertLogical` for more details.

## Open Issues
- Automated testing of all REST API functionality not implemented
- Suicide scoring unit tests are not passing due to updated scoring.
- Social scoring unit tests are not passing due to updated scoring.
- Multi-rule set unit test are not passing due to updated scoring.
- There is no easy way to test mood or sentiment scoring without changing the rules due to the 7 day minimum time between earliest and most recent values in the last 30 days.
- Documentation

## References
- Drools: https://docs.jboss.org/drools/release/latestFinal/drools-docs/html_single/
- MongoDB Java Driver: http://mongodb.github.io/mongo-java-driver/3.8/
- OkHttp: https://github.com/square/okhttp/wiki
- Gson: https://github.com/google/gson/blob/master/UserGuide.md
- Jersey: https://jersey.github.io/documentation/latest/index.html
- JUnit 5: https://junit.org/junit5/docs/current/user-guide/



