import numpy as np
from value_dicts import G, P, H, M, S, thresholds, default
from veteran import Veteran
from pprint import pprint


def calc_suicide_risk(vet: Veteran) -> (str, float, list):
    """
    Suicide Risk
    = (G2 sex value)/5 *
    0: [ (clip to 1 (P1 values: Depression + Manic-depressive + PTSD + other)/2 ) +
    1: (clip at 1( P2: sex based 'drinks/day' value)) +
    2: (clip at 1(P3: sum of drugs of abuse)/1.5) +
    3: (clip to 1(P4:psych med values)/2 ) +
    4: (sum(P5 questions: Mental visits, reduct.Rank, recentSep_Divorce)/3) ]

    """

    score_arr = np.zeros(5)
    score_categories = ['pysch Dx', 'drinks', 'drugs of abuse', 'psych meds', 'psych visits; rank reduction; recent sep/divorce']

    if vet.G2_sex is None:
        sex = 'default'
    else:
        sex = vet.G2_sex

    # 0: [ (clip to 1 (P1 values: Depression + Manic-depressive + PTSD + other)/2 )
    if is_default(vet.P1_psych_dx):  # Default case, no values checked
        score_arr[0] = P[1]['default']
    elif vet.P1_psych_dx['none']:
        pass  # value in array already initialized to 0
    else:
        keys = ('depression', 'manic-depression', 'PTSD', 'other')
        s = 0
        for k in keys:
            if vet.P1_psych_dx[k]:
                s += P[1][k]
        score_arr[0] += s
        score_arr[0] /= 2.0
        score_arr[0] = min(1, score_arr[0])  # Clip at 1

    # 1: (clip at 1( P2: sex based 'drinks/day' value))
    if vet.P2_weekly_drinks is None:
        score_arr[1] = 0.75
    else:
        score_arr[1] = vet.P2_weekly_drinks / P[2][sex]
        score_arr[1] = min(1, score_arr[1])  # Clip at 1

    # 2: (clip at 1(P3: sum of drugs of abuse)/1.5)
    if is_default(vet.P3_drugs):  # Default case, no values checked
        score_arr[2] = P[3]['default']
    elif vet.P3_drugs['none']:
        pass  # value in array already initialized to 0
    else:
        s = 0
        for k in vet.P3_drugs:
            if vet.P3_drugs[k]:
                s += P[3][k]
        score_arr[2] += s
        score_arr[2] /= 1.5
        score_arr[2] = min(1, score_arr[2])

    # 3: (clip to 1(P4:psych med values)/2)
    if is_default(vet.P4_psych_drugs):  # Default case, no values checked
        score_arr[3] = P[4]['default']
    elif vet.P4_psych_drugs['none']:
        pass  # value in array already initialized to 0
    else:
        s = 0
        for k in vet.P4_psych_drugs:
            if vet.P4_psych_drugs[k]:
                s += P[4][k]
        score_arr[3] += s
        score_arr[3] /= 2.0
        score_arr[3] = min(1, score_arr[3])

    # 4: (sum(P5 questions: Mental visits, reduct.Rank, recentSep_Divorce)/3)
    if vet.P5_annual_psych_visits is None:
        score_arr[4] = P[5]['default']
    else:
        if vet.P5_annual_psych_visits == 0:
            if not is_default(vet.P1_psych_dx):
                if not vet.P1_psych_dx['none']:
                    score_arr[4] += 1
        else:
            psych_visits_score = vet.P5_annual_psych_visits/P[5]['visits divisor']
            psych_visits_score = min(1, psych_visits_score)
            score_arr[4] += psych_visits_score

        score_arr[4] += M[5][vet.M5_rank_reduction]
        if vet.S1_marriage in S[1]:
            score_arr[4] += S[1][vet.S1_marriage]
        else:
            score_arr[4] += S[1]['default']
        score_arr[4] /= 3

    score = score_arr.sum() * G[2][sex] / score_arr.size

    cat = threshold(score, 'suicide')

    return cat, score, list(zip(score_categories, score_arr))


def calc_basic_needs_risk(vet: Veteran) -> (str, float, list):
    """
    Basic Needs Risk =
    0: Housing value +
    1: Transportation value +
    2: Food value
    3: Financial value

    Scale array by 1/4
    """
    score_arr = np.zeros(4)
    score_categories = ['housing', 'transportation', 'food', 'financial']

    if not vet.G6_housing:  # default case
        score_arr[0] = G[6]['default']
    else:
        score_arr[0] = G[6][vet.G6_housing]

    if not vet.G7_transportation:
        score_arr[1] = G[7]['default']
    else:
        score_arr[1] = G[7][vet.G7_transportation]

    if not vet.G8_food:
        score_arr[2] = G[8]['default']
    else:
        score_arr[2] = G[8][vet.G8_food]

    if not vet.G9_savings:
        score_arr[3] = G[9]['default']
    else:
        score_arr[3] = G[9][vet.G9_savings]

    score = score_arr.sum() / score_arr.size

    cat = threshold_basic_needs(score)
    return cat, score, list(zip(score_categories, score_arr))


def calc_employability_risk(vet: Veteran) -> (str, float, list):
    """
    Employability Risk
    [
    0: (clip decimal yrs unemployed at 2yr) / 2.0 +
    1: (disability flag + serious military injury) / 2 +
    2: ((manicDepressive val /2) + PTSD value)/2 +
    3: ( (1 - officer) + (1 - Education) ) / 2 +
    4: (conflict + housing + (alcohol score) ) / 3
    ]
    Sum and scale by 1/5
    """

    score_arr = np.zeros(5)
    score_categories = ['years unemployed',
                        'disability & serious mil injury',
                        'manic depression; PTSD',
                        'officer; education',
                        'conflict; housing; alcohol']

    # 0: (clip decimal yrs unemployed at 2yr) / 2.0
    if vet.E1_employed is None:  # Default (unanswered) choice
        score_arr[0] = 0.75
    elif not vet.E1_employed:
        years_unemp = vet.E2_months_unemp / 12.0
        score_arr[0] = min(2.0, years_unemp) / 2.0

    # 1: (disability flag + serious military injury) / 2
    try:
        disability = vet.H3_chronic_cond['physical disability']
        score_arr[1] = (disability + M[2][vet.M2_mil_inj]) / 2.0
    except KeyError:
        score_arr[1] = 0.75

    # 2: ((manicDepressive val /2) + PTSD value)/2
    if vet.P1_psych_dx['none']:
        pass
    else:
        if vet.P1_psych_dx['manic-depression']:
            score_arr[2] = P[1]['manic-depression'] / 2.0
        else:
            score_arr[2] += 0.75

        if vet.P1_psych_dx['PTSD']:
            score_arr[2] += P[1]['PTSD']
        else:
            score_arr[2] += 0.75

        score_arr[2] /= 2.0

    # 3: ( (1 - officer) + (1 - Education) ) / 2. TODO check on education being flipped
    if vet.M1_officer:
        officer_val = M[1]['officer']
    else:
        officer_val = M[1]['enlisted']

    try:
        score_arr[3] = ((1 - officer_val) + (S[5][vet.S5_education])) / 2.0
    except KeyError:
        score_arr[3] = 0.75

    # 4: (conflict + housing + (alcohol score) ) / 3
    try:
        score_arr[4] = (S[3][vet.S3_conflict] +
                        G[6][vet.G6_housing] +
                        (vet.P2_weekly_drinks / P[2][vet.G2_sex])) / 3.0
    except KeyError:
        score_arr[4] = 0.75

    score = score_arr.sum() / score_arr.size
    cat = threshold(score, 'employability')
    return cat, score, list(zip(score_categories, score_arr))


def calc_health_risk(vet: Veteran) -> (str, float, list):
    """
    Health Risk =
    0: 2 * (sum of 6 select Chronic Health diseases) /6 +
    1: Activity value +
    2: Coverage value +
    3: Compliance value
    Scale by 1/5
    """
    score_arr = np.zeros(4)
    score_categories = ['chronic disease', 'activity', 'coverage', 'compliance']

    # 0: 2 * (sum of 6 select Chronic Health diseases) /6
    try:
        score_arr[0] = 2 * sum(vet.H3_chronic_cond.values()) / 6
    except TypeError:
        score_arr[0] = 0.75

    # 1: Activity value
    try:
        score_arr[1] = H[1][vet.H1_activity]
    except KeyError:
        score_arr[1] = 0.75

    # 2: Coverage value
    coverage = 'none'
    try:
        if sum(vet.H2_coverage.values()) == 1:
            coverage = 'one'
        elif sum(vet.H2_coverage.values()) > 1:
            coverage = 'two'
        score_arr[2] = H[2][coverage]

    except TypeError:
        score_arr[2] = 0.75


    # 3: Compliance value
    try:
        score_arr[3] = H[5][vet.H5_compliance]
    except KeyError:
        score_arr[3] = 0.75

    score = score_arr.sum() / 5

    cat = threshold(score, 'health')
    return cat, score, list(zip(score_categories, score_arr))


def calc_social_risk(vet: Veteran) -> (str, float, list):
    """
    Social Risk =
    0: ( 1- Interpersonal marriage value ) +
    1: ( Interpersonal  friend value) +
    2: 1.5 * (conflict value) +
    3: .5 * Religion value +
    4: .5 * Education value)

    Scale by 1/4.5
    """
    score_arr = np.zeros(5)
    score_categories = ['marriage', 'friend', 'conflict', 'religion', 'education']

    # 0: ( 1 - Interpersonal marriage value ). TODO flip S1
    try:
        score_arr[0] = S[1][vet.S1_marriage]
    except KeyError:
        score_arr[0] = 0.75

    # 1: ( Interpersonal friend value). TODO flip S2
    try:
        score_arr[1] = 1 - min(1, vet.S2_interactions / S[2])
    except TypeError:
        score_arr[1] = 0.75

    # 2: 1.5 * (conflict value)
    try:
        score_arr[2] = 1.5 * S[3][vet.S3_conflict]
    except KeyError:
        score_arr[2] = 0.75

    # 3: .5 * Religion value. TODO religion scoring is off
    try:
        score_arr[3] = 0.5 * S[4][vet.S4_religion]
    except KeyError:
        score_arr[3] = 0.75

    # 4: .5 * Education value)
    try:
        score_arr[4] = 0.5 * S[5][vet.S5_education]
    except KeyError:
        score_arr[4] = 0.75

    score = score_arr.sum() / 4.5

    cat = threshold(score, 'social')
    return cat, score, list(zip(score_categories, score_arr))


def threshold(score: float, cat: str):
    if score < thresholds[cat]['l']:
        return 'low'
    elif score < thresholds[cat]['m']:
        return 'med'
    else:
        return 'hi'


def threshold_basic_needs(score: float):
    if score == thresholds['basic']['l']:
        return 'low'
    elif score < thresholds['basic']['m']:
        return 'med'
    else:
        return 'hi'


def is_default(d):
    """
    Checks if all values are set to None to determine the default case.
    :param d: dict of veteran values to check
    :return: True if all None, else False
    """
    for k in d:
        # if d is None:
        #     return False
        if d[k] is not None:
            return False
    return True

