# VA-GRIT Microservices

This readme file describes the VA-GRIT Midtier components and the set up for additional Loopback microservice environments.


# Requirements
- Docker
- Docker Compose
- Loopback (required to create a new microservice, not required to run the app or update existing microservices)

# Components

- App components each run in their own separate Docker container and each has their own Dockerfile.
- Docker Compose (./docker-compose.yml) manages building each container and networks them together so they can communicate.
- The Gateway is the only component accessible to the public


- Gateway - built with Express-Gateway
  - The Gateway provides the microservices' APIs to the public.
  - Acts as an intermediate between the microservices and the outside world.
  - Allows for microservices to remain private behind the gateway.
  - Allows to combine many services into a single API Service Point.
  - https://www.express-gateway.io
  - https://www.npmjs.com/package/express-gateway


- Microservices - built with LoopBack
  - The APIs behind the Gateway that the Gateway exposes to the public.
  - To create a microservice start a Loopback project with basic User auth. (See 'How to create a new microservice')
  - (http://loopback.io)


- MongoDb
  - Database that sits in its own Docker container.
  - This is created in docker-compose.yml
  - BE CAREFUL: Don't delete a MongoDb Container that has data.

# Running with docker and docker-compose

  This project consists of multiple Docker Containers to run each sub-project (the Gateway and Microservices). Each sub-project includes a Dockerfile in its root directory (and can be built and run on it's own via Docker CLI Commands or Node CLI). However the components are meant to work together as defined in Docker-compose.yml in the root folder of this project.

## Running all the services via docker-compose
- From console, go to project root directory

### To Build, Run, and Observe in logs
Use the "up" Docker command to build, run, and observe

IMPORTANT: For docker-compose.dev.yml change the .dev to .local or .prod depending on the environment you are building.

```
docker-compose -f ./midtier/docker-compose.dev.yml -p va-grit-midtier-gateway up
```
Can include the --force-recreate and --build flags to insure everything rebuilds
```
docker-compose -f ./docker-compose.dev.yml -p va-grit-midtier-gateway up --force-recreate --build  
```

### To Start Docker Compose project
Will start all containers in project (if already built with "up" command)
```
docker-compose -f ./midtier/docker-compose.dev.yml -p va-grit-midtier-gateway start
```

### To Stop Docker Compose project
Will stop all containers in project
```
docker-compose -f ./midtier/docker-compose.dev.yml -p va-grit-midtier-gateway stop
```

## URL
Once started (locally), the services will be available at:
```
http://localhost:8080
```
- Port 8080 points to the Gateway
- The exposed port can be changed in va-grit-midtier-gateway/config/gateway.config.yml

# Gateway

See the va-grit-midtier-gateway/README.md for more


# Microservices

## Current Microservices

- Vet
  - va-grit-midtier-microservice-vet
  - Signup, Authentication, User Info
- Survey
  - va-grit-midtier-microservice-survey
  - Related to the surveys, questions, and answers

## How to create a new microservice

### Requirements
- LoopBack (http://loopback.io)

### 1. Create new loopback project
- From the va-grit-midtier root folder

Create new loopback project
```
slc loopback
```
Change to new loopback project directory
```
cd new_microservice_project_folder
```

### 2. Change the port in server/config.json
- Give each microservice (and other components) its own port number.
- For a Loopback server change the port in <b>server/config.json</b>
- Make sure it doesn't match any other services port number.


### 3. Configure for mongodb
From the root folder of the new Loopback microservice project. Run ....
```
npm install loopback-connector-mongodb --save
```

Add the vaGritMidtierMongoDb to <b>server/datasources.json</b> - this will be the same database the other microservices are using
```
# server/datasources.json

"vaGritMidtierMongoDb": {
  "host": "va-grit-midtier-mongo-db",
  "port": 27017,
  "url": "",
  "database": "vaGritMidtier",
  "password": "",
  "name": "vaGritMidtierMongoDb",
  "user": "",
  "connector": "mongodb"
}
```
Configure Loopback Built-in and Custom models to use <b>vaGritMidtierMongoDb</b> dataSource
```
# server/model-config.json

"User": {
  "dataSource": "vaGritMidtierMongoDb"
},
"AccessToken": {
  "dataSource": "vaGritMidtierMongoDb",
  "public": false
},
"ACL": {
  "dataSource": "vaGritMidtierMongoDb",
  "public": false
},
"RoleMapping": {
  "dataSource": "vaGritMidtierMongoDb",
  "public": false,
  "options": {
    "strictObjectIDCoercion": true
  }
},
"Role": {
  "dataSource": "vaGritMidtierMongoDb",
  "public": false
},
"aCustomModel": {
  "dataSource": "vaGritMidtierMongoDb",
  "public": true
},
etc...
```

### 4. In server/config.json 

- Change restApiRoot
- Change port

### 5. Add /npm-packages/va-grit-microservices

The va-grit-microservices npm package contains code common to microservices, such as:
- Get userId from AccessToken
- Response Formatting

###### This is a temporary work around to add the va-grit-microservices npm package to a microservice.

- Create a folder called npm-packages
- Copy /common/npm-packages/va-grit-microservices in the root va-grit-midtier folder to the /npm-packages folder
-  Use the Dockerfile (see Configure for Docker) to install package AFTER normal npm install (For some reason, local npm packages install fails if referenced in package.json)


### 6. Configure for Docker
Add a Dockerfile to root folder of new Loopback microservice project
```
# Dockerfile in root folder of project

FROM node:8.11.3-onbuild
WORKDIR /app
## nodemon will restart server on changes, also need to mount volume in docker-compose.yml
RUN npm install -g nodemon
COPY . /app
RUN npm install

# Install Custom Local NPM Packages AFTER normal npm install
# For some reason, local npm packages install fails if referenced in package.json
RUN npm install ./npm-packages/va-grit-microservices

# Expose same port indicated in server/config.json
EXPOSE 5001
```
```
Add new microservice to Docker-compose.yml in va-grit-midtier root folder
```

### 7. Add APIs to Gateway
To add this microservice's API's to the gateway see gateway.config.yml:

```
va-grit-midtier-gateway/config/gateway.config.yml
```

# EXTRA: To Run microservice alone on localhost for developing

This app includes a Docker file as is meant to work with docker-compose project.  

However, the project can be run from it's root directory without docker by running:

```
npm start
```
- NOTE DEPENDENCY ISSUES COULD EXIST IF RUNNING LOCALLY AND NOT RUNNING IN THE DOCKER-COMPOSE APP, PARTICULARLY IF TRYING TO COMMUNICATE DIRECTLY WITH OTHER MICROSERVICES OR THE MONGODB IN OR OUT OF THE DOCKER NETWORK AS CONNECTION CONFIGURATION OPTIONS WILL NO LONGER BE VALID. <b>DATASOURCE CONFIGURATIONS FOR LOOPBACK PROJECTS CAN CAN BE CHANGED IN SERVER/DATASOURCES.JSON</b>




<!-- NOT NEEDED - WILL WORK WITH AUTHENTICATION MODEL WHICH IS SAVED TO MONGODB
# Add the vet model

For authentication purposes, create the vet model, which should extend the Loopback Built-in User model.  The vet model on the vet microservice do not need to be duplicated in any way other than:
 - Name it the same name (vet) as model on vet service
 - Have it extend the LoopBack Built-in User model
 - Have it's datasource as the mongodb
 - Any custom functions from vet-service does not need to be duplicated here
 --->
