'use strict';

const fs = require("fs");
const parse = require("csv-parse/lib/sync");
const Question = require("./question");
const ss = require("./questionSpreadsheetConstants");

function print_usage() {
    const nodeName = process.argv[0];
    const scriptName = process.argv[1];
    const message = "USAGE:  " + nodeName + " " + scriptName + " <input CSV file> <output JSON file>";
    console.log(message);
}

if (process.argv.length != 4) {
    print_usage();
    process.exit();
}

const inputFile = process.argv[2];
const outputFile = process.argv[3];

if (fs.existsSync(inputFile) === false) {
    console.log("ERROR:  Unable to find file " + inputFile);
    print_usage();
    process.exit();
}

console.log(">> Reading file...")
const inputSource = fs.readFileSync(inputFile, 'utf-8');
const lines = parse(inputSource);

console.log(">> " + lines.length + " lines read.");

var questionPool = [];
var currentLine = 4; // We'll start on the fifth line to skip past the header

// Skip down until we find the first line with something in the third column, which indicates the start of the actual data
while (lines.length > currentLine) {
    if (lines[currentLine][ss.QUESTION_ID].trim().length > 0) {
        break;
    }
    currentLine++;
}

console.log(">> Found first question at line " + currentLine);

// The main loop.
while (lines.length > currentLine) {
    let line = lines[currentLine];

    // Assume we're on the first line of the question
    let scope = line[ss.SCOPE];
    let questionId = line[ss.QUESTION_ID];
    let questionText1 = line[ss.QUESTION_TEXT_1];
    let questionText2 = line[ss.QUESTION_TEXT_2];
    let questionText3 = line[ss.QUESTION_TEXT_3];
    let type = line[ss.TYPE];

    console.log(">> Line " + currentLine + ": Processing question " + questionId + " of type " + type + "...");

    let question = new Question(questionId, scope, type, questionText1, questionText2, questionText3);
    questionPool.push(question);

    // Add defaults
    let defaultMetricName = line[ss.DEFAULT_METRIC_NAME];
    let defaultMetricValue = line[ss.DEFAULT_METRIC_VALUE];
    console.log(">> Line " + currentLine + ":  Adding default with metric " + defaultMetricName + "....");
    question.addDefault(defaultMetricName, defaultMetricValue);

    // if MULTI_SELECT_OTHER, add followup
    if (type === 'MULTI_SELECT_OTHER') {
        let followupText = line[ss.FOLLOW_UP_QUESTION_TEXT];
        let followupMetric = line[ss.FOLLOW_UP_METRIC];
        console.log(">> Line " + currentLine + ":  Adding followup question with metric " + followupMetric + "....");
        question.addFollowup(followupText, followupMetric);
    }

    let firstLine = true;

    // Deal with each of the answer options, starting with the current line
    while (lines.length > currentLine) {
        line = lines[currentLine];

        if (line[ss.QUESTION_ID] && line[ss.QUESTION_ID].trim().length > 0) {
            if (firstLine === false) {
                break;
            }
        }

        let metricName = line[ss.ANSWER_METRIC_NAME];
        let answerValue = line[ss.ANSWER_VALUE];
        let lineAlreadyProcessed = false;

        if (firstLine) {
            // if NUMERIC, deal with the Range
            if (type === 'NUMERICAL') {
                console.log(">> Line " + currentLine + ":  Processing numerical range....");
                question.addRange(metricName, line[ss.RANGE_MINIMUM], line[ss.RANGE_MAXIMUM], line[ss.RANGE_INTERVAL]);
                lineAlreadyProcessed = true;
            }

            // Handle None of the Above
            if (type === 'MULTI_SELECT' || type === 'MULTI_SELECT_OTHER') {
                let noneOfTheAboveFlag = line[ss.NONE_OF_THE_ABOVE_FLAG];

                // Any answer is considered yes.  Leave blank for no.
                if (noneOfTheAboveFlag && noneOfTheAboveFlag.trim().length > 0) {
                    console.log(">> Line " + currentLine + ":  Adding 'None of the above' option with metric " + metricName + "....");
                    question.addNoneOfTheAbove(metricName, answerValue);
                    lineAlreadyProcessed = true;
                }
            }

            firstLine = false;
        }

        if (lineAlreadyProcessed === false) {
            console.log(">> Line " + currentLine + ":  Adding answer option " + metricName + "....");
            question.addOption(metricName, answerValue, line[ss.ANSWER_LABEL], line[ss.FOLLOW_UP_FLAG]);
        }

        currentLine++;
    }
}

var outputObj = { questions: questionPool };
var outputString = JSON.stringify(outputObj);
console.log(">> Writing out to " + outputFile);
fs.writeFileSync(outputFile, outputString, 'utf-8');
