'use strict';
// This node file is used to create the json files that will be inserted into the badgeBase64 mongo database table
// generateBase64Json.js <inputFile>   If the inputFile is not passed, then it will default to "inputFile.csv"
// The json will be placed in a directory called "output"

const fs = require("fs");
const mkdirp = require('mkdirp');

var base64 = require('file-base64');
const parse = require("csv-parse/lib/sync");

// -------------------------------------------------------------------
// checkOutputDirectory.
// Determines the "output" directory exists. If not, it will create it
// -------------------------------------------------------------------
function checkOutputDirectory(outputDirectoryName,cb){
  if (fs.existsSync("output") === false) {
    console.log("The output directory does not exist. creating it.");
    mkdirp('output', function (err) {
      if (err) {
        console.error("Error creating directory output", err);
	    cb(err,null);
      }
        else {
	    console.log("Successfully created output directory where the json files will be created");
	    cb(null,true);
	  }
    });
  }
  else {
    console.log("output directory exists");
    cb(null,true);
  }

}

// -------------------------------------------------------------------
// Process each line of the CSV File
// Each line is separated by a comma. And contains 4 fields of:
// badgeId, imageSize (small,large), state (awarded, unawarded), pngFile
// -------------------------------------------------------------------
function processInputCSVFile(inputFile,cb){
  const inputSource = fs.readFileSync(inputFile, 'utf-8');
  const lines = parse(inputSource);

  console.log(">> " + lines.length + " lines read.");

  var dataRecs = new Array();

  var currentLine = 1;
  var dataRec;

  while (lines.length > currentLine) {
    let line = lines[currentLine];

	  if (line.length < 4) {
	     console.log("line length is " + line.length +  " and does not contain the required 4 fields, and will not be used ", line);
	  }
	  else {
	    dataRec = new Object();

      dataRec.badgeId = line[0];
      dataRec.imageSize = line[1];
	    dataRec.state = line[2];
	    dataRec.pngFile = line[3];

	    dataRecs.push(dataRec);
 	  }

    currentLine++;
  }  // end while


  for (var i=0; i < dataRecs.length; i++){
	  //console.log(i+ ". Processing dataRec :" , dataRecs[i]);
	  processBadgeBase64Data(dataRecs[i], function(err,status) {
      if (err){
        console.log("error with processBadgeBase64Data",err);
        cb(err,null);
      }

    });
  }

  cb(null,true);
}


// -------------------------------------------------------------------
// processBadgeBase64Data
// Process the data to create the badgeBadgeBase64 json file that is used
// to populate the mongo badgeBadgeBase64 table
// -------------------------------------------------------------------
function processBadgeBase64Data(dataRecord,cb){
  var dbRecId = dataRecord.badgeId + "_" + dataRecord.state + "_" + dataRecord.imageSize + "_" + "XHDPI";
  dbRecId = dbRecId.toUpperCase();
  var outputFile = "output/badgeBase64_" + dbRecId + ".json";

  if (fs.existsSync(dataRecord.pngFile) === false) {
    console.log("ERROR:  Unable to find png file " + dataRecord.pngFile);
    cb(null,false);
  }
  else if (fs.existsSync("output") === false) {
	  console.log("The output directory must be created");
	  cb(null,false);
  }
  else {

	base64.encode(dataRecord.pngFile, function(err, base64String) {
	   if (err) {
	     console.log("error with base64,encode", err);
		   cb(err,null);
	   }
	   else {
       var outputObject = new Object();

       outputObject.badgeBase64Id = dbRecId;
       outputObject.badgeId = dataRecord.badgeId;
       outputObject.state = dataRecord.state;
       outputObject.size = dataRecord.imageSize;
       outputObject.dpi = "xhdpi",

       outputObject.base64  = base64String;

       var outputString =  JSON.stringify(outputObject);

       console.log(">> Created [" + outputFile + "] for dataRecord", dataRecord);
       fs.writeFileSync(outputFile, outputString, 'utf-8');
       cb(null,true);
	   }

    });
  }
}  // end. processData function


// *******************************************************************
// Main part of the application
// *******************************************************************
  console.log("*************GenerateBadgeBase64Json Application");
  var inputFile =  "inputFile.csv";
  var outputDirectory = "output";

  var argvLen = process.argv.length;
  
  if (argvLen == 4) {
	outputDirectory = process.argv[3];
	inputFile =  process.argv[2];
    console.log("Using the Input CVS File that is defined with argument 2:", inputFile);
	console.log("Using the Output Directory that is defined with argument 3:", outputDirectory);
  }
	  
  else if (argvLen == 3) {
    inputFile =  process.argv[2];
    console.log("Using the Input CVS File that is defined with argument 2:", inputFile);
	console.log("Defaulting Output Directory to:", outputDirectory);
  }
  else{
    console.log("Defaulting Input CSV File to: " + inputFile);
	console.log("Defaulting Output Directory to:", outputDirectory);
  }

  if (fs.existsSync(inputFile) === false) {
    console.log("ERROR:  Unable to find CSV Input File [" + inputFile + "]");
    process.exit();
  }

  console.log("checking if output directory exists");
  checkOutputDirectory(outputDirectory, function(err,status){
    if (err){
      console.log("error with checkOutputDirectory:", err);
	    process.exit();
    }
    else {
	    processInputCSVFile(inputFile, function(err,status){
	      if (err) {
	        console.log("error with processInputCSVFile",err);
		      process.exit();
	     }
	   });
   }
 });
